@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ class CControllerHostEdit extends CController {
 	protected $host;
 
 	protected function init() {
-		$this->disableSIDValidation();
+		$this->disableCsrfValidation();
 	}
 
 	protected function checkInput(): bool {
@@ -189,7 +189,7 @@ class CControllerHostEdit extends CController {
 			'is_psk_edit' => $this->hasInput('tls_psk_identity') && $this->hasInput('tls_psk'),
 			'show_inherited_macros' => $this->getInput('show_inherited_macros', 0),
 			'allowed_ui_conf_templates' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES),
-			'warning' => null,
+			'warnings' => [],
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
 			]
@@ -251,7 +251,20 @@ class CControllerHostEdit extends CController {
 		unset($macro);
 
 		// Reset Secret text macros and set warning for cloned host.
+		if ($this->hasInput('clone') || $this->hasInput('full_clone')) {
+			foreach ($data['host']['macros'] as &$macro) {
+				if (array_key_exists('allow_revert', $macro) && array_key_exists('value', $macro)) {
+					$macro['deny_revert'] = true;
+
+					unset($macro['allow_revert']);
+				}
+			}
+			unset($macro);
+		}
+
 		if ($data['host']['hostid'] === null) {
+			$secret_macro_reset = false;
+
 			foreach ($data['host']['macros'] as &$macro) {
 				if ($macro['type'] == ZBX_MACRO_TYPE_SECRET && !array_key_exists('value', $macro)) {
 					$macro = [
@@ -259,12 +272,26 @@ class CControllerHostEdit extends CController {
 						'value' => ''
 					] + $macro;
 
-					$data['warning'] = _('The cloned host contains user defined macros with type "Secret text". The value and type of these macros were reset.');
+					unset($macro['allow_revert']);
+
+					$secret_macro_reset = true;
 				}
 			}
 			unset($macro);
+
+			if ($secret_macro_reset) {
+				$data['warnings'][] = _('The cloned host contains user defined macros with type "Secret text". The value and type of these macros were reset.');
+			}
 		}
 
+		foreach ($data['host']['macros'] as &$macro) {
+			if ($macro['type'] == ZBX_MACRO_TYPE_SECRET
+					&& !array_key_exists('deny_revert', $macro) && !array_key_exists('value', $macro)) {
+				$macro['allow_revert'] = true;
+			}
+		}
+		unset($macro);
+
 		order_result($data['host']['valuemaps'], 'name');
 		$data['host']['valuemaps'] = array_values($data['host']['valuemaps']);
 
@@ -273,9 +300,13 @@ class CControllerHostEdit extends CController {
 		}
 
 		// Extend data for view.
-		$data['groups_ms'] = $this->hostGroupsForMultiselect($data['host']['groups']);
+		$data['groups_ms'] = $this->hostGroupsForMultiselect($data['host']['groups'], $clone_hostid !== null);
 		unset($data['groups']);
 
+		if ($clone_hostid !== null && count($data['host']['groups']) != count($data['groups_ms'])) {
+			$data['warnings'][] = _("The host being cloned belongs to a host group you don't have write permissions to. Non-writable group has been removed from the new host.");
+		}
+
 		CArrayHelper::sort($data['host']['parentTemplates'], ['name']);
 		$this->extendLinkedTemplates($data['editable_templates']);
 		$this->extendProxies($data['proxies']);
@@ -297,10 +328,11 @@ class CControllerHostEdit extends CController {
 	 * Function to prepare data for host group multiselect.
 	 *
 	 * @param array $groups
+	 * @param bool  $skip_non_editable  Whether to include non-editable host groups into response.
 	 *
 	 * @return array
 	 */
-	protected function hostGroupsForMultiselect(array $groups): array {
+	protected function hostGroupsForMultiselect(array $groups, $skip_non_editable = false): array {
 		$groupids = [];
 		foreach ($groups as $group) {
 			if (array_key_exists('new', $group)) {
@@ -339,12 +371,16 @@ class CControllerHostEdit extends CController {
 				];
 			}
 			elseif (array_key_exists($group['groupid'], $groups_all)) {
+				$is_editable = array_key_exists($group['groupid'], $groups_rw);
+
+				if (CWebUser::getType() != USER_TYPE_SUPER_ADMIN && $skip_non_editable && !$is_editable) {
+					continue;
+				}
+
 				$groups_ms[] = [
 					'id' => $group['groupid'],
 					'name' => $groups_all[$group['groupid']]['name'],
-					'disabled' => (CWebUser::getType() != USER_TYPE_SUPER_ADMIN)
-						&& !array_key_exists($group['groupid'], $groups_rw
-					)
+					'disabled' => CWebUser::getType() != USER_TYPE_SUPER_ADMIN && !$is_editable
 				];
 			}
 		}
