@@ -1,7 +1,7 @@
-<?php
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,9 +21,13 @@
 
 class CControllerMenuPopup extends CController {
 
+	protected function init() {
+		$this->disableCsrfValidation();
+	}
+
 	protected function checkInput() {
 		$fields = [
-			'type' => 'required|in history,host,item,item_configuration,item_prototype_configuration,map_element,trigger,trigger_macro',
+			'type' => 'required|in history,host,item,item_prototype,map_element,trigger,trigger_macro',
 			'data' => 'array'
 		];
 
@@ -54,14 +58,13 @@ class CControllerMenuPopup extends CController {
 				break;
 
 			case 'history':
-			case 'item':
 				$rules = [
 					'itemid' => 'required|db items.itemid'
 				];
 				break;
 
-			case 'item_configuration':
-			case 'item_prototype_configuration':
+			case 'item':
+			case 'item_prototype':
 				$rules = [
 					'itemid' => 'required|db items.itemid',
 					'backurl' => 'required|string'
@@ -81,8 +84,12 @@ class CControllerMenuPopup extends CController {
 			case 'trigger':
 				$rules = [
 					'triggerid' => 'required|db triggers.triggerid',
+					'backurl' => 'required|string',
 					'eventid' => 'db events.eventid',
-					'acknowledge' => 'in 0,1'
+					'show_update_problem' => 'in 0,1',
+					'show_rank_change_cause' => 'in 0,1',
+					'show_rank_change_symptom' => 'in 0,1',
+					'ids' => 'array_db events.eventid'
 				];
 				break;
 
@@ -177,53 +184,46 @@ class CControllerMenuPopup extends CController {
 				]);
 			}
 
-			$scripts = CWebUser::checkAccess(CRoleHelper::ACTIONS_EXECUTE_SCRIPTS)
+			$all_scripts = CWebUser::checkAccess(CRoleHelper::ACTIONS_EXECUTE_SCRIPTS)
 				? API::Script()->getScriptsByHosts([$data['hostid']])[$data['hostid']]
 				: [];
 
-			// Filter only host scope scripts, get rid of excess spaces and unify slashes in menu path.
-			if ($scripts) {
-				foreach ($scripts as $num => &$script) {
+			$scripts = [];
+			$urls = [];
+
+			if (array_key_exists('urls', $data)) {
+				foreach ($data['urls'] as &$url) {
+					$url['new_window'] = ZBX_SCRIPT_URL_NEW_WINDOW_YES;
+					$url['confirmation'] = '';
+					$url['menu_path'] = '';
+					$url['name'] = $url['label'];
+
+					unset($url['label']);
+				}
+				unset($url);
+
+				$urls = $data['urls'];
+			}
+
+			if ($all_scripts) {
+				foreach ($all_scripts as $num => $script) {
+					// Filter only host scope scripts, get rid of excess spaces and unify slashes in menu path.
 					if ($script['scope'] != ZBX_SCRIPT_SCOPE_HOST) {
-						unset($scripts[$num]);
+						unset($all_scripts[$num]);
 						continue;
 					}
 
-					$script['menu_path'] = trimPath($script['menu_path']);
-					$script['sort'] = '';
-
-					if (strlen($script['menu_path']) > 0) {
-						// First or only slash from beginning is trimmed.
-						if (substr($script['menu_path'], 0, 1) === '/') {
-							$script['menu_path'] = substr($script['menu_path'], 1);
-						}
-
-						$script['sort'] = $script['menu_path'];
-
-						// If there is something more, check if last slash is present.
-						if (strlen($script['menu_path']) > 0) {
-							if (substr($script['menu_path'], -1) !== '/'
-									&& substr($script['menu_path'], -2) === '\\/') {
-								$script['sort'] = $script['menu_path'].'/';
-							}
-							else {
-								$script['sort'] = $script['menu_path'];
-							}
-
-							if (substr($script['menu_path'], -1) === '/'
-									&& substr($script['menu_path'], -2) !== '\\/') {
-								$script['menu_path'] = substr($script['menu_path'], 0, -1);
-							}
-						}
+					// Split scripts and URLs.
+					if ($script['type'] == ZBX_SCRIPT_TYPE_URL) {
+						$urls[] = $script;
+					}
+					else {
+						$scripts[] = $script;
 					}
-
-					$script['sort'] = $script['sort'].$script['name'];
 				}
-				unset($script);
-			}
 
-			if ($scripts) {
-				CArrayHelper::sort($scripts, ['sort']);
+				$scripts = self::sortEntitiesByMenuPath($scripts);
+				$urls = self::sortEntitiesByMenuPath($urls);
 			}
 
 			$menu_data = [
@@ -234,7 +234,8 @@ class CControllerMenuPopup extends CController {
 				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
 				'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
 				'allowed_ui_hosts' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_HOSTS),
-				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
+				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS),
+				'csrf_token' => CCsrfTokenHelper::get('scriptexec')
 			];
 
 			if ($has_goto) {
@@ -263,8 +264,27 @@ class CControllerMenuPopup extends CController {
 				];
 			}
 
-			if (array_key_exists('urls', $data)) {
-				$menu_data['urls'] = $data['urls'];
+			foreach (array_values($urls) as $url) {
+				$menu_data['urls'][] = [
+					'label' => $url['name'],
+					'menu_path' => $url['menu_path'],
+					'url' => $url['url'],
+					'target' => $url['new_window'] == ZBX_SCRIPT_URL_NEW_WINDOW_YES ? '_blank' : '',
+					'confirmation' => $url['confirmation'],
+					'rel' => 'noopener'.(ZBX_NOREFERER ? ' noreferrer' : '')
+				];
+			}
+
+			if (array_key_exists('urls', $menu_data)) {
+				foreach ($menu_data['urls'] as &$url) {
+					if (!CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false])) {
+						$url['url'] = 'javascript: alert('.
+							json_encode(_s('Provided URL "%1$s" is invalid.', $url['url'])).
+						');';
+						unset($url['target'], $url['rel']);
+					}
+				}
+				unset($url);
 			}
 
 			if (array_key_exists('tags', $data)) {
@@ -290,7 +310,9 @@ class CControllerMenuPopup extends CController {
 	 */
 	private static function getMenuDataItem(array $data) {
 		$db_items = API::Item()->get([
-			'output' => ['hostid', 'type', 'value_type', 'history', 'trends'],
+			'output' => ['hostid', 'key_', 'name', 'flags', 'type', 'value_type', 'history', 'trends'],
+			'selectHosts' => ['host'],
+			'selectTriggers' => ['triggerid', 'description'],
 			'itemids' => $data['itemid'],
 			'webitems' => true
 		]);
@@ -306,7 +328,7 @@ class CControllerMenuPopup extends CController {
 				}
 				elseif (CWebUser::getType() == USER_TYPE_ZABBIX_ADMIN) {
 					$is_writable = (bool) API::Host()->get([
-						'output' => [],
+						'output' => ['hostid'],
 						'hostids' => $db_item['hostid'],
 						'editable' => true
 					]);
@@ -319,15 +341,22 @@ class CControllerMenuPopup extends CController {
 
 			return [
 				'type' => 'item',
+				'backurl' => $data['backurl'],
 				'itemid' => $data['itemid'],
+				'name' => $db_item['name'],
+				'key' => $db_item['key_'],
 				'hostid' => $db_item['hostid'],
+				'host' => $db_item['hosts'][0]['host'],
+				'triggers' => $db_item['triggers'],
 				'showGraph' => ($db_item['value_type'] == ITEM_VALUE_TYPE_FLOAT
 					|| $db_item['value_type'] == ITEM_VALUE_TYPE_UINT64
 				),
 				'history' => $db_item['history'] != 0,
 				'trends' => $db_item['trends'] != 0,
-				'isWriteable' => $is_writable,
+				'isDiscovery' => $db_item['flags'] == ZBX_FLAG_DISCOVERY_CREATED,
 				'isExecutable' => $is_executable,
+				'isWriteable' => $is_writable,
+				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
 				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
 			];
 		}
@@ -337,59 +366,6 @@ class CControllerMenuPopup extends CController {
 		return null;
 	}
 
-	/**
-	 * Prepare data for item configuration context menu popup.
-	 *
-	 * @param array  $data
-	 * @param string $data['itemid']
-	 * @param string $data['backurl']
-	 *
-	 * @return mixed
-	 */
-	private static function getMenuDataItemConfiguration(array $data) {
-		$db_items = API::Item()->get([
-			'output' => ['hostid', 'key_', 'name', 'flags'],
-			'selectHosts' => ['host'],
-			'itemids' => $data['itemid'],
-			'webitems' => true
-		]);
-
-		if ($db_items) {
-			$db_item = $db_items[0];
-			$menu_data = [
-				'type' => 'item_configuration',
-				'backurl' => $data['backurl'],
-				'itemid' => $data['itemid'],
-				'hostid' => $db_item['hostid'],
-				'host' => $db_item['hosts'][0]['host'],
-				'name' => $db_item['name'],
-				'key' => $db_item['key_'],
-				'create_dependent_item' => ($db_item['flags'] != ZBX_FLAG_DISCOVERY_CREATED),
-				'create_dependent_discovery' => ($db_item['flags'] != ZBX_FLAG_DISCOVERY_CREATED),
-				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
-				'triggers' => []
-			];
-
-			$db_triggers = API::Trigger()->get([
-				'output' => ['triggerid', 'description'],
-				'itemids' => $data['itemid']
-			]);
-
-			foreach ($db_triggers as $db_trigger) {
-				$menu_data['triggers'][] = [
-					'triggerid' => $db_trigger['triggerid'],
-					'name' => $db_trigger['description']
-				];
-			}
-
-			return $menu_data;
-		}
-
-		error(_('No permissions to referred object or it does not exist!'));
-
-		return null;
-	}
-
 	/**
 	 * Prepare data for item prototype configuration context menu popup.
 	 *
@@ -399,11 +375,12 @@ class CControllerMenuPopup extends CController {
 	 *
 	 * @return mixed
 	 */
-	private static function getMenuDataItemPrototypeConfiguration(array $data) {
+	private static function getMenuDataItemPrototype(array $data) {
 		$db_item_prototypes = API::ItemPrototype()->get([
 			'output' => ['name', 'key_'],
 			'selectDiscoveryRule' => ['itemid'],
 			'selectHosts' => ['host'],
+			'selectTriggers' => ['triggerid', 'description'],
 			'itemids' => $data['itemid']
 		]);
 
@@ -411,27 +388,16 @@ class CControllerMenuPopup extends CController {
 			$db_item_prototype = $db_item_prototypes[0];
 
 			$menu_data = [
-				'type' => 'item_prototype_configuration',
+				'type' => 'item_prototype',
 				'backurl' => $data['backurl'],
 				'itemid' => $data['itemid'],
 				'name' => $db_item_prototype['name'],
 				'key' => $db_item_prototype['key_'],
 				'host' => $db_item_prototype['hosts'][0]['host'],
 				'parent_discoveryid' => $db_item_prototype['discoveryRule']['itemid'],
-				'trigger_prototypes' => []
+				'trigger_prototypes' => $db_item_prototype['triggers']
 			];
 
-			$db_trigger_prototypes = API::TriggerPrototype()->get([
-				'output' => ['triggerid', 'description'],
-				'itemids' => $data['itemid']
-			]);
-
-			foreach ($db_trigger_prototypes as $db_trigger_prototype) {
-				$menu_data['trigger_prototypes'][] = [
-					'triggerid' => $db_trigger_prototype['triggerid'],
-					'name' => $db_trigger_prototype['description']
-				];
-			}
 			return $menu_data;
 		}
 
@@ -440,6 +406,25 @@ class CControllerMenuPopup extends CController {
 		return null;
 	}
 
+	/**
+	 * Validates URLs for supported schemes.
+	 *
+	 * @param array  $urls
+	 * @param array  $urls[]['url']
+	 *
+	 * @return array
+	 */
+	private static function sanitizeMapElementUrls(array $urls): array {
+		foreach ($urls as &$url) {
+			if (CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false]) === false) {
+				$url['url'] = 'javascript: alert('.json_encode(_s('Provided URL "%1$s" is invalid.', $url['url'])).');';
+			}
+		}
+		unset($url);
+
+		return $urls;
+	}
+
 	/**
 	 * Combines map URLs with element's URLs and performs other modifications with the URLs.
 	 *
@@ -470,10 +455,6 @@ class CControllerMenuPopup extends CController {
 			if ($url['name'] === '' || $url['url'] === '') {
 				unset($selement['urls'][$url_nr]);
 			}
-			elseif (CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false]) === false) {
-				$selement['urls'][$url_nr]['url'] = 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.',
-					zbx_jsvalue($url['url'], false, false)).'\');';
-			}
 		}
 
 		CArrayHelper::sort($selement['urls'], ['name']);
@@ -544,7 +525,7 @@ class CControllerMenuPopup extends CController {
 						}
 
 						if ($selement['urls']) {
-							$menu_data['urls'] = $selement['urls'];
+							$menu_data['urls'] = self::sanitizeMapElementUrls($selement['urls']);
 						}
 						return $menu_data;
 
@@ -564,7 +545,7 @@ class CControllerMenuPopup extends CController {
 							$menu_data['show_suppressed'] = true;
 						}
 						if ($selement['urls']) {
-							$menu_data['urls'] = $selement['urls'];
+							$menu_data['urls'] = self::sanitizeMapElementUrls($selement['urls']);
 						}
 						if ($selement['tags']) {
 							$menu_data['evaltype'] = $selement['evaltype'];
@@ -592,11 +573,77 @@ class CControllerMenuPopup extends CController {
 						return self::getMenuDataHost($host_data);
 
 					case SYSMAP_ELEMENT_TYPE_TRIGGER:
+						$items = [];
+						$triggers = [];
+						$hosts = [];
+						$show_events = true;
+						$unique_itemids = [];
+
+						$db_triggers = API::Trigger()->get([
+							'output' => ['triggerid', 'description'],
+							'selectHosts' => ['hostid', 'name', 'status'],
+							'selectItems' => ['itemid', 'hostid', 'name', 'value_type', 'type'],
+							'triggerids' => array_column($selement['elements'], 'triggerid'),
+							'preservekeys' => true
+						]);
+
+						foreach ($db_triggers as $db_trigger) {
+							foreach ($db_trigger['hosts'] as $host) {
+								$hosts[$host['hostid']] = $host['name'];
+							}
+
+							foreach ($db_trigger['items'] as &$item) {
+								$item['hostname'] = $hosts[$item['hostid']];
+
+								if ($host['status'] != HOST_STATUS_MONITORED) {
+									$show_events = false;
+								}
+							}
+							unset($item);
+
+							CArrayHelper::sort($db_trigger['items'], ['name', 'hostname', 'itemid']);
+
+							$with_hostname = count($hosts) > 1;
+
+							foreach ($db_trigger['items'] as $item) {
+								if (in_array($item['itemid'], $unique_itemids)) {
+									continue;
+								}
+
+								$items[] = [
+									'name' => $with_hostname
+										? $item['hostname'].NAME_DELIMITER.$item['name']
+										: $item['name'],
+									'params' => [
+										'itemid' => $item['itemid'],
+										'action' => in_array(
+											$item['value_type'], [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]
+										)
+											? HISTORY_GRAPH
+											: HISTORY_VALUES,
+										'is_webitem' => $item['type'] == ITEM_TYPE_HTTPTEST
+									]
+								];
+
+								$unique_itemids[] = $item['itemid'];
+							}
+
+							$triggers[] = [
+								'triggerid' => $db_trigger['triggerid'],
+								'description' => $db_trigger['description']
+							];
+						}
+
 						$menu_data = [
 							'type' => 'map_element_trigger',
-							'triggerids' => zbx_objectValues($selement['elements'], 'triggerid'),
-							'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS)
+							'triggers' => $triggers,
+							'items' => $items,
+							'show_events' => $show_events,
+							'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+							'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS),
+							'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA)
 						];
+
 						if (array_key_exists('severity_min', $data)) {
 							$menu_data['severities'] = array_column(
 								CSeverityHelper::getSeverities((int) $data['severity_min']),
@@ -606,9 +653,11 @@ class CControllerMenuPopup extends CController {
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
 						}
+
 						if ($selement['urls']) {
-							$menu_data['urls'] = $selement['urls'];
+							$menu_data['urls'] = self::sanitizeMapElementUrls($selement['urls']);
 						}
+
 						return $menu_data;
 
 					case SYSMAP_ELEMENT_TYPE_IMAGE:
@@ -616,7 +665,7 @@ class CControllerMenuPopup extends CController {
 							'type' => 'map_element_image'
 						];
 						if ($selement['urls']) {
-							$menu_data['urls'] = $selement['urls'];
+							$menu_data['urls'] = self::sanitizeMapElementUrls($selement['urls']);
 						}
 						return $menu_data;
 				}
@@ -632,17 +681,23 @@ class CControllerMenuPopup extends CController {
 	 * Prepare data for trigger context menu popup.
 	 *
 	 * @param array  $data
-	 * @param string $data['triggerid']
-	 * @param string $data['eventid']      (optional) Mandatory for Acknowledge menu.
-	 * @param bool   $data['acknowledge']  (optional) Whether to show Acknowledge menu.
+	 *        string $data['triggerid']                 Trigger ID.
+	 *        string $data['backurl']                   URL from where the menu popup was called.
+	 *        string $data['eventid']                   (optional) Mandatory for "Update problem", "Mark as cause"
+	 *                                                  and "Mark selected as symptoms" context menus.
+	 *        array  $data['ids']                       (optional) Event IDs that are used in event rank change to
+	 *                                                  symptom.
+	 *        bool   $data['show_update_problem']       (optional) Whether to show "Update problem".
+	 *        bool   $data['show_rank_change_cause']    (optional) Whether to show "Mark as cause".
+	 *        bool   $data['show_rank_change_symptom']  (optional) Whether to show "Mark selected as symptoms".
 	 *
-	 * @return mixed
+	 * @return array|null
 	 */
-	private static function getMenuDataTrigger(array $data) {
+	private static function getMenuDataTrigger(array $data): ?array {
 		$db_triggers = API::Trigger()->get([
-			'output' => ['expression', 'url', 'comments', 'manual_close'],
+			'output' => ['expression', 'url_name', 'url', 'comments', 'manual_close'],
 			'selectHosts' => ['hostid', 'name', 'status'],
-			'selectItems' => ['itemid', 'hostid', 'name', 'value_type'],
+			'selectItems' => ['itemid', 'hostid', 'name', 'value_type', 'type'],
 			'triggerids' => $data['triggerid'],
 			'preservekeys' => true
 		]);
@@ -656,6 +711,12 @@ class CControllerMenuPopup extends CController {
 
 			$db_trigger['url'] = CMacrosResolverHelper::resolveTriggerUrl($db_trigger, $url) ? $url : '';
 
+			if ($db_trigger['url'] !== '') {
+				$db_trigger['url_name'] = CMacrosResolverHelper::resolveTriggerUrlName($db_trigger, $url_name)
+					? $url_name
+					: '';
+			}
+
 			$hosts = [];
 			$show_events = true;
 
@@ -666,7 +727,6 @@ class CControllerMenuPopup extends CController {
 					$show_events = false;
 				}
 			}
-			unset($db_trigger['hosts']);
 
 			foreach ($db_trigger['items'] as &$item) {
 				$item['hostname'] = $hosts[$item['hostid']];
@@ -687,7 +747,8 @@ class CControllerMenuPopup extends CController {
 						'itemid' => $item['itemid'],
 						'action' => in_array($item['value_type'], [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64])
 							? HISTORY_GRAPH
-							: HISTORY_VALUES
+							: HISTORY_VALUES,
+						'is_webitem' => $item['type'] == ITEM_TYPE_HTTPTEST
 					]
 				];
 			}
@@ -695,22 +756,18 @@ class CControllerMenuPopup extends CController {
 			$menu_data = [
 				'type' => 'trigger',
 				'triggerid' => $data['triggerid'],
+				'backurl' => $data['backurl'],
 				'items' => $items,
-				'showEvents' => $show_events,
+				'show_events' => $show_events,
 				'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
 				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS),
-				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA)
+				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
+				'allowed_actions_change_problem_ranking' =>
+					CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_PROBLEM_RANKING)
 			];
 
-			if ($db_trigger['url'] !== '') {
-				$menu_data['urls'][] = [
-					'label' => _('Trigger URL'),
-					'url' => $db_trigger['url']
-				];
-			}
-
 			$can_be_closed = ($db_trigger['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED
-					&& CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+				&& CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
 			);
 			$event = [];
 
@@ -718,7 +775,7 @@ class CControllerMenuPopup extends CController {
 				$menu_data['eventid'] = $data['eventid'];
 
 				$events = API::Event()->get([
-					'output' => ['r_eventid', 'urls'],
+					'output' => ['eventid', 'r_eventid', 'urls', 'cause_eventid'],
 					'select_acknowledges' => ['action'],
 					'eventids' => $data['eventid']
 				]);
@@ -730,88 +787,95 @@ class CControllerMenuPopup extends CController {
 						$can_be_closed = !isEventClosed($event);
 					}
 
-					foreach ($event['urls'] as $url) {
-						$menu_data['urls'][] = [
-							'label' => $url['name'],
-							'url' => $url['url'],
-							'target' => '_blank',
-							'rel' => 'noopener'.(ZBX_NOREFERER ? ' noreferrer' : '')
-						];
+					if (CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_PROBLEM_RANKING)) {
+						// Can change rank to cause if event is not already cause.
+						$menu_data['mark_as_cause'] = ($event['cause_eventid'] != 0);
+
+						// Check if selected events can change rank to symptom for given cause.
+						$menu_data['mark_selected_as_symptoms'] = array_key_exists('ids', $data) && $data['ids']
+							? (bool) validateEventRankChangeToSymptom($data['ids'], $data['eventid'])
+							: false;
+
+						$menu_data['eventids'] = array_key_exists('ids', $data) ? $data['ids'] : [];
+
+						// Show individual menus depending on location.
+						$menu_data['show_rank_change_cause'] = array_key_exists('show_rank_change_cause', $data)
+							&& $data['show_rank_change_cause'];
+						$menu_data['show_rank_change_symptom'] = array_key_exists('show_rank_change_symptom', $data)
+							&& $data['show_rank_change_symptom'];
+						$menu_data['csrf_tokens']['acknowledge'] = CCsrfTokenHelper::get('acknowledge');
 					}
 				}
 			}
 
-			if (array_key_exists('urls', $menu_data)) {
-				foreach ($menu_data['urls'] as &$url) {
-					if (!CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false])) {
-						$url['url'] = 'javascript: alert(\''.
-							_s('Provided URL "%1$s" is invalid.', zbx_jsvalue($url['url'], false, false)).
-						'\');';
-						unset($url['target'], $url['rel']);
-					}
-				}
-				unset($url);
+			if (array_key_exists('show_update_problem', $data)) {
+				$menu_data['show_update_problem'] = $data['show_update_problem']
+					&& (CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS)
+						|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
+						|| CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
+						|| $can_be_closed
+						|| CWebUser::checkAccess(CRoleHelper::ACTIONS_SUPPRESS_PROBLEMS)
+					);
 			}
 
-			if (array_key_exists('acknowledge', $data)) {
-				$menu_data['acknowledge'] = ((bool) $data['acknowledge']
-						&& (CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS)
-							|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
-							|| CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
-							|| $can_be_closed
-							|| CWebUser::checkAccess(CRoleHelper::ACTIONS_SUPPRESS_PROBLEMS)
-						)
-				);
-			}
+			$scripts_by_events = [];
 
-			$scripts_by_hosts = CWebUser::checkAccess(CRoleHelper::ACTIONS_EXECUTE_SCRIPTS)
-				? $event ? API::Script()->getScriptsByHosts(array_keys($hosts)) : []
-				: [];
+			if (CWebUser::checkAccess(CRoleHelper::ACTIONS_EXECUTE_SCRIPTS) && $event) {
+				$scripts_by_events = API::Script()->getScriptsByEvents([$event['eventid']]);
+			}
 
 			// Filter only event scope scripts and get rid of excess spaces and create full name with menu path included.
 			$scripts = [];
-			foreach ($scripts_by_hosts as &$host_scripts) {
-				foreach ($host_scripts as &$host_script) {
-					if (!array_key_exists($host_script['scriptid'], $scripts)
-							&& $host_script['scope'] == ZBX_SCRIPT_SCOPE_EVENT) {
-						$host_script['menu_path'] = trimPath($host_script['menu_path']);
-						$host_script['sort'] = '';
+			$urls = [];
 
-						if (strlen($host_script['menu_path']) > 0) {
-							// First or only slash from beginning is trimmed.
-							if (substr($host_script['menu_path'], 0, 1) === '/') {
-								$host_script['menu_path'] = substr($host_script['menu_path'], 1);
-							}
+			foreach ($scripts_by_events as &$event_scripts) {
+				foreach ($event_scripts as $num => &$event_script) {
+					if ($event_script['scope'] != ZBX_SCRIPT_SCOPE_EVENT) {
+						unset($event_script[$num]);
+						continue;
+					}
 
-							$host_script['sort'] = $host_script['menu_path'];
+					$scriptid = $event_script['scriptid'];
 
-							// If there is something more, check if last slash is present.
-							if (strlen($host_script['menu_path']) > 0) {
-								if (substr($host_script['menu_path'], -1) !== '/'
-										&& substr($host_script['menu_path'], -2) === '\\/') {
-									$host_script['sort'] = $host_script['menu_path'].'/';
-								}
-								else {
-									$host_script['sort'] = $host_script['menu_path'];
-								}
-
-								if (substr($host_script['menu_path'], -1) === '/'
-										&& substr($host_script['menu_path'], -2) !== '\\/') {
-									$host_script['menu_path'] = substr($host_script['menu_path'], 0, -1);
-								}
-							}
+					// Split scripts and URLs.
+					if ($event_script['type'] == ZBX_SCRIPT_TYPE_URL) {
+						if (!array_key_exists($scriptid, $urls)) {
+							$urls[$scriptid] = $event_script;
+						}
+					}
+					else {
+						if (!array_key_exists($scriptid, $scripts)) {
+							$scripts[$scriptid] = $event_script;
 						}
-
-						$host_script['sort'] = $host_script['sort'].$host_script['name'];
-
-						$scripts[$host_script['scriptid']] = $host_script;
 					}
 				}
-				unset($host_script);
+				unset($event_script);
 			}
-			unset($host_scripts);
+			unset($event_scripts);
 
-			CArrayHelper::sort($scripts, ['sort']);
+			if ($event) {
+				foreach ($event['urls'] as &$url) {
+					$url['new_window'] = ZBX_SCRIPT_URL_NEW_WINDOW_YES;
+					$url['confirmation'] = '';
+					$url['menu_path'] = '';
+				}
+				unset($url);
+
+				$urls = array_merge($urls, $event['urls']);
+			}
+
+			if ($db_trigger['url'] !== '') {
+				$urls = array_merge($urls, [[
+					'name' => $db_trigger['url_name'] !== '' ? $db_trigger['url_name'] : _('Trigger URL'),
+					'url' => $db_trigger['url'],
+					'menu_path' => '',
+					'new_window' => ZBX_SCRIPT_URL_NEW_WINDOW_NO,
+					'confirmation' => ''
+				]]);
+			}
+
+			$scripts = self::sortEntitiesByMenuPath($scripts);
+			$urls = self::sortEntitiesByMenuPath($urls);
 
 			foreach (array_values($scripts) as $script) {
 				$menu_data['scripts'][] = [
@@ -822,6 +886,33 @@ class CControllerMenuPopup extends CController {
 				];
 			}
 
+			if ($scripts) {
+				$menu_data['csrf_tokens']['scriptexec'] = CCsrfTokenHelper::get('scriptexec');
+			}
+
+			foreach (array_values($urls) as $url) {
+				$menu_data['urls'][] = [
+					'label' => $url['name'],
+					'menu_path' => $url['menu_path'],
+					'url' => $url['url'],
+					'target' => $url['new_window'] == ZBX_SCRIPT_URL_NEW_WINDOW_YES ? '_blank' : '',
+					'confirmation' => $url['confirmation'],
+					'rel' => 'noopener'.(ZBX_NOREFERER ? ' noreferrer' : '')
+				];
+			}
+
+			if (array_key_exists('urls', $menu_data)) {
+				foreach ($menu_data['urls'] as &$url) {
+					if (!CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false])) {
+						$url['url'] = 'javascript: alert('.
+							json_encode(_s('Provided URL "%1$s" is invalid.', $url['url'])).
+						');';
+						unset($url['target'], $url['rel']);
+					}
+				}
+				unset($url);
+			}
+
 			return $menu_data;
 		}
 
@@ -830,6 +921,56 @@ class CControllerMenuPopup extends CController {
 		return null;
 	}
 
+	/**
+	 * Process menu path and sort scripts or URLs according to it.
+	 *
+	 * @param array  $entities                 Scripts and URLs.
+	 * @param string $entities[]['name']       Name of the ccript or URL.
+	 * @param string $entities[]['menu_path']  Menu path of the ccript or URL.
+	 *
+	 * @return array
+	 */
+	private static function sortEntitiesByMenuPath(array $entities): array {
+		if ($entities) {
+			foreach ($entities as &$entity) {
+				$entity['menu_path'] = trimPath($entity['menu_path']);
+				$entity['sort'] = '';
+
+				if (strlen($entity['menu_path']) > 0) {
+					// First or only slash from beginning is trimmed.
+					if (substr($entity['menu_path'], 0, 1) === '/') {
+						$entity['menu_path'] = substr($entity['menu_path'], 1);
+					}
+
+					$entity['sort'] = $entity['menu_path'];
+
+					// If there is something more, check if last slash is present.
+					if (strlen($entity['menu_path']) > 0) {
+						if (substr($entity['menu_path'], -1) !== '/'
+								&& substr($entity['menu_path'], -2) === '\\/') {
+							$entity['sort'] = $entity['menu_path'].'/';
+						}
+						else {
+							$entity['sort'] = $entity['menu_path'];
+						}
+
+						if (substr($entity['menu_path'], -1) === '/'
+								&& substr($entity['menu_path'], -2) !== '\\/') {
+							$entity['menu_path'] = substr($entity['menu_path'], 0, -1);
+						}
+					}
+				}
+
+				$entity['sort'] = $entity['sort'].$entity['name'];
+			}
+			unset($entity);
+
+			CArrayHelper::sort($entities, ['sort']);
+		}
+
+		return $entities;
+	}
+
 	/**
 	 * Prepare data for trigger macro context menu popup.
 	 *
@@ -855,12 +996,8 @@ class CControllerMenuPopup extends CController {
 				$menu_data = self::getMenuDataItem($data);
 				break;
 
-			case 'item_configuration':
-				$menu_data = self::getMenuDataItemConfiguration($data);
-				break;
-
-			case 'item_prototype_configuration':
-				$menu_data = self::getMenuDataItemPrototypeConfiguration($data);
+			case 'item_prototype':
+				$menu_data = self::getMenuDataItemPrototype($data);
 				break;
 
 			case 'map_element':
