@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ require_once dirname(__FILE__).'/../../include/items.inc.php';
 require_once dirname(__FILE__).'/../../include/users.inc.php';
 require_once dirname(__FILE__).'/../../include/js.inc.php';
 require_once dirname(__FILE__).'/../../include/discovery.inc.php';
+
 class CControllerPopupGeneric extends CController {
 
 	/**
@@ -32,25 +33,37 @@ class CControllerPopupGeneric extends CController {
 	 *
 	 * @var array
 	 */
-	const ALLOWED_ITEM_TYPES = [ITEM_TYPE_ZABBIX, ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL,
-		ITEM_TYPE_IPMI, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_JMX
+	const ALLOWED_ITEM_TYPES = [ITEM_TYPE_ZABBIX, ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_SIMPLE, ITEM_TYPE_SNMPTRAP,
+		ITEM_TYPE_INTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_IPMI, ITEM_TYPE_JMX
 	];
 
 	/**
-	 * Popups having group filter selector.
+	 * Popups having host group filter selector.
 	 *
 	 * @array
 	 */
-	const POPUPS_HAVING_GROUP_FILTER = ['hosts', 'templates', 'host_templates'];
+	const POPUPS_HAVING_HOST_GROUP_FILTER = ['hosts', 'host_templates'];
+
+	/**
+	 * Popups having template group filter selector.
+	 *
+	 * @array
+	 */
+	const POPUPS_HAVING_TEMPLATE_GROUP_FILTER = ['templates'];
 
 	/**
 	 * Popups having host filter selector.
 	 *
 	 * @array
 	 */
-	const POPUPS_HAVING_HOST_FILTER = ['triggers', 'items', 'graphs', 'graph_prototypes',
-		'item_prototypes'
-	];
+	const POPUPS_HAVING_HOST_FILTER = ['items', 'item_prototypes', 'triggers', 'graphs', 'graph_prototypes'];
+
+	/**
+	 * Popups having template filter selector.
+	 *
+	 * @array
+	 */
+	const POPUPS_HAVING_TEMPLATE_FILTER = ['template_items', 'template_triggers'];
 
 	/**
 	 * General properties for supported dialog types.
@@ -67,12 +80,19 @@ class CControllerPopupGeneric extends CController {
 	private $source_table;
 
 	/**
-	 * Groups set in filter.
+	 * Host groups set in filter.
 	 *
 	 * @var array
 	 */
 	protected $groupids = [];
 
+	/**
+	 * Template groups set in filter.
+	 *
+	 * @var array
+	 */
+	protected $template_groupids = [];
+
 	/**
 	 * Hosts set in filter.
 	 *
@@ -80,6 +100,13 @@ class CControllerPopupGeneric extends CController {
 	 */
 	protected $hostids = [];
 
+	/**
+	 * Template set in filter.
+	 *
+	 * @var array
+	 */
+	protected $templateids = [];
+
 	/**
 	 * Either Host filter need to be filled to load results.
 	 *
@@ -87,6 +114,13 @@ class CControllerPopupGeneric extends CController {
 	 */
 	protected $host_preselect_required;
 
+	/**
+	 * Either template filter need to be filled to load results.
+	 *
+	 * @var bool
+	 */
+	protected $template_preselect_required;
+
 	/**
 	 * Either Host group filter need to be filled to load results.
 	 *
@@ -94,6 +128,13 @@ class CControllerPopupGeneric extends CController {
 	 */
 	protected $group_preselect_required;
 
+	/**
+	 * Either template group filter need to be filled to load results.
+	 *
+	 * @var bool
+	 */
+	protected $template_group_preselect_required;
+
 	/**
 	 * Set of disabled options.
 	 *
@@ -101,8 +142,13 @@ class CControllerPopupGeneric extends CController {
 	 */
 	protected $disableids = [];
 
+	/**
+	 * @var array
+	 */
+	private $page_options = [];
+
 	protected function init() {
-		$this->disableSIDvalidation();
+		$this->disableCsrfValidation();
 
 		$this->popup_properties = [
 			'hosts' => [
@@ -153,6 +199,18 @@ class CControllerPopupGeneric extends CController {
 					_('Name')
 				]
 			],
+			'template_groups' => [
+				'title' => _('Template groups'),
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
+				'allowed_src_fields' => 'groupid,name',
+				'form' => [
+					'name' => 'templateGroupsform',
+					'id' => 'templateGroups'
+				],
+				'table_columns' => [
+					_('Name')
+				]
+			],
 			'proxies' => [
 				'title' => _('Proxies'),
 				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
@@ -179,6 +237,20 @@ class CControllerPopupGeneric extends CController {
 					_('Status')
 				]
 			],
+			'template_triggers' => [
+				'title' => _('Triggers'),
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
+				'allowed_src_fields' => 'description,triggerid,expression',
+				'form' => [
+					'name' => 'triggerform',
+					'id' => 'triggers'
+				],
+				'table_columns' => [
+					_('Name'),
+					_('Severity'),
+					_('Status')
+				]
+			],
 			'trigger_prototypes' => [
 				'title' => _('Trigger prototypes'),
 				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
@@ -235,6 +307,22 @@ class CControllerPopupGeneric extends CController {
 					_('Status')
 				]
 			],
+			'template_items' => [
+				'title' => _('Items'),
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
+				'allowed_src_fields' => 'itemid,name',
+				'form' => [
+					'name' => 'itemform',
+					'id' => 'items'
+				],
+				'table_columns' => [
+					_('Name'),
+					_('Key'),
+					_('Type'),
+					_('Type of information'),
+					_('Status')
+				]
+			],
 			'help_items' => [
 				'title' => _('Standard items'),
 				'min_user_type' => USER_TYPE_ZABBIX_USER,
@@ -366,6 +454,19 @@ class CControllerPopupGeneric extends CController {
 					_('Mapping')
 				]
 			],
+			'template_valuemaps' => [
+				'title' => _('Value mapping'),
+				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
+				'allowed_src_fields' => 'valuemapid,name',
+				'form' => [
+					'name' => 'valuemapform',
+					'id' => 'valuemaps'
+				],
+				'table_columns' => [
+					_('Name'),
+					_('Mapping')
+				]
+			],
 			'dashboard' => [
 				'title' => _('Dashboards'),
 				'min_user_type' => USER_TYPE_ZABBIX_USER,
@@ -380,7 +481,7 @@ class CControllerPopupGeneric extends CController {
 			],
 			'sla' => [
 				'title' => _('SLA'),
-				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
 				'allowed_src_fields' => 'slaid,name',
 				'form' => [
 					'name' => 'slaform',
@@ -389,6 +490,30 @@ class CControllerPopupGeneric extends CController {
 				'table_columns' => [
 					_('Name')
 				]
+			],
+			'actions' => [
+				'title' => _('Actions'),
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
+				'allowed_src_fields' => 'actionid,name',
+				'form' => [
+					'name' => 'actionform',
+					'id' => 'actions'
+				],
+				'table_columns' => [
+					_('Actions')
+				]
+			],
+			'media_types' => [
+				'title' => _('Media types'),
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
+				'allowed_src_fields' => 'mediatypeid,name',
+				'form' => [
+					'name' => 'media_typeform',
+					'id' => 'media_types'
+				],
+				'table_columns' => [
+					_('Media type')
+				]
 			]
 		];
 	}
@@ -409,8 +534,11 @@ class CControllerPopupGeneric extends CController {
 			'srctbl' =>								'string',
 			'srcfld1' =>							'string|required|in '.$this->popup_properties[$this->source_table]['allowed_src_fields'],
 			'groupid' =>							'db hstgrp.groupid',
+			'templategroupid' =>					'db hstgrp.groupid',
 			'group' =>								'string',
+			'templategroup' =>						'string',
 			'hostid' =>								'db hosts.hostid',
+			'templateid' =>							'db hosts.hostid',
 			'host' =>								'string',
 			'parent_discoveryid' =>					'db items.itemid',
 			'templates' =>							'string|not_empty',
@@ -434,24 +562,25 @@ class CControllerPopupGeneric extends CController {
 			'with_monitored_triggers' =>			'in 1',
 			'with_monitored_items' =>				'in 1',
 			'with_httptests' => 					'in 1',
-			'with_hosts_and_templates' =>			'in 1',
-			'with_webitems' =>						'in 1',
 			'with_inherited' =>						'in 1',
 			'itemtype' =>							'in '.implode(',', self::ALLOWED_ITEM_TYPES),
 			'value_types' =>						'array',
-			'context' =>							'string|in host,template',
+			'context' =>							'string|in host,template,audit',
 			'enabled_only' =>						'in 1',
 			'disable_names' =>						'array',
 			'numeric' =>							'in 1',
 			'reference' =>							'string',
 			'writeonly' =>							'in 1',
-			'noempty' =>							'in 1',
-			'submit_parent' =>						'in 1',
 			'enrich_parent_groups' =>				'in 1',
 			'filter_groupid_rst' =>					'in 1',
 			'filter_hostid_rst' =>					'in 1',
+			'filter_templateid_rst' =>				'in 1',
 			'user_type' =>							'in '.implode(',', [USER_TYPE_ZABBIX_USER, USER_TYPE_ZABBIX_ADMIN, USER_TYPE_SUPER_ADMIN]),
-			'hostids' => 							'array'
+			'group_status' =>						'in '.implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED]),
+			'hostids' =>							'array',
+			'host_pattern' =>						'array|not_empty',
+			'host_pattern_wildcard_allowed' =>		'in 1',
+			'host_pattern_multiple' =>				'in 1'
 		];
 
 		// Set destination and source field validation roles.
@@ -482,13 +611,12 @@ class CControllerPopupGeneric extends CController {
 		}
 
 		if (!$ret) {
-			$output = [];
-			if (($messages = getMessages()) !== null) {
-				$output['errors'] = $messages->toString();
-			}
-
 			$this->setResponse(
-				(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
+				(new CControllerResponseData(['main_block' => json_encode([
+					'error' => [
+						'messages' => array_column(get_and_clear_messages(), 'message')
+					]
+				])]))->disableView()
 			);
 		}
 
@@ -521,6 +649,29 @@ class CControllerPopupGeneric extends CController {
 			$this->groupids = array_keys($host_groups);
 		}
 
+		// Check template group permissions.
+		$templategroup_options = [];
+
+		if ($this->getInput('templategroup', '') !== '') {
+			$templategroup_options['filter']['name'] = $this->getInput('templategroup');
+		}
+		elseif ($this->hasInput('templategroupid')) {
+			$templategroup_options['groupids'] = $this->getInput('templategroupid');
+		}
+
+		if ($templategroup_options) {
+			$template_groups = API::TemplateGroup()->get([
+				'output' => [],
+				'preservekeys' => true
+			] + $templategroup_options);
+
+			if (!$template_groups) {
+				return false;
+			}
+
+			$this->template_groupids = array_keys($template_groups);
+		}
+
 		// Check host permissions.
 		$host_options = [];
 
@@ -549,6 +700,26 @@ class CControllerPopupGeneric extends CController {
 			$this->hostids = array_keys($hosts);
 		}
 
+		// Check template permissions.
+		$template_options = [];
+
+		if ($this->hasInput('templateid')) {
+			$template_options['templateids'] = $this->getInput('templateid');
+		}
+
+		if ($template_options) {
+			$templates = API::Template()->get([
+				'output' => [],
+				'preservekeys' => true
+			] + $template_options);
+
+			if (!$templates) {
+				return false;
+			}
+
+			$this->templateids = array_keys($templates);
+		}
+
 		// Check discovery rule permissions.
 		if ($this->hasInput('parent_discoveryid')) {
 			$lld_rules = API::DiscoveryRule()->get([
@@ -574,15 +745,21 @@ class CControllerPopupGeneric extends CController {
 
 		$group_options = [];
 		$host_options = [];
+		$templategroup_options = [];
+		$template_options = [];
 
 		if ($this->hasInput('writeonly')) {
 			$group_options['editable'] = 1;
 			$host_options['editable'] = 1;
+			$templategroup_options['editable'] = 1;
+			$template_options['editable'] = 1;
 		}
 
 		if ($this->hasInput('with_items')) {
 			$group_options['with_items'] = 1;
 			$host_options['with_items'] = 1;
+			$templategroup_options['with_items'] = 1;
+			$template_options['with_items'] = 1;
 		}
 		elseif ($this->hasInput('with_monitored_items')) {
 			$group_options['with_monitored_items'] = 1;
@@ -592,14 +769,15 @@ class CControllerPopupGeneric extends CController {
 		if ($this->hasInput('with_httptests')) {
 			$group_options['with_httptests'] = 1;
 			$host_options['with_httptests'] = 1;
+			$templategroup_options['with_httptests'] = 1;
 		}
 
 		if ($this->source_table === 'hosts' && !$this->hasInput('templated_hosts')) {
-			$group_options['real_hosts'] = 1;
+			$group_options['with_hosts'] = 1;
 		}
 		elseif ($this->source_table === 'templates') {
 			$host_options['templated_hosts'] = 1;
-			$group_options['templated_hosts'] = 1;
+			$templategroup_options['with_templates'] = 1;
 		}
 
 		if ($this->hasInput('monitored_hosts')) {
@@ -607,24 +785,29 @@ class CControllerPopupGeneric extends CController {
 			$host_options['monitored_hosts'] = 1;
 		}
 		elseif ($this->hasInput('real_hosts')) {
-			$group_options['real_hosts'] = 1;
+			$group_options['with_hosts'] = 1;
 			$host_options['real_hosts'] = 1;
 		}
 		elseif ($this->hasInput('templated_hosts')) {
 			$host_options['templated_hosts'] = 1;
-			$group_options['templated_hosts'] = 1;
+			$templategroup_options['with_templates'] = 1;
 		}
 		elseif ($this->source_table !== 'templates' && $this->source_table !== 'host_templates') {
-			$group_options['with_hosts_and_templates'] = 1;
+			$group_options['with_hosts'] = 1;
 		}
 
 		if ($this->hasInput('groupid')) {
 			$host_options['groupid'] = $this->getInput('groupid');
-			$group_options['groupid'] = $this->getInput('groupid');
 		}
 
-		if ($this->hasInput('enrich_parent_groups') || $this->group_preselect_required) {
+		if ($this->hasInput('templategroupid')) {
+			$template_options['groupid'] = $this->getInput('templategroupid');
+		}
+
+		if ($this->hasInput('enrich_parent_groups') || $this->group_preselect_required
+				|| $this->template_group_preselect_required) {
 			$group_options['enrich_parent_groups'] = 1;
+			$templategroup_options['enrich_parent_groups'] = 1;
 		}
 
 		foreach (['with_graphs', 'with_graph_prototypes', 'with_simple_graph_items',
@@ -632,6 +815,8 @@ class CControllerPopupGeneric extends CController {
 			if ($this->hasInput($name)) {
 				$group_options[$name] = 1;
 				$host_options[$name] = 1;
+				$templategroup_options[$name] = 1;
+				$template_options[$name] = 1;
 				break;
 			}
 		}
@@ -663,6 +848,32 @@ class CControllerPopupGeneric extends CController {
 			];
 		}
 
+		// Template group dropdown.
+		if ($this->template_group_preselect_required) {
+			$groups = $this->template_groupids
+				? API::TemplateGroup()->get([
+					'output' => ['name', 'groupid'],
+					'groupids' => $this->template_groupids
+				])
+				: [];
+
+			$filter['templategroups'] = [
+				'multiple' => false,
+				'name' => 'popup_template_group',
+				'object_name' => 'templateGroup',
+				'data' => CArrayHelper::renameObjectsKeys($groups, ['groupid' => 'id']),
+				'selectedLimit' => 1,
+				'popup' => [
+					'parameters' => [
+						'srctbl' => 'template_groups',
+						'srcfld1' => 'groupid',
+						'dstfld1' => 'popup_template_group'
+					] + $templategroup_options
+				],
+				'add_post_js' => false
+			];
+		}
+
 		// Host dropdown.
 		if (in_array($this->source_table, self::POPUPS_HAVING_HOST_FILTER)
 				&& ($this->source_table !== 'item_prototypes' || !$this->page_options['parent_discoveryid'])) {
@@ -714,6 +925,38 @@ class CControllerPopupGeneric extends CController {
 			];
 		}
 
+		// Template dropdown.
+		if (in_array($this->source_table, self::POPUPS_HAVING_TEMPLATE_FILTER)) {
+			$src_name = 'templates';
+
+			$templates = $this->templateids
+				? API::Template()->get([
+					'output' => ['name', 'hostid'],
+					'templateids' => $this->templateids
+				])
+				: [];
+
+			$templates = CArrayHelper::renameObjectsKeys($templates, ['templateid' => 'id']);
+
+			$this->templateids = array_column($templates, 'id');
+			$filter['templates'] = [
+				'multiple' => false,
+				'name' => 'popup_template',
+				'object_name' => $src_name,
+				'data' => array_values($templates),
+				'selectedLimit' => 1,
+				'disabled' => $this->hasInput('only_hostid'),
+				'popup' => [
+					'parameters' => [
+						'srctbl' => $src_name,
+						'srcfld1' => 'hostid',
+						'dstfld1' => 'popup_template'
+					] + $template_options
+				],
+				'add_post_js' => false
+			];
+		}
+
 		return $filter;
 	}
 
@@ -723,7 +966,7 @@ class CControllerPopupGeneric extends CController {
 	 * @return array
 	 */
 	protected function getPageOptions(): array {
-		$option_fields_binary = ['noempty', 'real_hosts', 'submit_parent', 'with_items', 'writeonly'];
+		$option_fields_binary = ['real_hosts', 'with_items', 'writeonly'];
 		$option_fields_value = ['host_templates'];
 
 		$page_options = [
@@ -740,9 +983,7 @@ class CControllerPopupGeneric extends CController {
 			'reference' => $this->getInput('reference', $this->getInput('srcfld1', 'unknown'))
 		];
 
-		$page_options['parentid'] = ($page_options['dstfld1'] !== '')
-			? zbx_jsvalue($page_options['dstfld1'])
-			: 'null';
+		$page_options['parentid'] = $page_options['dstfld1'] !== '' ? $page_options['dstfld1'] : null;
 
 		foreach ($option_fields_binary as $field) {
 			if ($this->hasInput($field)) {
@@ -776,9 +1017,45 @@ class CControllerPopupGeneric extends CController {
 			$this->groupids = CProfile::getArray('web.popup.generic.filter_groupid', []);
 		}
 
+		if ($this->template_groupids) {
+			CProfile::updateArray( 'web.popup.generic.filter_templategroupid', $this->template_groupids,
+				PROFILE_TYPE_ID
+			);
+		}
+		elseif ($this->hasInput('filter_groupid_rst')) {
+			CProfile::delete('web.popup.generic.filter_templategroupid');
+		}
+		else {
+			$this->template_groupids = CProfile::getArray('web.popup.generic.filter_templategroupid', []);
+		}
+
 		if ($this->hostids) {
 			CProfile::updateArray('web.popup.generic.filter_hostid', $this->hostids, PROFILE_TYPE_ID);
 		}
+		elseif ($this->hasInput('host_pattern')) {
+			$host_pattern_multiple = $this->hasInput('host_pattern_multiple');
+			$host_patterns = $host_pattern_multiple
+				? $this->getInput('host_pattern')
+				: [$this->getInput('host_pattern')];
+			$host_pattern_wildcard_enabled = $this->hasInput('host_pattern_wildcard_allowed')
+				&& !in_array('*', $host_patterns, true);
+
+			$hosts = API::Host()->get([
+				'output' => ['name'],
+				'search' => [
+					'name' => $host_pattern_wildcard_enabled ? $host_patterns : null
+				],
+				'searchWildcardsEnabled' => $host_pattern_wildcard_enabled,
+				'searchByAny' => true,
+				'preservekeys' => true
+			]);
+
+			CArrayHelper::sort($hosts, ['name']);
+
+			$this->hostids = $hosts ? [array_key_first($hosts)] : [];
+
+			CProfile::updateArray('web.popup.generic.filter_hostid', $this->hostids, PROFILE_TYPE_ID);
+		}
 		elseif ($this->hasInput('filter_hostid_rst')) {
 			CProfile::delete('web.popup.generic.filter_hostid');
 		}
@@ -786,10 +1063,25 @@ class CControllerPopupGeneric extends CController {
 			$this->hostids = CProfile::getArray('web.popup.generic.filter_hostid', []);
 		}
 
+		if ($this->templateids) {
+			CProfile::updateArray('web.popup.generic.filter_templateid', $this->templateids, PROFILE_TYPE_ID);
+		}
+		elseif ($this->hasInput('filter_templateid_rst')) {
+			CProfile::delete('web.popup.generic.filter_templateid');
+		}
+		else {
+			$this->templateids = CProfile::getArray('web.popup.generic.filter_templateid', []);
+		}
+
 		// Set popup options.
 		$this->host_preselect_required = in_array($this->source_table, self::POPUPS_HAVING_HOST_FILTER);
-		$this->group_preselect_required = in_array($this->source_table, self::POPUPS_HAVING_GROUP_FILTER)
+		$this->group_preselect_required = in_array($this->source_table, self::POPUPS_HAVING_HOST_GROUP_FILTER)
 			|| ($this->source_table === 'valuemaps' && !$this->hasInput('hostids'));
+		$this->template_group_preselect_required = in_array(
+			$this->source_table,
+			self::POPUPS_HAVING_TEMPLATE_GROUP_FILTER
+		) || ($this->source_table === 'template_valuemaps' && !$this->hasInput('hostids'));
+		$this->template_preselect_required = in_array($this->source_table, self::POPUPS_HAVING_TEMPLATE_FILTER);
 		$this->page_options = $this->getPageOptions();
 
 		// Make control filters. Must be called before extending groupids.
@@ -797,6 +1089,7 @@ class CControllerPopupGeneric extends CController {
 
 		// Select subgroups.
 		$this->groupids = getSubGroups($this->groupids);
+		$this->template_groupids = getTemplateSubGroups($this->template_groupids);
 
 		// Load results.
 		$records = $this->fetchResults();
@@ -814,7 +1107,9 @@ class CControllerPopupGeneric extends CController {
 			'table_columns' => $popup['table_columns'],
 			'table_records' => $records,
 			'preselect_required' => (($this->host_preselect_required && !$this->hostids)
-				|| ($this->group_preselect_required && !$this->groupids)),
+				|| ($this->group_preselect_required && !$this->groupids)
+				|| ($this->template_group_preselect_required && !$this->template_groupids)
+				|| ($this->template_preselect_required && !$this->templateids)),
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
 			]
@@ -902,7 +1197,7 @@ class CControllerPopupGeneric extends CController {
 
 			case 'items':
 				foreach ($records as $itemid => $row) {
-					$records[$row['name']] = ['itemid' => $row['name']] + $row;
+					$records[$row['name']] = ['pattern' => $row['name']] + $row;
 					unset($records[$itemid]);
 				}
 				break;
@@ -955,26 +1250,34 @@ class CControllerPopupGeneric extends CController {
 					'output' => API_OUTPUT_EXTEND
 				];
 
+				if ($this->hasInput('group_status')) {
+					$options['status'] = $this->getInput('group_status');
+				}
+
 				$records = API::UserGroup()->get($options);
 				CArrayHelper::sort($records, ['name']);
 				break;
 
 			case 'users':
 				$options += [
-					'output' => ['username', 'name', 'surname', 'type', 'theme', 'lang']
+					'output' => ['userid', 'username', 'name', 'surname', 'type', 'theme', 'lang']
 				];
 
 				$records = API::User()->get($options);
+
+				if ($this->hasInput('context')) {
+					$records[0] = ['userid' => 0, 'username' => 'System', 'name' => '', 'surname' => ''];
+				}
+
 				CArrayHelper::sort($records, ['username']);
 				break;
 
 			case 'templates':
 				$options += [
 					'output' => ['templateid', 'name'],
-					'groupids' => $this->groupids ? $this->groupids : null
+					'groupids' => $this->template_groupids ? $this->template_groupids : null
 				];
-
-				$records = (!$this->group_preselect_required || $this->groupids)
+				$records = (!$this->template_group_preselect_required || $this->template_groupids)
 					? API::Template()->get($options)
 					: [];
 
@@ -989,8 +1292,7 @@ class CControllerPopupGeneric extends CController {
 					'real_hosts' => $this->hasInput('real_hosts') ? '1' : null,
 					'with_httptests' => $this->hasInput('with_httptests') ? '1' : null,
 					'with_items' => $this->hasInput('with_items') ? true : null,
-					'with_triggers' => $this->hasInput('with_triggers') ? true : null,
-					'templated_hosts' => $this->hasInput('with_hosts_and_templates') ? true : null
+					'with_triggers' => $this->hasInput('with_triggers') ? true : null
 				];
 
 				if ($this->hasInput('with_monitored_triggers')) {
@@ -1032,22 +1334,15 @@ class CControllerPopupGeneric extends CController {
 			case 'host_groups':
 				$options += [
 					'output' => ['groupid', 'name'],
-					'with_triggers' => $this->hasInput('with_triggers') ? true : null
+					'with_triggers' => $this->hasInput('with_triggers')
 				];
 
 				if (array_key_exists('real_hosts', $this->page_options)) {
-					$options['real_hosts'] = 1;
-				}
-				elseif ($this->hasInput('templated_hosts')) {
-					$options['templated_hosts'] = 1;
+					$options['with_hosts'] = true;
 				}
 
 				if ($this->hasInput('with_httptests')) {
-					$options['with_httptests'] = 1;
-				}
-
-				if ($this->hasInput('with_hosts_and_templates')) {
-					$options['with_hosts_and_templates'] = 1;
+					$options['with_httptests'] = true;
 				}
 
 				if ($this->hasInput('with_items')) {
@@ -1071,6 +1366,33 @@ class CControllerPopupGeneric extends CController {
 				$records = CArrayHelper::renameObjectsKeys($records, ['groupid' => 'id']);
 				break;
 
+			case 'template_groups':
+				$options += [
+					'output' => ['groupid', 'name'],
+					'with_triggers' => $this->hasInput('with_triggers')
+				];
+
+				if ($this->hasInput('templated_hosts')) {
+					$options['with_templates'] = true;
+				}
+
+				if ($this->hasInput('with_httptests')) {
+					$options['with_httptests'] = true;
+				}
+
+				if ($this->hasInput('with_items')) {
+					$options['with_items'] = true;
+				}
+
+				$records = API::TemplateGroup()->get($options);
+				if ($this->hasInput('enrich_parent_groups')) {
+					$records = enrichParentTemplateGroups($records);
+				}
+
+				CArrayHelper::sort($records, ['name']);
+				$records = CArrayHelper::renameObjectsKeys($records, ['groupid' => 'id']);
+				break;
+
 			case 'help_items':
 				$records = CItemData::getByType($this->page_options['itemtype']);
 				break;
@@ -1108,6 +1430,31 @@ class CControllerPopupGeneric extends CController {
 				CArrayHelper::sort($records, ['description']);
 				break;
 
+			case 'template_triggers':
+				$options += [
+					'output' => ['triggerid', 'expression', 'description', 'status', 'priority', 'state'],
+					'selectHosts' => ['name'],
+					'selectDependencies' => ['triggerid', 'expression', 'description'],
+					'expandDescription' => true
+				];
+
+				if ($this->templateids) {
+					$options['templateids'] = $this->templateids;
+				}
+				elseif ($this->groupids) {
+					$options['groupids'] = $this->groupids;
+				}
+
+				if (!$this->template_preselect_required || $this->templateids) {
+					$records = API::Trigger()->get($options);
+				}
+				else {
+					$records = [];
+				}
+
+				CArrayHelper::sort($records, ['description']);
+				break;
+
 			case 'trigger_prototypes':
 				$options += [
 					'output' => ['triggerid', 'expression', 'description', 'status', 'priority', 'state'],
@@ -1137,11 +1484,11 @@ class CControllerPopupGeneric extends CController {
 				];
 
 				if ($this->source_table === 'items') {
-					$options['output'] = array_merge($options['output'], ['state']);
+					$options['output'][] = 'state';
 				}
 
 				if ($this->page_options['parent_discoveryid']) {
-					$options['discoveryids'] = [$this->page_options['parent_discoveryid']];
+					$options['discoveryids'] = $this->page_options['parent_discoveryid'];
 				}
 				elseif ($this->hostids) {
 					$options['hostids'] = $this->hostids;
@@ -1163,31 +1510,49 @@ class CControllerPopupGeneric extends CController {
 					$records = API::ItemPrototype()->get($options);
 				}
 				else {
-					if ($this->hasInput('with_webitems')) {
-						$options['webitems'] = true;
-					}
-
 					if ($this->hasInput('normal_only')) {
 						$options['filter']['flags'] = ZBX_FLAG_DISCOVERY_NORMAL;
 					}
 
-					$records = (!$this->host_preselect_required || $this->hostids)
-						? API::Item()->get($options)
+					$records = !$this->host_preselect_required || $this->hostids
+						? API::Item()->get($options + ['webitems' => true])
 						: [];
 				}
 
 				CArrayHelper::sort($records, ['name']);
 				break;
 
+			case 'template_items':
+				$options += [
+					'output' => ['itemid', 'name', 'key_', 'flags', 'type', 'value_type', 'status'],
+					'selectHosts' => ['name']
+				];
+
+				if ($this->page_options['parent_discoveryid']) {
+					$options['discoveryids'] = $this->page_options['parent_discoveryid'];
+				}
+				elseif ($this->templateids) {
+					$options['templateids'] = $this->templateids;
+				}
+
+				$records = !$this->template_preselect_required || $this->templateids
+					? API::Item()->get($options + ['webitems' => true])
+					: [];
+
+				CArrayHelper::sort($records, ['name']);
+				break;
+
 			case 'graphs':
 			case 'graph_prototypes':
 				$options += [
 					'output' => API_OUTPUT_EXTEND,
-					'selectHosts' => ['name'],
+					'selectHosts' => ['hostid', 'name'],
 					'hostids' => $this->hostids ? $this->hostids : null
 				];
 
 				if ($this->source_table === 'graph_prototypes') {
+					$options['selectDiscoveryRule'] = ['hostid'];
+
 					$records = (!$this->host_preselect_required || $this->hostids)
 						? API::GraphPrototype()->get($options)
 						: [];
@@ -1202,19 +1567,26 @@ class CControllerPopupGeneric extends CController {
 				break;
 
 			case 'sysmaps':
-				$options += [
-					'output' => API_OUTPUT_EXTEND
-				];
+				$records = API::Map()->get([
+					'output' => ['sysmapid', 'name'],
+					'preservekeys' => true
+				]);
 
-				$records = API::Map()->get($options);
+				$records = CArrayHelper::renameObjectsKeys($records, ['sysmapid' => 'id']);
 
 				CArrayHelper::sort($records, ['name']);
 				break;
 
 			case 'drules':
+				$filter = [];
+
+				if ($this->getInput('enabled_only', 0)) {
+					$filter['status'] = DRULE_STATUS_ACTIVE;
+				}
+
 				$records = API::DRule()->get([
 					'output' => ['druleid', 'name'],
-					'filter' => ['status' => DRULE_STATUS_ACTIVE],
+					'filter' => $filter,
 					'preservekeys' => true
 				]);
 
@@ -1307,53 +1679,40 @@ class CControllerPopupGeneric extends CController {
 				break;
 
 			case 'valuemaps':
+			case 'template_valuemaps':
 				/**
 				 * Show list of value maps with their mappings for defined hosts or templates.
 				 *
-				 * context  Define context for hostids value maps: host, template. Required together with "hostids".
+				 * context  Define context for hostids value maps: host, template. Required.
 				 * hostids  Array of host or template ids to get value maps from. Filter by groups will be displayed if
-				 *          this parameter is not set;
+				 *          this parameter is not set.
 				 */
 				$records = [];
+				$hostids = $this->getInput('hostids', []);
+				$context = $this->getInput('context', '');
 
-				if (($this->hasInput('hostids') && !$this->hasInput('context'))
-						|| (!$this->hasInput('hostids') && !$this->groupids)) {
+				if ($context === '' || (!$hostids && !$this->groupids && !$this->template_groupids)) {
 					break;
 				}
 
 				$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT);
 
-				if ($this->hasInput('hostids')) {
-					$hostids = $this->getInput('hostids');
-					$context = $this->getInput('context');
+				$options = [
+					'output' => ['name'],
+					'preservekeys' => true
+				];
 
-					if ($context === 'host') {
-						$hosts = API::Host()->get([
-							'output' => ['name'],
-							'hostids' => $hostids,
-							'preservekeys' => true
-						]);
-					}
-					else {
-						$hosts = API::Template()->get([
-							'output' => ['name'],
-							'templateids' => $hostids,
-							'preservekeys' => true
-						]);
-					}
+				if ($hostids) {
+					$hosts = $context === 'host'
+						? API::Host()->get($options + ['hostids' => $hostids])
+						: API::Template()->get($options + ['templateids' => $hostids]);
 				}
 				else {
-					$hosts = API::Host()->get([
-						'output' => ['name'],
-						'groupids' => $this->groupids,
-						'preservekeys' => true,
-						'limit' => $limit
-					]) + API::Template()->get([
-						'output' => ['name'],
-						'groupids' => $this->groupids,
-						'preservekeys' => true,
-						'limit' => $limit
-					]);
+					$options['limit'] = $limit;
+
+					$hosts = $context === 'host'
+						? API::Host()->get($options + ['groupids' => $this->groupids])
+						: API::Template()->get($options + ['groupids' => $this->template_groupids]);
 
 					$hostids = array_keys($hosts);
 				}
@@ -1409,6 +1768,20 @@ class CControllerPopupGeneric extends CController {
 				CArrayHelper::sort($records, ['name']);
 				$records = CArrayHelper::renameObjectsKeys($records, ['slaid' => 'id']);
 				break;
+
+			case 'actions':
+				$options += ['output' => ['name']];
+
+				$records = API::Action()->get($options);
+				CArrayHelper::sort($records, ['name']);
+				break;
+
+			case 'media_types':
+				$options += ['output' => ['name']];
+
+				$records = API::MediaType()->get($options);
+				CArrayHelper::sort($records, ['name']);
+				break;
 		}
 
 		return $records;
