@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -149,8 +149,6 @@ abstract class CControllerPopupItemTest extends CController {
 			'support_lld_macros' => true
 		],
 		'http_proxy' => [
-			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
-			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
 			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
@@ -247,9 +245,9 @@ abstract class CControllerPopupItemTest extends CController {
 	protected $is_item_testable;
 
 	/**
-	 * @var object
+	 * @var string
 	 */
-	protected $preproc_item;
+	protected $test_type;
 
 	/**
 	 * @var array
@@ -331,26 +329,6 @@ abstract class CControllerPopupItemTest extends CController {
 		return $ret;
 	}
 
-	/**
-	 * Function returns instance of item, item prototype or discovery rule class.
-	 *
-	 * @param int $test_type
-	 *
-	 * @return CItem|CItemPrototype|CDiscoveryRule
-	 */
-	protected static function getPreprocessingItemClassInstance($test_type) {
-		switch ($test_type) {
-			case self::ZBX_TEST_TYPE_ITEM:
-				return new CItem;
-
-			case self::ZBX_TEST_TYPE_ITEM_PROTOTYPE:
-				return new CItemPrototype;
-
-			case self::ZBX_TEST_TYPE_LLD:
-				return new CDiscoveryRule;
-		}
-	}
-
 	/**
 	 * Function returns list of proxies.
 	 *
@@ -563,7 +541,7 @@ abstract class CControllerPopupItemTest extends CController {
 					'key' => $input['key'],
 					'http_authtype' => array_key_exists('http_authtype', $input)
 						? $input['http_authtype']
-						: HTTPTEST_AUTH_NONE,
+						: ZBX_HTTP_AUTH_NONE,
 					'follow_redirects' => array_key_exists('follow_redirects', $input) ? $input['follow_redirects'] : 0,
 					'headers' => array_key_exists('headers', $input) ? $input['headers'] : [],
 					'http_proxy' => array_key_exists('http_proxy', $input) ? $input['http_proxy'] : null,
@@ -589,7 +567,7 @@ abstract class CControllerPopupItemTest extends CController {
 					'verify_peer' => array_key_exists('verify_peer', $input) ? $input['verify_peer'] : 0
 				];
 
-				if ($data['http_authtype'] != HTTPTEST_AUTH_NONE) {
+				if ($data['http_authtype'] != ZBX_HTTP_AUTH_NONE) {
 					$data += [
 						'http_username' => array_key_exists('http_username', $input) ? $input['http_username'] : null,
 						'http_password' => array_key_exists('http_password', $input) ? $input['http_password'] : null
@@ -719,7 +697,8 @@ abstract class CControllerPopupItemTest extends CController {
 				'privpassphrase' => '',
 				'authprotocol' => ITEM_SNMPV3_AUTHPROTOCOL_MD5,
 				'privprotocol' => ITEM_SNMPV3_PRIVPROTOCOL_DES,
-				'contextname' => ''
+				'contextname' => '',
+				'max_repetitions' => '10'
 			]
 		];
 
@@ -929,7 +908,7 @@ abstract class CControllerPopupItemTest extends CController {
 				'supported_macros' => array_diff_key($this->macros_by_item_props['key'],
 					['support_user_macros' => true, 'support_lld_macros' => true]
 				),
-				'support_lldmacros' => ($this->preproc_item instanceof CItemPrototype),
+				'support_lldmacros' => ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE),
 				'texts_support_macros' => [$inputs['key']],
 				'texts_support_lld_macros' => [$inputs['key']],
 				'texts_support_user_macros' => [$inputs['key']],
@@ -1025,7 +1004,7 @@ abstract class CControllerPopupItemTest extends CController {
 	protected function resolvePreprocessingStepMacros(array $steps) {
 		// Resolve macros used in parameter fields.
 		$macros_posted = $this->getInput('macros', []);
-		$macros_types = ($this->preproc_item instanceof CItemPrototype)
+		$macros_types = ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE)
 			? ['usermacros' => true, 'lldmacros' => true]
 			: ['usermacros' => true];
 
@@ -1038,7 +1017,7 @@ abstract class CControllerPopupItemTest extends CController {
 
 			// Resolve macros in parameter fields before send data to Zabbix server.
 			foreach (['params', 'error_handler_params'] as $field) {
-				$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($step[$field], $macros_types);
+				$matched_macros = CMacrosResolverGeneral::getMacroPositions($step[$field], $macros_types);
 
 				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 					$macro_value = array_key_exists($macro, $macros_posted)
@@ -1069,7 +1048,7 @@ abstract class CControllerPopupItemTest extends CController {
 
 		$expression_parser = new CExpressionParser([
 			'usermacros' => true,
-			'lldmacros' => ($this->preproc_item instanceof CItemPrototype),
+			'lldmacros' => ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE),
 			'calculated' => true,
 			'host_macro' => true,
 			'empty_host' => true
@@ -1200,7 +1179,7 @@ abstract class CControllerPopupItemTest extends CController {
 				'macros_n' => []
 			];
 
-			if ($this->preproc_item instanceof CItemPrototype) {
+			if ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE) {
 				$types += ['lldmacros' => true];
 			}
 
@@ -1220,7 +1199,7 @@ abstract class CControllerPopupItemTest extends CController {
 					foreach (array_keys($inputs[$field][$key]) as $nr) {
 						$str = &$inputs[$field][$key][$nr];
 						if (strstr($str, '{') !== false) {
-							$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($str, $types);
+							$matched_macros = CMacrosResolverGeneral::getMacroPositions($str, $types);
 
 							foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 								$macro_value = array_key_exists($macro, $macros_posted)
@@ -1240,7 +1219,7 @@ abstract class CControllerPopupItemTest extends CController {
 					$inputs[$field] = CMacrosResolverGeneral::resolveItemKeyMacros($inputs[$field], $macros_posted, $types);
 				}
 				else {
-					$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($inputs[$field], $types);
+					$matched_macros = CMacrosResolverGeneral::getMacroPositions($inputs[$field], $types);
 
 					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 						$macro_value = array_key_exists($macro, $macros_posted)
@@ -1268,12 +1247,13 @@ abstract class CControllerPopupItemTest extends CController {
 		if (array_key_exists('interface', $inputs) && array_key_exists('details', $inputs['interface'])) {
 			foreach ($inputs['interface']['details'] as &$field) {
 				if (strstr($field, '{') !== false) {
-					$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($field, ['usermacros' => true]);
+					$matched_macros = CMacrosResolverGeneral::getMacroPositions($field, ['usermacros' => true]);
 
 					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+						// If matching macro is not found, return unresolved macro string.
 						$macro_value = array_key_exists($macro, $macros_posted)
 							? $macros_posted[$macro]
-							: '';
+							: $macro;
 
 						$field = substr_replace($field, $macro_value, $pos, strlen($macro));
 					}
@@ -1325,6 +1305,33 @@ abstract class CControllerPopupItemTest extends CController {
 
 				return false;
 			}
+
+			if ($interface['details']['version'] == SNMP_V2C || $interface['details']['version'] == SNMP_V3) {
+				if (!array_key_exists('max_repetitions', $interface['details'])
+						|| $interface['details']['max_repetitions'] === '') {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _('cannot be empty')));
+
+					return false;
+				}
+
+				if (!is_numeric($interface['details']['max_repetitions'])) {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _('a numeric value is expected')));
+
+					return false;
+				}
+
+				if ($interface['details']['max_repetitions'] < 1) {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _s('value must be no less than "%1$s"', 1)));
+
+					return false;
+				}
+
+				if ($interface['details']['max_repetitions'] > ZBX_MAX_INT32) {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _s('value must be no greater than "%1$s"', ZBX_MAX_INT32)));
+
+					return false;
+				}
+			}
 		}
 
 		if ($this->items_require_interface[$this->item_type]['address']
