@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -540,7 +540,7 @@ class CControllerPopupMassupdateHost extends CControllerPopupMassupdateAbstract
 			];
 
 			$data['proxies'] = API::Proxy()->get([
-				'output' => ['hostid', 'host'],
+				'output' => ['proxyid', 'host'],
 				'filter' => [
 					'status' => [HOST_STATUS_PROXY_ACTIVE, HOST_STATUS_PROXY_PASSIVE]
 				],
