@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -30,15 +30,15 @@ class CControllerPopupMediatypeMessage extends CController {
 	protected $message_types = [];
 
 	protected function init() {
-		$this->disableSIDvalidation();
+		$this->disableCsrfValidation();
 
 		$this->message_types = CMediatypeHelper::getAllMessageTypes();
 	}
 
 	protected function checkInput() {
 		$fields = [
-			'type' =>				'required|in '.implode(',', array_keys(media_type2str())),
-			'content_type' =>		'required|in '.SMTP_MESSAGE_FORMAT_PLAIN_TEXT.','.SMTP_MESSAGE_FORMAT_HTML,
+			'type' =>				'in '.implode(',', array_keys(CMediatypeHelper::getMediaTypes())),
+			'content_type' =>		'in '.SMTP_MESSAGE_FORMAT_PLAIN_TEXT.','.SMTP_MESSAGE_FORMAT_HTML,
 			'message_type' =>		'in -1,'.implode(',', $this->message_types),
 			'old_message_type' =>	'in -1,'.implode(',', $this->message_types),
 			'message_types' =>		'array',
