@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -120,7 +120,9 @@ if (!$data['system_info']['float_double_precision']) {
 	$info_table->addRow([
 		_('Database history tables upgraded'),
 		(new CSpan(_('No')))->addClass(ZBX_STYLE_RED),
-		''
+		(new CCol(
+			_('Support for the old numeric type is deprecated. Please upgrade to numeric values of extended range.')
+		))->addClass(ZBX_STYLE_RED)
 	]);
 }
 
@@ -198,7 +200,7 @@ if ($data['user_type'] == USER_TYPE_SUPER_ADMIN) {
 			(new CCol([
 				_('This setting should be enabled, because history tables contain compressed chunks.'),
 				' ',
-				new CLink(_('Configuration').'&hellip;',
+				new CLink([_('Configuration'), HELLIP()],
 					(new CUrl('zabbix.php'))->setArgument('action', 'housekeeping.edit')
 				)
 			]))->addClass(ZBX_STYLE_RED)
@@ -212,7 +214,7 @@ if ($data['user_type'] == USER_TYPE_SUPER_ADMIN) {
 			(new CCol([
 				_('This setting should be enabled, because trend tables contain compressed chunks.'),
 				' ',
-				new CLink(_('Configuration').'&hellip;',
+				new CLink([_('Configuration'), HELLIP()],
 					(new CUrl('zabbix.php'))->setArgument('action', 'housekeeping.edit')
 				)
 			]))->addClass(ZBX_STYLE_RED)
