@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * @var CView $this
  */
 
-$widget = (new CWidget())
+$html_page = (new CHtmlPage())
 	->setTitle(_('Icon mapping'))
 	->setTitleSubmenu(getAdministrationGeneralSubmenu())
 	->setDocUrl(CDocHelper::getUrl(CDocHelper::ADMINISTRATION_ICONMAP_LIST))
@@ -39,17 +39,21 @@ $widget = (new CWidget())
 $table = (new CTableInfo())->setHeader([_('Name'), _('Icon map')]);
 
 foreach ($data['iconmaps'] as $icon_map) {
-	$row = [];
+	$mappings = [];
+
 	foreach ($icon_map['mappings'] as $mapping) {
-		$row[] = $data['inventory_list'][$mapping['inventory_link']].NAME_DELIMITER.
-				$mapping['expression'].SPACE.'&rArr;'.SPACE.$data['icon_list'][$mapping['iconid']];
-		$row[] = BR();
+		$mappings[] = [
+			$data['inventory_list'][$mapping['inventory_link']].NAME_DELIMITER.$mapping['expression'],
+			NBSP(), RARR(), NBSP(),
+			$data['icon_list'][$mapping['iconid']],
+			BR()
+		];
 	}
 
 	$table->addRow([new CLink($icon_map['name'], (new CUrl('zabbix.php'))
 		->setArgument('action', 'iconmap.edit')
 		->setArgument('iconmapid', $icon_map['iconmapid'])
-	), $row]);
+	), $mappings]);
 }
 
-$widget->addItem($table)->show();
+$html_page->addItem($table)->show();
