@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,17 +19,25 @@
 **/
 
 
+/**
+ * @var CView $this
+ * @var array $data
+ */
+
 if ($data['uncheck']) {
 	uncheckTableRows('modules');
 }
 
-$widget = (new CWidget())
+$csrf_token = CCsrfTokenHelper::get('module');
+
+$html_page = (new CHtmlPage())
 	->setTitle(_('Modules'))
 	->setTitleSubmenu(getAdministrationGeneralSubmenu())
 	->setDocUrl(CDocHelper::getUrl(CDocHelper::ADMINISTRATION_MODULE_LIST))
 	->setControls(
 		(new CTag('nav', true,
 			(new CForm())
+				->addItem((new CVar(CCsrfTokenHelper::CSRF_TOKEN_NAME, $csrf_token))->removeId())
 				->addVar('action', 'module.scan')
 				->addItem((new CList())
 					->addItem(new CSubmit('form', _('Scan directory')))
@@ -94,15 +102,15 @@ foreach ($data['modules'] as $moduleid => $module) {
 
 	if ($module['status'] == MODULE_STATUS_ENABLED) {
 		$status = (new CLink(_('Enabled'), $status_url))
+			->addCsrfToken($csrf_token)
 			->addClass(ZBX_STYLE_LINK_ACTION)
-			->addClass(ZBX_STYLE_GREEN)
-			->addSID();
+			->addClass(ZBX_STYLE_GREEN);
 	}
 	else {
 		$status = (new CLink(_('Disabled'), $status_url))
+			->addCsrfToken($csrf_token)
 			->addClass(ZBX_STYLE_LINK_ACTION)
-			->addClass(ZBX_STYLE_RED)
-			->addSID();
+			->addClass(ZBX_STYLE_RED);
 	}
 
 	// append table row
@@ -121,12 +129,16 @@ $form->addItem([
 	$table,
 	$data['paging'],
 	new CActionButtonList('action', 'moduleids', [
-		'module.enable' => ['name' => _('Enable'), 'confirm' => _('Enable selected modules?')],
-		'module.disable' => ['name' => _('Disable'), 'confirm' => _('Disable selected modules?')]
+		'module.enable' => ['name' => _('Enable'), 'confirm' => _('Enable selected modules?'),
+			'csrf_token' => $csrf_token
+		],
+		'module.disable' => ['name' => _('Disable'), 'confirm' => _('Disable selected modules?'),
+			'csrf_token' => $csrf_token
+		]
 	], 'modules')
 ]);
 
 // append form to widget
-$widget->addItem($form);
+$html_page->addItem($form);
 
-$widget->show();
+$html_page->show();
