@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,26 +29,14 @@ if ($data['uncheck']) {
 	uncheckTableRows('user');
 }
 
-$widget = (new CWidget())
+$html_page = (new CHtmlPage())
 	->setTitle(_('Users'))
-	->setDocUrl(CDocHelper::getUrl(CDocHelper::ADMINISTRATION_USER_LIST))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::USERS_USER_LIST))
 	->setControls((new CList([
 		(new CForm('get'))
-			->cleanItems()
 			->setName('main_filter')
 			->setAttribute('aria-label', _('Main filter'))
-			->addItem((new CVar('action', 'user.list'))->removeId())
-			->addItem((new CList())
-				->addItem([
-					new CLabel(_('User group'), 'label-filter-usrgrpid'),
-					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-					(new CSelect('filter_usrgrpid'))
-						->setId('filter-usrgrpid')
-						->setValue($data['filter_usrgrpid'])
-						->setFocusableElementId('label-filter-usrgrpid')
-						->addOptions(CSelect::createOptionsFromArray($data['user_groups']))
-				])
-			),
+			->addItem((new CVar('action', 'user.list'))->removeId()),
 			(new CTag('nav', true,
 				(new CList())
 					->addItem(new CRedirectButton(_('Create user'), 'zabbix.php?action=user.edit'))
@@ -60,32 +48,66 @@ $widget = (new CWidget())
 		->setProfile($data['profileIdx'])
 		->setActiveTab($data['active_tab'])
 		->addFilterTab(_('Filter'), [
-			(new CFormList())->addRow(_('Username'),
-				(new CTextBox('filter_username', $data['filter']['username']))
-					->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
-					->setAttribute('autofocus', 'autofocus')
-			),
-			(new CFormList())->addRow(_('Name'),
-				(new CTextBox('filter_name', $data['filter']['name']))->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
-			),
-			(new CFormList())->addRow(_('Last name'),
-				(new CTextBox('filter_surname', $data['filter']['surname']))->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
-			),
-			(new CFormList())->addRow((new CLabel(_('User roles'), 'filter_roles__ms')),
-				(new CMultiSelect([
-					'name' => 'filter_roles[]',
-					'object_name' => 'roles',
-					'data' => $data['filter']['roles'],
-					'popup' => [
-						'parameters' => [
-							'srctbl' => 'roles',
-							'srcfld1' => 'roleid',
-							'dstfrm' => 'zbx_filter',
-							'dstfld1' => 'filter_roles_'
-						]
-					]
-				]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			)
+			(new CFormGrid())
+				->addItem([
+					new CLabel(_('Username'), 'filter_username'),
+					new CFormField(
+						(new CTextBox('filter_username', $data['filter']['username']))
+							->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+							->setAttribute('autofocus', 'autofocus')
+					)
+				])
+				->addItem([
+					new CLabel(_('Name'), 'filter_name'),
+					new CFormField(
+						(new CTextBox('filter_name', $data['filter']['name']))
+							->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+					)
+				])
+				->addItem([
+					new CLabel(_('Last name'), 'filter_surname'),
+					new CFormField(
+						(new CTextBox('filter_surname', $data['filter']['surname']))
+							->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+					)
+				]),
+			(new CFormGrid())
+				->addItem([(
+					new CLabel(_('User roles'), 'filter_roles__ms')),
+					new CFormField(
+						(new CMultiSelect([
+							'name' => 'filter_roles[]',
+							'object_name' => 'roles',
+							'data' => $data['filter']['roles'],
+							'popup' => [
+								'parameters' => [
+									'srctbl' => 'roles',
+									'srcfld1' => 'roleid',
+									'dstfrm' => 'zbx_filter',
+									'dstfld1' => 'filter_roles_'
+								]
+							]
+						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+					)
+				])
+				->addItem([
+					new CLabel(_('User groups'), 'filter_usrgrpids__ms'),
+					new CFormField(
+						(new CMultiSelect([
+							'name' => 'filter_usrgrpids[]',
+							'object_name' => 'usersGroups',
+							'data' => $data['filter']['usrgrpids'],
+							'popup' => [
+								'parameters' => [
+									'srctbl' => 'usrgrp',
+									'srcfld1' => 'usrgrpid',
+									'dstfrm' => 'zbx_filter',
+									'dstfld1' => 'filter_usrgrpids_'
+								]
+							]
+						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+					)
+				])
 		])
 		->addVar('action', 'user.list')
 	);
@@ -114,9 +136,13 @@ $table = (new CTableInfo())
 		_('Frontend access'),
 		_('API access'),
 		_('Debug mode'),
-		_('Status')
+		_('Status'),
+		make_sorting_header(_('Provisioned'), 'ts_provisioned', $data['sort'], $data['sortorder'], $url),
+		_('Info')
 	]);
 
+$csrf_token = CCsrfTokenHelper::get('user');
+
 foreach ($data['users'] as $user) {
 	$userid = $user['userid'];
 	$session = $data['sessions'][$userid];
@@ -144,7 +170,7 @@ foreach ($data['users'] as $user) {
 		? (new CLink(_('Blocked'), 'zabbix.php?action=user.unblock&userids[]='.$userid))
 			->addClass(ZBX_STYLE_LINK_ACTION)
 			->addClass(ZBX_STYLE_RED)
-			->addSID()
+			->addCsrfToken($csrf_token)
 		: (new CSpan(_('Ok')))->addClass(ZBX_STYLE_GREEN);
 
 	order_result($user['usrgrps'], 'name');
@@ -156,7 +182,7 @@ foreach ($data['users'] as $user) {
 		$i++;
 
 		if ($i > $data['config']['max_in_table']) {
-			$users_groups[] = ' &hellip;';
+			$users_groups[] = [' ', HELLIP()];
 
 			break;
 		}
@@ -181,64 +207,89 @@ foreach ($data['users'] as $user) {
 		$users_groups[] = $group->addClass($style);
 	}
 
-	// GUI Access style.
-	switch ($user['gui_access']) {
-		case GROUP_GUI_ACCESS_INTERNAL:
-			$gui_access_style = ZBX_STYLE_ORANGE;
-			break;
-
-		case GROUP_GUI_ACCESS_DISABLED:
-			$gui_access_style = ZBX_STYLE_GREY;
-			break;
-
-		default:
-			$gui_access_style = ZBX_STYLE_GREEN;
-	}
-
-	$username = new CLink($user['username'], (new CUrl('zabbix.php'))
-		->setArgument('action', 'user.edit')
-		->setArgument('userid', $userid)
+	$provisioned = $user['userdirectoryid'] ? new CDiv(date(ZBX_DATE_TIME, $user['ts_provisioned'])) : '';
+	$checkbox = new CCheckBox('userids['.$userid.']', $userid);
+	$info = $users_groups ? '' : makeWarningIcon(_('User does not have user groups.'));
+	$username = new CLink($user['username'],
+		(new CUrl('zabbix.php'))
+			->setArgument('action', 'user.edit')
+			->setArgument('userid', $userid)
 	);
 
-	if (!CRoleHelper::checkAccess('api.access', $user['roleid'])) {
+	if ($user['userdirectoryid'] && $data['idp_names'][$user['userdirectoryid']]['idp_type'] == IDP_TYPE_LDAP) {
+		$checkbox->setAttribute('data-actions', 'ldap');
+	}
+
+	if ($user['userdirectoryid']) {
+		$idp = $data['idp_names'][$user['userdirectoryid']];
+		$provisioned->setHint($idp['idp_type'] == IDP_TYPE_SAML ? _('SAML') : $idp['name']);
+		$gui_access = new CSpan($idp['idp_type'] == IDP_TYPE_LDAP ? _('LDAP') : _('SAML'));
+	}
+	else {
+		$gui_access = new CSpan(user_auth_type2str($user['gui_access']));
+	}
+
+	if (!$user['roleid']) {
+		$info = makeErrorIcon(_('User does not have user role.'));
+		$gui_access = (new CSpan(_('Disabled')))->addClass(ZBX_STYLE_GREY);
 		$api_access = (new CSpan(_('Disabled')))->addClass(ZBX_STYLE_RED);
 	}
 	else {
-		$api_access = (new CSpan(_('Enabled')))->addClass(ZBX_STYLE_GREEN);
-		$api_methods = CRoleHelper::getRoleApiMethods($user['roleid']);
+		switch ($user['gui_access']) {
+			case GROUP_GUI_ACCESS_INTERNAL:
+				$gui_access->addClass(ZBX_STYLE_ORANGE);
+				break;
 
-		if ($api_methods) {
-			$hint_api_methods = [];
-			$status_class = CRoleHelper::checkAccess('api.mode', $user['roleid'])
-				? ZBX_STYLE_STATUS_GREEN
-				: ZBX_STYLE_STATUS_GREY;
+			case GROUP_GUI_ACCESS_DISABLED:
+				$gui_access->addClass(ZBX_STYLE_GREY);
+				break;
 
-			foreach ($api_methods as $api_method) {
-				$hint_api_methods[] = (new CSpan($api_method))->addClass($status_class);
+			default:
+				$gui_access->addClass(ZBX_STYLE_GREEN);
+		}
+
+		if (!CRoleHelper::checkAccess('api.access', $user['roleid'])) {
+			$api_access = (new CSpan(_('Disabled')))->addClass(ZBX_STYLE_RED);
+		}
+		else {
+			$api_access = (new CSpan(_('Enabled')))->addClass(ZBX_STYLE_GREEN);
+			$api_methods = CRoleHelper::getRoleApiMethods($user['roleid']);
+
+			if ($api_methods) {
+				$hint_api_methods = [];
+				$status_class = CRoleHelper::checkAccess('api.mode', $user['roleid'])
+					? ZBX_STYLE_STATUS_GREEN
+					: ZBX_STYLE_STATUS_GREY;
+
+				foreach ($api_methods as $api_method) {
+					$hint_api_methods[] = (new CSpan($api_method))->addClass($status_class);
+				}
+
+				$api_access->setHint((new CDiv($hint_api_methods))->addClass('rules-status-container'));
 			}
-
-			$api_access->setHint((new CDiv($hint_api_methods))->addClass('rules-status-container'));
 		}
 	}
 
 	// Append user to table.
 	$table->addRow([
-		new CCheckBox('userids['.$userid.']', $userid),
+		$checkbox,
 		(new CCol($username))->addClass(ZBX_STYLE_NOWRAP),
 		$user['name'],
 		$user['surname'],
-		$user['role']['name'],
+		$user['role_name'],
 		$users_groups,
 		$online,
 		$blocked,
-		(new CSpan(user_auth_type2str($user['gui_access'])))->addClass($gui_access_style),
+		$gui_access,
 		$api_access,
 		($user['debug_mode'] == GROUP_DEBUG_MODE_ENABLED)
 			? (new CSpan(_('Enabled')))->addClass(ZBX_STYLE_ORANGE)
 			: (new CSpan(_('Disabled')))->addClass(ZBX_STYLE_GREEN),
-		($user['users_status'] == GROUP_STATUS_DISABLED)
+		($user['users_status'] == GROUP_STATUS_DISABLED || !$user['roleid'])
 			? (new CSpan(_('Disabled')))->addClass(ZBX_STYLE_RED)
-			: (new CSpan(_('Enabled')))->addClass(ZBX_STYLE_GREEN)
+			: (new CSpan(_('Enabled')))->addClass(ZBX_STYLE_GREEN),
+		$provisioned,
+		$info
 	]);
 }
 
@@ -247,12 +298,22 @@ $form->addItem([
 	$table,
 	$data['paging'],
 	new CActionButtonList('action', 'userids', [
-		'user.unblock' => ['name' => _('Unblock'), 'confirm' => _('Unblock selected users?')],
-		'user.delete' => ['name' => _('Delete'), 'confirm' => _('Delete selected users?')]
+		'user.provision' => [
+			'name' => _('Provision now'),
+			'attributes' => ['data-required' => 'ldap'],
+			'confirm' => _('Provision selected LDAP users?'),
+			'csrf_token' => $csrf_token
+		],
+		'user.unblock' => ['name' => _('Unblock'), 'confirm' => _('Unblock selected users?'),
+			'csrf_token' => $csrf_token
+		],
+		'user.delete' => ['name' => _('Delete'), 'confirm' => _('Delete selected users?'),
+			'csrf_token' => $csrf_token
+		]
 	], 'user')
 ]);
 
 // Append form to widget.
-$widget
+$html_page
 	->addItem($form)
 	->show();
