@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -249,7 +249,7 @@
 				url.setArgument(field.name, field.value);
 			});
 
-			redirect(url.getUrl(), 'post', 'action', undefined, false, true);
+			redirect(url.getUrl(), 'post', 'action', undefined, true);
 		});
 	});
 </script>
