@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -128,10 +128,11 @@ function cleanPreviousTestResults() {
 
 	jQuery('[id^="preproc-test-step-"][id$="-result"]', $form).empty();
 	jQuery('[id^="preproc-test-step-"][id$="-name"] > div', $form).remove();
-	jQuery('#final-result', $form)
+	jQuery('.js-final-result', $form)
 		.hide()
-		.find('.table-forms-td-right')
-		.empty();
+		.next()
+		.empty()
+		.hide();
 }
 
 /**
@@ -148,13 +149,15 @@ function itemGetValueTest(overlay) {
 		url = new Curl('zabbix.php');
 
 	url.setArgument('action', 'popup.itemtest.getvalue');
+	url.setArgument('<?= CCsrfTokenHelper::CSRF_TOKEN_NAME ?>', <?= json_encode(CCsrfTokenHelper::get('itemtest')) ?>);
 
 	post_data = jQuery.extend(post_data, {
 		interface: {
 			address: interface ? interface['address'].trim() : '',
 			port: (interface && interface['port']) ? interface['port'].trim() : '',
 			interfaceid: interface ? interface['interfaceid'] : null,
-			useip: interface ? interface['useip'] : null
+			useip: interface ? interface['useip'] : null,
+			details: interface ? interface['details'] : null
 		},
 		macros: form_data['macros'],
 		proxy_hostid: form_data['proxy_hostid'],
@@ -189,24 +192,27 @@ function itemGetValueTest(overlay) {
 		success: function(ret) {
 			overlay.$dialogue.find('.msg-bad, .msg-good, .msg-warning').remove();
 
-			if (typeof ret.messages !== 'undefined') {
-				jQuery($body).prepend(ret.messages);
+			if ('error' in ret) {
+				const message_box = makeMessageBox('bad', ret.error.messages, ret.error.title);
+
+				jQuery($body).prepend(message_box);
+
+				return;
 			}
-			else {
-				<?php if ($data['show_prev']): ?>
-					if (typeof ret.prev_value !== 'undefined') {
-						jQuery('#prev_value', $form).multilineInput('value', ret.prev_value);
-						jQuery('#prev_time', $form).val(ret.prev_time);
-						jQuery('#upd_prev', $form).val(form_data['upd_last']);
-						jQuery('#upd_last', $form).val(Math.ceil(+new Date() / 1000));
-					}
-				<?php endif ?>
 
-				jQuery('#value', $form).multilineInput('value', ret.value);
-
-				if (typeof ret.eol !== 'undefined') {
-					jQuery("input[value=" + ret.eol + "]", jQuery("#eol")).prop("checked", "checked");
+			<?php if ($data['show_prev']): ?>
+				if (typeof ret.prev_value !== 'undefined') {
+					jQuery('#prev_value', $form).multilineInput('value', ret.prev_value);
+					jQuery('#prev_time', $form).val(ret.prev_time);
+					jQuery('#upd_prev', $form).val(form_data['upd_last']);
+					jQuery('#upd_last', $form).val(Math.ceil(+new Date() / 1000));
 				}
+			<?php endif ?>
+
+			jQuery('#value', $form).multilineInput('value', ret.value);
+
+			if (typeof ret.eol !== 'undefined') {
+				jQuery("input[value=" + ret.eol + "]", jQuery("#eol")).prop("checked", "checked");
 			}
 		},
 		dataType: 'json',
@@ -228,6 +234,7 @@ function itemCompleteTest(overlay) {
 		url = new Curl('zabbix.php');
 
 	url.setArgument('action', 'popup.itemtest.send');
+	url.setArgument('<?= CCsrfTokenHelper::CSRF_TOKEN_NAME ?>', <?= json_encode(CCsrfTokenHelper::get('itemtest')) ?>);
 
 	post_data = jQuery.extend(post_data, {
 		get_value: form_data['get_value'] || 0,
@@ -236,7 +243,8 @@ function itemCompleteTest(overlay) {
 			address: interface ? interface['address'].trim() : '',
 			port: (interface && interface['port']) ? interface['port'].trim() : '',
 			interfaceid: interface ? interface['interfaceid'] : null,
-			useip: interface ? interface['useip'] : null
+			useip: interface ? interface['useip'] : null,
+			details: interface ? interface['details'] : null
 		},
 		macros: form_data['macros'],
 		proxy_hostid: form_data['proxy_hostid'],
@@ -244,7 +252,8 @@ function itemCompleteTest(overlay) {
 		test_type: <?= $data['test_type'] ?>,
 		hostid: <?= $data['hostid'] ?>,
 		valuemapid: <?= $data['valuemapid'] ?>,
-		value: form_data['value']
+		value: form_data['value'],
+		not_supported: form_data['not_supported']
 	});
 
 	<?php if ($data['show_prev']): ?>
@@ -275,11 +284,13 @@ function itemCompleteTest(overlay) {
 		success: function(ret) {
 			overlay.$dialogue.find('.msg-bad, .msg-good, .msg-warning').remove();
 
-			if (typeof ret.messages !== 'undefined') {
-				jQuery($body).prepend(ret.messages);
+			if ('error' in ret) {
+				const message_box = makeMessageBox('bad', ret.error.messages, ret.error.title);
+
+				jQuery($body).prepend(message_box);
 			}
 
-			processItemPreprocessingTestResults(ret.steps);
+			processItemPreprocessingTestResults(ret.steps ?? []);
 
 			<?php if ($data['show_prev']): ?>
 				if (typeof ret.prev_value !== 'undefined') {
@@ -303,6 +314,7 @@ function itemCompleteTest(overlay) {
 				}
 
 				$result_row = jQuery('<div>', {'class': '<?= ZBX_STYLE_TABLE_FORMS_SEPARATOR ?>'})
+					.css({whiteSpace: 'normal'})
 					.append(jQuery('<div>').append(ret.final.action, $result))
 					.css({display: 'block', width: '675px'});
 
@@ -319,10 +331,11 @@ function itemCompleteTest(overlay) {
 					);
 				}
 
-				jQuery('#final-result')
+				jQuery('.js-final-result')
 					.show()
-					.find('.table-forms-td-right')
-					.append($result_row);
+					.next()
+					.append($result_row)
+					.show();
 			}
 		},
 		dataType: 'json',
@@ -390,6 +403,8 @@ function saveItemTestInputs() {
 			value: jQuery('#value').multilineInput('value'),
 			eol: jQuery('#eol').find(':checked').val()
 		},
+		form_data = $form.serializeJSON(),
+		interface = (typeof form_data['interface'] !== 'undefined') ? form_data['interface'] : null,
 		macros = {};
 
 	<?php if ($data['is_item_testable']): ?>
@@ -398,7 +413,8 @@ function saveItemTestInputs() {
 			proxy_hostid: jQuery('#proxy_hostid', $form).val(),
 			interfaceid: <?= $data['interfaceid'] ?> || 0,
 			address: jQuery('#interface_address', $form).val(),
-			port: jQuery('#interface_port', $form).val()
+			port: jQuery('#interface_port', $form).val(),
+			interface_details: (interface && 'details' in interface) ? interface['details'] : null
 		});
 	<?php endif ?>
 
@@ -427,7 +443,7 @@ function saveItemTestInputs() {
 }
 
 jQuery(document).ready(function($) {
-	$('#final-result').hide();
+	$('.js-final-result').hide().next().hide();
 
 	<?php if ($data['show_prev']): ?>
 		jQuery('#upd_last').val(Math.ceil(+new Date() / 1000));
@@ -437,7 +453,6 @@ jQuery(document).ready(function($) {
 		placeholder: <?= json_encode(_('value')) ?>,
 		value: <?= json_encode($data['value']) ?>,
 		monospace_font: false,
-		maxlength: 65535,
 		autofocus: true,
 		readonly: false,
 		grow: 'auto',
@@ -448,22 +463,32 @@ jQuery(document).ready(function($) {
 		placeholder: <?= $data['show_prev'] ? json_encode(_('value')) : '""' ?>,
 		value: <?= json_encode($data['prev_value']) ?>,
 		monospace_font: false,
-		maxlength: 65535,
 		disabled: <?= $data['show_prev'] ? 'false' : 'true' ?>,
 		grow: 'auto',
 		rows: 0
 	});
 
 	<?php if ($data['is_item_testable']): ?>
-		$('#get_value').on('change', function() {
-			var $rows = $('#host_address_row, #proxy_hostid_row, #get_value_row, #empty_row_1, #empty_row_2,'
-					+ ' #host_port_row'
-				),
-				$form = $('#preprocessing-test-form'),
-				$submit_btn = overlays_stack.getById('item-test').$btn_submit;
+		$('#not_supported').on('change', function() {
+			var $form = $('#preprocessing-test-form');
 
 			if ($(this).is(':checked')) {
 				$('#value', $form).multilineInput('setReadOnly');
+			}
+			else {
+				$('#value', $form).multilineInput('unsetReadOnly');
+			}
+		});
+
+		$('#get_value').on('change', function() {
+			var $rows = $('.js-host-address-row, .js-proxy-hostid-row, .js-get-value-row, [class*=js-popup-row-snmp]'),
+				$form = $('#preprocessing-test-form'),
+				$submit_btn = overlays_stack.getById('item-test').$btn_submit,
+				$not_supported = $('#not_supported', $form);
+
+			if ($(this).is(':checked')) {
+				$('#value', $form).multilineInput('setReadOnly');
+				$not_supported.prop('disabled', true);
 
 				<?php if ($data['show_prev']): ?>
 					$('#prev_value', $form).multilineInput('setReadOnly');
@@ -484,9 +509,56 @@ jQuery(document).ready(function($) {
 
 				$submit_btn.html('<?= _('Get value and test') ?>');
 				$rows.show();
+
+				<?php if ($data['show_snmp_form']): ?>
+					$('#interface_details_version').on('change', function (e) {
+						$(`.js-popup-row-snmp-community, .js-popup-row-snmp-max-repetition,
+							.js-popup-row-snmpv3-contextname, .js-popup-row-snmpv3-securityname,
+							.js-popup-row-snmpv3-securitylevel, .js-popup-row-snmpv3-authprotocol,
+							.js-popup-row-snmpv3-authpassphrase, .js-popup-row-snmpv3-privprotocol,
+							.js-popup-row-snmpv3-privpassphrase`).hide();
+
+						switch (e.target.value) {
+							case '<?= SNMP_V1 ?>':
+								$('#interface_details_securitylevel').off('change');
+								$('.js-popup-row-snmp-community').show();
+								break;
+							case '<?= SNMP_V2C ?>':
+								$('#interface_details_securitylevel').off('change');
+								$('.js-popup-row-snmp-community').show();
+								$('.js-popup-row-snmp-max-repetition').show();
+								break;
+							case '<?= SNMP_V3 ?>':
+								$(`.js-popup-row-snmpv3-contextname, .js-popup-row-snmpv3-securityname,
+									.js-popup-row-snmpv3-securitylevel, .js-popup-row-snmp-max-repetition`).show();
+
+								$('#interface_details_securitylevel').on('change', function (e) {
+									$(`.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase,
+										.js-popup-row-snmpv3-privprotocol, .js-popup-row-snmpv3-privpassphrase`).hide();
+									switch (e.target.value) {
+										case '<?= ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV ?>':
+											$(`.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase`)
+												.show();
+											break;
+										case '<?= ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV ?>':
+											$(`.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase,
+												.js-popup-row-snmpv3-privprotocol, .js-popup-row-snmpv3-privpassphrase`)
+												.show();
+											break;
+									}
+
+									overlays_stack.end().centerDialog();
+								}).trigger('change');
+								break;
+						}
+
+						overlays_stack.end().centerDialog();
+					}).trigger('change');
+				<?php endif ?>
 			}
 			else {
-				$('#value', $form).multilineInput('unsetReadOnly');
+				!$not_supported.is(':checked') && $('#value', $form).multilineInput('unsetReadOnly');
+				$not_supported.prop('disabled', false);
 
 				<?php if ($data['show_prev']): ?>
 					$('#prev_value', $form).multilineInput('unsetReadOnly');
