@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ if ($data['can_create_hosts']) {
 
 $nav_items->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode]));
 
-$widget = (new CWidget())
+$html_page = (new CHtmlPage())
 	->setTitle(_('Hosts'))
 	->setWebLayoutMode($web_layout_mode)
 	->setDocUrl(CDocHelper::getUrl(CDocHelper::MONITORING_HOST_VIEW))
@@ -66,18 +66,19 @@ if ($web_layout_mode == ZBX_LAYOUT_NORMAL) {
 
 	// Set javascript options for tab filter initialization in monitoring.host.view.js.php file.
 	$data['filter_options'] = $filter->options;
-	$widget->addItem($filter);
+	$html_page->addItem($filter);
 }
 else {
 	$data['filter_options'] = null;
 }
 
-$widget->addItem((new CForm())
-	->setName('host_view')
-	->addClass('is-loading')
-);
-
-$widget->show();
+$html_page
+	->addItem(
+		(new CForm())
+			->setName('host_view')
+			->addClass('is-loading')
+	)
+	->show();
 
 (new CScriptTag('
 	view.init('.json_encode([
