@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -91,7 +91,7 @@ else {
 
 $output = [
 	'header' => ($data['hostid'] == 0) ? _('New host') : _('Host'),
-	'doc_url' => CDocHelper::getUrl(CDocHelper::CONFIGURATION_HOST_EDIT),
+	'doc_url' => CDocHelper::getUrl(CDocHelper::DATA_COLLECTION_HOST_EDIT),
 	'body' => (new CPartial('configuration.host.edit.html', $data))->getOutput(),
 	'script_inline' => getPagePostJs().
 		$this->readJsFile('popup.host.edit.js.php').
@@ -100,7 +100,7 @@ $output = [
 			'form_name' => $data['form_name'],
 			'host_interfaces' => $data['host']['interfaces'],
 			'host_is_discovered' => ($data['host']['flags'] == ZBX_FLAG_DISCOVERY_CREATED),
-			'warning' => $data['warning']
+			'warnings' => $data['warnings']
 		]).');',
 	'buttons' => $buttons
 ];
