@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,15 +31,16 @@ $output = [
 $options = $data['options'];
 
 $http_popup_form = (new CForm())
-	->cleanItems()
 	->setId('http_step')
 	->addVar('no', $options['no'])
 	->addVar('httpstepid', $options['httpstepid'])
 	->addItem((new CVar('templated', $options['templated']))->removeId())
 	->addVar('old_name', $options['old_name'])
 	->addVar('steps_names', $options['steps_names'])
-	->addVar('action', 'popup.httpstep')
-	->addItem((new CInput('submit', 'submit'))->addStyle('display: none;'));
+	->addVar('action', 'popup.httpstep');
+
+// Enable form submitting on Enter.
+$http_popup_form->addItem((new CSubmitButton(null))->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
 
 $http_popup_form_list = (new CFormList())
 	->addRow(
@@ -108,8 +109,11 @@ $http_popup_form_list->addRow(_('Post fields'),
 		'post-fields-row'
 );
 
-$http_popup_form_list->addRow(_('Raw post'), (new CTextArea('posts', $options['posts']))
-	->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH), 'post-raw-row'
+$http_popup_form_list->addRow(_('Raw post'),
+	(new CTextArea('posts', $options['posts']))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->disableSpellcheck(),
+	'post-raw-row'
 );
 
 $http_popup_form_list->addRow(_('Variables'),
