@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -48,13 +48,11 @@ function drawChangeType(string $name, array $entity_types): CTag {
 	return (new CTag('li', true))
 		->addItem((new CDiv())
 			->addClass(ZBX_STYLE_TOC_ROW)
-			->addItem((new CTag('button', true))
+			->addItem((new CSimpleButton([
+				(new CSpan())->addClass(ZBX_STYLE_ARROW_DOWN), $name
+			]))
 				->addClass(ZBX_STYLE_TOC_ITEM)
 				->addClass(ZBX_STYLE_TOC_ARROW)
-				->addItem((new CSpan())
-					->addClass(ZBX_STYLE_ARROW_DOWN)
-				)
-				->addItem($name)
 			)
 		)
 		->addItem($entity_types_list);
@@ -71,13 +69,11 @@ function drawEntityType(string $name, array $entities): CTag {
 	return (new CTag('li', true))
 		->addItem((new CDiv())
 			->addClass(ZBX_STYLE_TOC_ROW)
-			->addItem((new CTag('button', true))
+			->addItem((new CSimpleButton([
+				(new CSpan())->addClass(ZBX_STYLE_ARROW_DOWN), $name
+			]))
 				->addClass(ZBX_STYLE_TOC_ITEM)
 				->addClass(ZBX_STYLE_TOC_ARROW)
-				->addItem((new CSpan())
-					->addClass(ZBX_STYLE_ARROW_DOWN)
-				)
-				->addItem($name)
 			)
 		)
 		->addItem($entities_list);
@@ -150,11 +146,11 @@ else {
 	if ($data['diff']) {
 		$buttons[] = [
 			'title' => _('Import'),
-			'class' => '',
+			'class' => 'js-import',
 			'keepOpen' => true,
 			'isSubmit' => true,
 			'focused' => true,
-			'action' => 'submitImportComparePopup(overlay);'
+			'action' => 'popup_import_compare.submitImportComparePopup();'
 		];
 	}
 
@@ -174,9 +170,11 @@ else {
 				->toString()
 			: (new CForm())
 				->addClass('import-compare')
-				->addVar('import_overlayid', $data['import_overlayid'])
 				->addItem(drawToc($data['diff_toc']))
 				->addItem(drawDiff($data['diff']))
+				->addItem(
+					(new CScriptTag('popup_import_compare.init();'))->setOnDocumentReady()
+				)
 				->toString(),
 		'buttons' => $buttons,
 		'no_changes' => !$data['diff']
