@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,12 +25,13 @@
  */
 
 $form = (new CForm('post'))
+	->addItem((new CVar(CCsrfTokenHelper::CSRF_TOKEN_NAME, CCsrfTokenHelper::get('service')))->removeId())
 	->setId('service-form')
 	->setName('service_form')
 	->addItem(getMessages());
 
 // Enable form submitting on Enter.
-$form->addItem((new CInput('submit'))->addStyle('display: none;'));
+$form->addItem((new CSubmitButton(null))->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
 
 // Service tab.
 
@@ -73,7 +74,7 @@ $service_tab = (new CFormGrid())
 								->addClass('element-table-add')
 						))
 					),
-				(new CScriptTemplate('problem-tag-row-tmpl'))
+				(new CTemplateTag('problem-tag-row-tmpl'))
 					->addItem(
 						(new CRow([
 							(new CTextBox('problem_tags[#{rowNum}][tag]', '#{tag}', false,
@@ -115,12 +116,11 @@ $service_tab = (new CFormGrid())
 	->addItem([
 		new CLabel([
 			_('Status calculation rule'),
-			(new CSpan([
-				' ',
+			(new CSpan(
 				makeWarningIcon(
 					_('Status calculation rule and additional rules are only applicable if child services exist.')
 				)
-			]))
+			))
 				->setId('algorithm-not-applicable-warning')
 				->addStyle($data['form']['children'] ? 'display: none' : '')
 		], 'algorithm_focusable'),
@@ -252,7 +252,7 @@ $tags_tab = (new CFormGrid())
 					renderTagTable($data['form']['tags'])
 						->addClass('tags-table')
 						->setHeader((new CRowHeader([_('Name'), _('Value'), _('Action')]))->addClass(ZBX_STYLE_GREY)),
-					(new CScriptTemplate('tag-row-tmpl'))
+					(new CTemplateTag('tag-row-tmpl'))
 						->addItem(renderTagTableRow('#{rowNum}', '', '', ZBX_TAG_MANUAL, ['add_post_js' => false]))
 				])
 		)
