@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 require_once __DIR__.'/../../include/blocks.inc.php';
 
-(new CWidget())
+(new CHtmlPage())
 	->setTitle(_('System information'))
 	->setDocUrl(CDocHelper::getUrl(CDocHelper::REPORT_STATUS))
 	->addItem(
