@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ $fields = [
 	'to' =>				[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,		null,				null],
 	'profileIdx' =>		[T_ZBX_STR,			O_OPT, null,		null,				null],
 	'profileIdx2' =>	[T_ZBX_STR,			O_OPT, null,		null,				null],
-	'httptestid' =>		[T_ZBX_INT,			O_OPT, P_NZERO,	null,				null],
+	'httptestid' =>		[T_ZBX_INT,			O_OPT, P_NZERO,		null,				null],
 	'http_item_type' =>	[T_ZBX_INT,			O_OPT, null,		null,				null],
 	'name' =>			[T_ZBX_STR,			O_OPT, null,		null,				null],
 	'width' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_WIDTH_MIN, 65535),	null],
@@ -50,12 +50,13 @@ $fields = [
 	'percent_left' =>	[T_ZBX_DBL,			O_OPT, null,		BETWEEN_DBL(0, 100, 4),	null],
 	'percent_right' =>	[T_ZBX_DBL,			O_OPT, null,		BETWEEN_DBL(0, 100, 4),	null],
 	'outer' =>			[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
-	'items' =>			[T_ZBX_STR,			O_OPT, null,		null,				null],
-	'i' =>				[T_ZBX_STR,			O_OPT, null,		null,				null],
+	'items' =>			[T_ZBX_STR,			O_OPT, P_ONLY_TD_ARRAY,	null,			null],
+	'i' =>				[T_ZBX_STR,			O_OPT, P_ONLY_ARRAY,	null,			null],
 	'onlyHeight' =>		[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
 	'widget_view' =>	[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null]
 ];
 if (!check_fields($fields)) {
+	session_write_close();
 	exit();
 }
 validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
@@ -102,10 +103,10 @@ if ($httptestid = getRequest('httptestid', false)) {
 }
 elseif (hasRequest('i') || hasRequest('items')) {
 	if (hasRequest('i')) {
-		$items = array_map('expandShortGraphItem', getRequest('i', []));
+		$items = array_map('expandShortGraphItem', getRequest('i'));
 	}
 	else {
-		$items = getRequest('items', []);
+		$items = getRequest('items');
 	}
 
 	CArrayHelper::sort($items, ['sortorder']);
@@ -143,7 +144,8 @@ elseif (hasRequest('i') || hasRequest('items')) {
 }
 else {
 	show_error_message(_('No items defined.'));
-	exit;
+	session_write_close();
+	exit();
 }
 
 /*
@@ -194,4 +196,4 @@ else {
 	$graph->draw();
 }
 
-require_once dirname(__FILE__).'/include/page_footer.php';
+require_once __DIR__.'/include/page_footer.php';
