@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,44 +32,44 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'new_httpstep'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM,	null,				null],
-	'group_httptestid'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,				null],
+	'new_httpstep'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM,	null,	null],
+	'group_httptestid'	=> [T_ZBX_INT, O_OPT, P_ONLY_ARRAY,	DB_ID,	null],
 	// form
-	'hostid'          => [T_ZBX_INT, O_OPT, P_SYS, DB_ID.NOT_ZERO,          'isset({form}) || isset({add}) || isset({update})'],
-	'httptestid'      => [T_ZBX_INT, O_NO,  P_SYS, DB_ID,                   'isset({form}) && {form} == "update"'],
-	'name'            => [T_ZBX_STR, O_OPT, null,  NOT_EMPTY,               'isset({add}) || isset({update})', _('Name')],
-	'delay'           => [T_ZBX_STR, O_OPT, null,  null,					'isset({add}) || isset({update})'],
-	'retries'         => [T_ZBX_INT, O_OPT, null,  BETWEEN(1, 10),          'isset({add}) || isset({update})',
+	'hostid'          => [T_ZBX_INT, O_OPT, P_SYS,	DB_ID.NOT_ZERO,	'isset({form}) || isset({add}) || isset({update})'],
+	'httptestid'      => [T_ZBX_INT, O_NO,  P_SYS,	DB_ID,			'isset({form}) && {form} == "update"'],
+	'name'            => [T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'isset({add}) || isset({update})', _('Name')],
+	'delay'           => [T_ZBX_STR, O_OPT, null,	null,			'isset({add}) || isset({update})'],
+	'retries'         => [T_ZBX_INT, O_OPT, null,	BETWEEN(1, 10),	'isset({add}) || isset({update})',
 		_('Attempts')
 	],
-	'status'          => [T_ZBX_STR, O_OPT, null,  null,                    null],
-	'agent'           => [T_ZBX_STR, O_OPT, null, null,                     'isset({add}) || isset({update})'],
-	'agent_other'     => [T_ZBX_STR, O_OPT, null, null,
-		'(isset({add}) || isset({update})) && {agent} == '.ZBX_AGENT_OTHER
+	'status'          => [T_ZBX_STR, O_OPT, null,	null,	null],
+	'agent'           => [T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
+	'agent_other'     => [T_ZBX_STR, O_OPT, null,	null,
+		'(isset({add}) || isset({update})) && isset({agent}) && {agent} == '.ZBX_AGENT_OTHER
 	],
-	'pairs'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    null],
-	'steps'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    'isset({add}) || isset({update})', _('Steps')],
-	'authentication' =>		[T_ZBX_INT, O_OPT, null,
-								IN([HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS,
-									HTTPTEST_AUTH_DIGEST
+	'pairs'           => [T_ZBX_STR, O_OPT, P_NO_TRIM|P_ONLY_TD_ARRAY,	null,	null],
+	'steps'           => [null,      O_OPT, P_NO_TRIM|P_ONLY_TD_ARRAY,	null,	'isset({add}) || isset({update})', _('Steps')],
+	'authentication'  => [T_ZBX_INT, O_OPT, null,
+								IN([ZBX_HTTP_AUTH_NONE, ZBX_HTTP_AUTH_BASIC, ZBX_HTTP_AUTH_NTLM, ZBX_HTTP_AUTH_KERBEROS,
+									ZBX_HTTP_AUTH_DIGEST
 								]),
 								'isset({add}) || isset({update})'
 							],
 	'http_user' =>			[T_ZBX_STR, O_OPT, null,  null,
 								'(isset({add}) || isset({update})) && isset({authentication})'.
-									' && ({authentication}=='.HTTPTEST_AUTH_BASIC.
-										' || {authentication}=='.HTTPTEST_AUTH_NTLM.
-										' || {authentication}=='.HTTPTEST_AUTH_KERBEROS.
-										' || {authentication} == '.HTTPTEST_AUTH_DIGEST.
+									' && ({authentication} == '.ZBX_HTTP_AUTH_BASIC.
+										' || {authentication} == '.ZBX_HTTP_AUTH_NTLM.
+										' || {authentication} == '.ZBX_HTTP_AUTH_KERBEROS.
+										' || {authentication} == '.ZBX_HTTP_AUTH_DIGEST.
 									')',
 								_('User')
 							],
 	'http_password' =>		[T_ZBX_STR, O_OPT, P_NO_TRIM, null,
 								'(isset({add}) || isset({update})) && isset({authentication})'.
-									' && ({authentication}=='.HTTPTEST_AUTH_BASIC.
-										' || {authentication}=='.HTTPTEST_AUTH_NTLM.
-										' || {authentication}=='.HTTPTEST_AUTH_KERBEROS.
-										' || {authentication} == '.HTTPTEST_AUTH_DIGEST.
+									' && ({authentication} == '.ZBX_HTTP_AUTH_BASIC.
+										' || {authentication} == '.ZBX_HTTP_AUTH_NTLM.
+										' || {authentication} == '.ZBX_HTTP_AUTH_KERBEROS.
+										' || {authentication} == '.ZBX_HTTP_AUTH_DIGEST.
 									')',
 								_('Password')
 							],
@@ -78,22 +78,22 @@ $fields = [
 	'templated'			=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'verify_host'		=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'verify_peer'		=> [T_ZBX_STR, O_OPT, null,	null,				null],
-	'ssl_cert_file'		=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
-	'ssl_key_file'		=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
-	'ssl_key_password'	=> [T_ZBX_STR, O_OPT, P_NO_TRIM, null,				'isset({add}) || isset({update})'],
-	'context' =>			[T_ZBX_STR, O_MAND, P_SYS,	IN('"host", "template"'),	null],
-	'tags'				=> [T_ZBX_STR, O_OPT, null,	null,				null],
-	'show_inherited_tags' => [T_ZBX_INT, O_OPT, null, IN([0,1]),		null],
+	'ssl_cert_file'		=> [T_ZBX_STR, O_OPT, null, null,				'isset({add}) || isset({update})'],
+	'ssl_key_file'		=> [T_ZBX_STR, O_OPT, null, null,				'isset({add}) || isset({update})'],
+	'ssl_key_password'	=> [T_ZBX_STR, O_OPT, P_NO_TRIM, null,			'isset({add}) || isset({update})'],
+	'context'			=> [T_ZBX_STR, O_MAND, P_SYS,	IN('"host", "template"'),	null],
+	'tags'				=> [T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,				null],
+	'show_inherited_tags' => [T_ZBX_INT, O_OPT, null,	IN([0,1]),					null],
 	// filter
 	'filter_set' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_rst' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_status' =>		[T_ZBX_INT, O_OPT, null,
 		IN([-1, HTTPTEST_STATUS_ACTIVE, HTTPTEST_STATUS_DISABLED]), null
 	],
-	'filter_groupids'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,			null],
-	'filter_hostids'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,			null],
+	'filter_groupids'	=> [T_ZBX_INT, O_OPT, P_ONLY_ARRAY,	DB_ID,		null],
+	'filter_hostids'	=> [T_ZBX_INT, O_OPT, P_ONLY_ARRAY,	DB_ID,		null],
 	'filter_evaltype'	=> [T_ZBX_INT, O_OPT, null, IN([TAG_EVAL_TYPE_AND_OR, TAG_EVAL_TYPE_OR]), null],
-	'filter_tags'		=> [T_ZBX_STR, O_OPT, null,	null,			null],
+	'filter_tags'		=> [T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,	null],
 	// actions
 	'action'			=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 								IN('"httptest.massclearhistory","httptest.massdelete","httptest.massdisable",'.
@@ -101,14 +101,14 @@ $fields = [
 								),
 								null
 							],
-	'clone'				=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
-	'del_history'		=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
-	'add'				=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
-	'update'			=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
-	'delete'			=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
-	'cancel'			=> [T_ZBX_STR, O_OPT, P_SYS,	null,				null],
-	'form'				=> [T_ZBX_STR, O_OPT, P_SYS,	null,				null],
-	'form_refresh'		=> [T_ZBX_INT, O_OPT, null,	null,				null],
+	'clone'				=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'del_history'		=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add'				=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'update'			=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'delete'			=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel'			=> [T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'form'				=> [T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'form_refresh'		=> [T_ZBX_INT, O_OPT, P_SYS,	null,		null],
 	// sort and sortorder
 	'sort'				=> [T_ZBX_STR, O_OPT, P_SYS, IN('"hostname","name","status"'),				null],
 	'sortorder'			=> [T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
@@ -254,10 +254,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'variables' => [],
 			'http_proxy' => $_REQUEST['http_proxy'],
 			'steps' => $steps,
-			'http_user' => ($_REQUEST['authentication'] == HTTPTEST_AUTH_NONE) ? '' : $_REQUEST['http_user'],
-			'http_password' => ($_REQUEST['authentication'] == HTTPTEST_AUTH_NONE) ? '' : $_REQUEST['http_password'],
-			'verify_peer' => getRequest('verify_peer', HTTPTEST_VERIFY_PEER_OFF),
-			'verify_host' => getRequest('verify_host', HTTPTEST_VERIFY_HOST_OFF),
+			'http_user' => $_REQUEST['authentication'] == ZBX_HTTP_AUTH_NONE ? '' : $_REQUEST['http_user'],
+			'http_password' => $_REQUEST['authentication'] == ZBX_HTTP_AUTH_NONE ? '' : $_REQUEST['http_password'],
+			'verify_peer' => getRequest('verify_peer', ZBX_HTTP_VERIFY_PEER_OFF),
+			'verify_host' => getRequest('verify_host', ZBX_HTTP_VERIFY_HOST_OFF),
 			'ssl_cert_file' => getRequest('ssl_cert_file'),
 			'ssl_key_file' => getRequest('ssl_key_file'),
 			'ssl_key_password' => getRequest('ssl_key_password'),
@@ -460,7 +460,7 @@ if (isset($_REQUEST['form'])) {
 		'hostid' => getRequest('hostid', 0),
 		'httptestid' => getRequest('httptestid'),
 		'form' => getRequest('form'),
-		'form_refresh' => getRequest('form_refresh'),
+		'form_refresh' => getRequest('form_refresh', 0),
 		'templates' => [],
 		'context' => getRequest('context'),
 		'show_inherited_tags' => getRequest('show_inherited_tags', 0)
@@ -499,8 +499,7 @@ if (isset($_REQUEST['form'])) {
 		$data['retries'] = $db_httptest['retries'];
 		$data['status'] = $db_httptest['status'];
 		$data['templates'] = makeHttpTestTemplatesHtml($db_httptest['httptestid'],
-			getHttpTestParentTemplates($db_httptests), CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES),
-			$data['context']
+			getHttpTestParentTemplates($db_httptests), CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES)
 		);
 
 		$data['agent'] = ZBX_AGENT_OTHER;
@@ -609,7 +608,7 @@ if (isset($_REQUEST['form'])) {
 			}
 		}
 
-		$data['authentication'] = getRequest('authentication', HTTPTEST_AUTH_NONE);
+		$data['authentication'] = getRequest('authentication', ZBX_HTTP_AUTH_NONE);
 		$data['http_user'] = getRequest('http_user', '');
 		$data['http_password'] = getRequest('http_password', '');
 		$data['http_proxy'] = getRequest('http_proxy', '');
@@ -740,12 +739,15 @@ else {
 
 	$filter = [
 		'status' => CProfile::get($prefix.'httpconf.filter_status', -1),
-		'groups' => CProfile::getArray($prefix.'httpconf.filter_groupids', null),
-		'hosts' => CProfile::getArray($prefix.'httpconf.filter_hostids', null),
+		'groups' => [],
+		'hosts' => [],
 		'evaltype' => CProfile::get($prefix.'httpconf.filter.evaltype', TAG_EVAL_TYPE_AND_OR),
 		'tags' => []
 	];
 
+	$filter_groupids = CProfile::getArray($prefix.'httpconf.filter_groupids', []);
+	$filter_hostids = CProfile::getArray($prefix.'httpconf.filter_hostids', []);
+
 	foreach (CProfile::getArray($prefix.'httpconf.filter.tags.tag', []) as $i => $tag) {
 		$filter['tags'][] = [
 			'tag' => $tag,
@@ -755,35 +757,23 @@ else {
 	}
 
 	// Get host groups.
-	$filter['groups'] = $filter['groups']
-		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
-			'output' => ['groupid', 'name'],
-			'groupids' => $filter['groups'],
-			'editable' => true,
-			'preservekeys' => true
-		]), ['groupid' => 'id'])
-		: [];
-
-	$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
-	if ($filter_groupids) {
-		$filter_groupids = getSubGroups($filter_groupids);
-	}
+	$filter_groupids = getSubGroups($filter_groupids, $filter['groups'], ['editable' => true], $data['context']);
 
 	if ($data['context'] === 'host') {
-		$filter['hosts'] = $filter['hosts']
+		$filter['hosts'] = $filter_hostids
 			? CArrayHelper::renameObjectsKeys(API::Host()->get([
 				'output' => ['hostid', 'name'],
-				'hostids' => $filter['hosts'],
+				'hostids' => $filter_hostids,
 				'editable' => true,
 				'preservekeys' => true
 			]), ['hostid' => 'id'])
 			: [];
 	}
 	else {
-		$filter['hosts'] = $filter['hosts']
+		$filter['hosts'] = $filter_hostids
 			? CArrayHelper::renameObjectsKeys(API::Template()->get([
 				'output' => ['templateid', 'name'],
-				'templateids' => $filter['hosts'],
+				'templateids' => $filter_hostids,
 				'editable' => true,
 				'preservekeys' => true
 			]), ['templateid' => 'id'])
@@ -808,7 +798,7 @@ else {
 		'output' => ['httptestid', $sortField],
 		'selectTags' => ['tag', 'value'],
 		'hostids' => $filter['hosts'] ? array_keys($filter['hosts']) : null,
-		'groupids' => $filter_groupids,
+		'groupids' => $filter_groupids ? $filter_groupids : null,
 		'tags' => $data['filter']['tags'],
 		'evaltype' => $data['filter']['evaltype'],
 		'templated' => ($data['context'] === 'template'),
