@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,11 @@
  */
 class CEvent extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'acknowledge' => ['min_user_type' => USER_TYPE_ZABBIX_USER]
+	];
+
 	protected $tableName = 'events';
 	protected $tableAlias = 'e';
 	protected $sortColumns = ['eventid', 'objectid', 'clock'];
@@ -57,7 +62,6 @@ class CEvent extends CApiService {
 	 * @param array $options['hostids']
 	 * @param array $options['groupids']
 	 * @param array $options['eventids']
-	 * @param array $options['applicationids']
 	 * @param array $options['status']
 	 * @param bool  $options['editable']
 	 * @param array $options['count']
@@ -72,7 +76,6 @@ class CEvent extends CApiService {
 			'eventids'					=> null,
 			'groupids'					=> null,
 			'hostids'					=> null,
-			'applicationids'			=> null,
 			'objectids'					=> null,
 
 			'editable'					=> false,
@@ -90,6 +93,7 @@ class CEvent extends CApiService {
 			'problem_time_till'			=> null,
 			'acknowledged'				=> null,
 			'suppressed'				=> null,
+			'symptom'					=> null,
 			'evaltype'					=> TAG_EVAL_TYPE_AND_OR,
 			'tags'						=> null,
 			'filter'					=> null,
@@ -121,7 +125,8 @@ class CEvent extends CApiService {
 			zbx_value2array($options['value']);
 		}
 
-		if ($options['source'] == EVENT_SOURCE_TRIGGERS && $options['object'] == EVENT_OBJECT_TRIGGER) {
+		if (($options['source'] == EVENT_SOURCE_TRIGGERS && $options['object'] == EVENT_OBJECT_TRIGGER)
+				|| ($options['source'] == EVENT_SOURCE_SERVICE && $options['object'] == EVENT_OBJECT_SERVICE)) {
 			if ($options['value'] === null) {
 				$options['value'] = ($options['problem_time_from'] !== null && $options['problem_time_till'] !== null)
 					? [TRIGGER_VALUE_TRUE]
@@ -282,7 +287,8 @@ class CEvent extends CApiService {
 			}
 		}
 
-		if ($options['source'] == EVENT_SOURCE_TRIGGERS && $options['object'] == EVENT_OBJECT_TRIGGER) {
+		if (($options['source'] == EVENT_SOURCE_TRIGGERS && $options['object'] == EVENT_OBJECT_TRIGGER)
+				|| ($options['source'] == EVENT_SOURCE_SERVICE && $options['object'] == EVENT_OBJECT_SERVICE)) {
 			if ($options['problem_time_from'] !== null && $options['problem_time_till'] !== null) {
 				if ($options['value'][0] == TRIGGER_VALUE_TRUE) {
 					$sqlParts['where'][] =
@@ -322,9 +328,8 @@ class CEvent extends CApiService {
 		}
 
 		// objectids
-		if ($options['objectids'] !== null
-				&& in_array($options['object'], [EVENT_OBJECT_TRIGGER, EVENT_OBJECT_ITEM, EVENT_OBJECT_LLDRULE])) {
-
+		if ($options['objectids'] !== null && in_array($options['object'], [EVENT_OBJECT_TRIGGER, EVENT_OBJECT_ITEM,
+				EVENT_OBJECT_LLDRULE, EVENT_OBJECT_SERVICE])) {
 			zbx_value2array($options['objectids']);
 			$sqlParts['where'][] = dbConditionInt('e.objectid', $options['objectids']);
 
@@ -377,31 +382,10 @@ class CEvent extends CApiService {
 			}
 		}
 
-		// applicationids
-		if ($options['applicationids'] !== null) {
-			zbx_value2array($options['applicationids']);
-
-			// triggers
-			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
-				$sqlParts['from']['f'] = 'functions f';
-				$sqlParts['from']['ia'] = 'items_applications ia';
-				$sqlParts['where']['e-f'] = 'e.objectid=f.triggerid';
-				$sqlParts['where']['f-ia'] = 'f.itemid=ia.itemid';
-				$sqlParts['where']['ia'] = dbConditionInt('ia.applicationid', $options['applicationids']);
-			}
-			// items
-			elseif ($options['object'] == EVENT_OBJECT_ITEM) {
-				$sqlParts['from']['ia'] = 'items_applications ia';
-				$sqlParts['where']['e-ia'] = 'e.objectid=ia.itemid';
-				$sqlParts['where']['ia'] = dbConditionInt('ia.applicationid', $options['applicationids']);
-			}
-			// ignore this filter for lld rules
-		}
-
 		// severities
 		if ($options['severities'] !== null) {
 			// triggers
-			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
+			if ($options['object'] == EVENT_OBJECT_TRIGGER || $options['object'] == EVENT_OBJECT_SERVICE) {
 				zbx_value2array($options['severities']);
 				$sqlParts['where'][] = dbConditionInt('e.severity', $options['severities']);
 			}
@@ -417,11 +401,21 @@ class CEvent extends CApiService {
 		// suppressed
 		if ($options['suppressed'] !== null) {
 			$sqlParts['where'][] = (!$options['suppressed'] ? 'NOT ' : '').
-					'EXISTS ('.
-						'SELECT NULL'.
-						' FROM event_suppress es'.
-						' WHERE es.eventid=e.eventid'.
-					')';
+				'EXISTS ('.
+					'SELECT NULL'.
+					' FROM event_suppress es'.
+					' WHERE es.eventid=e.eventid'.
+				')';
+		}
+
+		// symptom
+		if ($options['symptom'] !== null) {
+			$sqlParts['where'][] = (!$options['symptom'] ? 'NOT ' : '').
+				'EXISTS ('.
+					'SELECT NULL'.
+					' FROM event_symptom es'.
+					' WHERE es.eventid=e.eventid'.
+				')';
 		}
 
 		// tags
@@ -464,6 +458,15 @@ class CEvent extends CApiService {
 		// filter
 		if (is_array($options['filter'])) {
 			$this->dbFilter('events e', $options, $sqlParts);
+
+			// Filter symptom events for given cause.
+			if (array_key_exists('cause_eventid', $options['filter']) && $options['filter']['cause_eventid'] !== null) {
+				zbx_value2array($options['filter']['cause_eventid']);
+
+				$sqlParts['from']['event_symptom'] = 'event_symptom es';
+				$sqlParts['where']['ese'] = 'es.eventid=e.eventid';
+				$sqlParts['where']['es'] = dbConditionId('es.cause_eventid', $options['filter']['cause_eventid']);
+			}
 		}
 
 		// limit
@@ -531,48 +534,76 @@ class CEvent extends CApiService {
 	/**
 	 * Acknowledges the given events and closes them if necessary.
 	 *
-	 * @param array  $data                  And array of operation data.
-	 * @param mixed  $data['eventids']      An event ID or an array of event IDs.
-	 * @param string $data['message']       Message if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
-	 * @param string $data['severity']      New severity level if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
-	 * @param int    $data['action']        Flags of performed operations combined:
-	 *                                       - 0x01 - ZBX_PROBLEM_UPDATE_CLOSE
-	 *                                       - 0x02 - ZBX_PROBLEM_UPDATE_ACKNOWLEDGE
-	 *                                       - 0x04 - ZBX_PROBLEM_UPDATE_MESSAGE
-	 *                                       - 0x08 - ZBX_PROBLEM_UPDATE_SEVERITY
-	 *                                       - 0x10 - ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
+	 * @param array  $data                  	And array of operation data.
+	 * @param mixed  $data['eventids']      	An event ID or an array of event IDs.
+	 * @param string $data['cause_eventid']     Cause event ID. Used if $data['action'] yields 0x100.
+	 * @param string $data['message']      		Message if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
+	 * @param string $data['severity']      	New severity level if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
+	 * @param string $data['suppress_until']	Suppress until time if ZBX_PROBLEM_UPDATE_SUPPRESS flag is passed.
+	 * @param int    $data['action']        	Flags of performed operations combined:
+	 *                                       	 - 0x01  - ZBX_PROBLEM_UPDATE_CLOSE
+	 *                                       	 - 0x02  - ZBX_PROBLEM_UPDATE_ACKNOWLEDGE
+	 *                                       	 - 0x04  - ZBX_PROBLEM_UPDATE_MESSAGE
+	 *                                       	 - 0x08  - ZBX_PROBLEM_UPDATE_SEVERITY
+	 *                                       	 - 0x10  - ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
+	 *                                       	 - 0x20  - ZBX_PROBLEM_UPDATE_SUPPRESS
+	 *                                       	 - 0x40  - ZBX_PROBLEM_UPDATE_UNSUPPRESS
+	 *                                           - 0x80  - ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE
+	 *                                           - 0x100 - ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM
 	 *
 	 * @return array
 	 */
 	public function acknowledge(array $data) {
-		$this->validateAcknowledge($data);
+		$time = time();
+		$this->validateAcknowledge($data, $time);
 
 		$data['eventids'] = zbx_toArray($data['eventids']);
 		$data['eventids'] = array_keys(array_flip($data['eventids']));
 
 		$has_close_action = (($data['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE);
+		$has_suppress_action = (($data['action'] & ZBX_PROBLEM_UPDATE_SUPPRESS) == ZBX_PROBLEM_UPDATE_SUPPRESS);
+		$has_unsuppress_action = (($data['action'] & ZBX_PROBLEM_UPDATE_UNSUPPRESS) == ZBX_PROBLEM_UPDATE_UNSUPPRESS);
+		$has_change_rank_to_symptom_action =
+			(($data['action'] & ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) == ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM);
 
+		// Validation of event permissions has already been done in validateAcknowledge().
 		$events = $this->get([
-			'output' => ['objectid', 'acknowledged', 'severity', 'r_eventid'],
-			'select_acknowledges' => $has_close_action? ['action'] : null,
+			'output' => ['objectid', 'acknowledged', 'severity', 'r_eventid', 'cause_eventid'],
+			// "acknowledges" used in CEvent::isEventClosed().
+			'select_acknowledges' => $has_close_action || $has_suppress_action || $has_unsuppress_action
+				? ['action']
+				: null,
+			// "suppression_data" used in CEvent::isEventSuppressed().
+			'selectSuppressionData' => $has_unsuppress_action ? ['maintenanceid'] : null,
 			'eventids' => $data['eventids'],
 			'source' => EVENT_SOURCE_TRIGGERS,
 			'object' => EVENT_OBJECT_TRIGGER,
 			'value' => TRIGGER_VALUE_TRUE,
-			'preservekeys' => true
+			'preservekeys' => true,
+			'nopermissions' => true
 		]);
 
+		// Get current data of the new cause event and get symptom events of the given cause events.
+		if ($has_change_rank_to_symptom_action) {
+			$update_symptom_eventids = validateEventRankChangeToSymptom($data['eventids'], $data['cause_eventid']);
+		}
+
 		$ack_eventids = [];
 		$unack_eventids = [];
 		$sev_change_eventids = [];
 		$acknowledges = [];
-		$time = time();
+		$suppress_eventids = [];
+		$unsuppress_eventids = [];
+		$tasks_update_event_rank_cause = [];
+		$tasks_update_event_rank_symptom = [];
+		$n = 0;
 
 		foreach ($events as $eventid => $event) {
 			$action = ZBX_PROBLEM_UPDATE_NONE;
 			$old_severity = 0;
 			$new_severity = 0;
 			$message = '';
+			$suppress_until = 0;
 
 			// Perform ZBX_PROBLEM_UPDATE_CLOSE action flag.
 			if ($has_close_action && !$this->isEventClosed($event)) {
@@ -599,7 +630,7 @@ class CEvent extends CApiService {
 				$message = $data['message'];
 			}
 
-			// Perform ZBX_PROBLEM_UPDATE_MESSAGE action flag.
+			// Perform ZBX_PROBLEM_UPDATE_SEVERITY action flag.
 			if (($data['action'] & ZBX_PROBLEM_UPDATE_SEVERITY) == ZBX_PROBLEM_UPDATE_SEVERITY
 					&& $data['severity'] != $event['severity']) {
 				$action |= ZBX_PROBLEM_UPDATE_SEVERITY;
@@ -608,17 +639,49 @@ class CEvent extends CApiService {
 				$sev_change_eventids[] = $eventid;
 			}
 
+			// Perform ZBX_PROBLEM_UPDATE_SUPPRESS action flag.
+			if ($has_suppress_action && !$this->isEventClosed($event)) {
+				$action |= ZBX_PROBLEM_UPDATE_SUPPRESS;
+				$suppress_until = $data['suppress_until'];
+				$suppress_eventids[] = $eventid;
+			}
+
+			// Perform ZBX_PROBLEM_UPDATE_UNSUPPRESS action flag.
+			if ($has_unsuppress_action && $this->isEventSuppressed($event) && !$this->isEventClosed($event)) {
+				$action |= ZBX_PROBLEM_UPDATE_UNSUPPRESS;
+				$unsuppress_eventids[] = $eventid;
+			}
+
+			// Perform ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE action flag.
+			if (($data['action'] & ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE) == ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE
+					&& $event['cause_eventid'] != 0) {
+				$action |= ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE;
+				$tasks_update_event_rank_cause[$n] = ['eventid' => $eventid];
+			}
+
+			// Perform ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM action flag.
+			if ($has_change_rank_to_symptom_action && $update_symptom_eventids
+					&& in_array($eventid, $update_symptom_eventids)) {
+				$action |= ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM;
+				$tasks_update_event_rank_symptom[$n] = [
+					'eventid' => $eventid,
+					'cause_eventid' => $data['cause_eventid']
+				];
+			}
+
 			// For some of selected events action might not be performed, as event is already with given change.
 			if ($action != ZBX_PROBLEM_UPDATE_NONE) {
-				$acknowledges[] = [
+				$acknowledges[$n] = [
 					'userid' => self::$userData['userid'],
 					'eventid' => $eventid,
 					'clock' => $time,
 					'message' => $message,
 					'action' => $action,
 					'old_severity' => $old_severity,
-					'new_severity' => $new_severity
+					'new_severity' => $new_severity,
+					'suppress_until' => $suppress_until
 				];
+				$n++;
 			}
 		}
 
@@ -692,6 +755,59 @@ class CEvent extends CApiService {
 				DB::insertBatch('task_close_problem', $task_close, false);
 			}
 
+			// Create tasks for suppress/unsuppress actions.
+			$tasks = [];
+			$task_suppress = [];
+
+			foreach ($acknowledgeids as $k => $id) {
+				$acknowledgement = $acknowledges[$k];
+
+				// Create tasks to suppress problems manually.
+				if (($acknowledgement['action'] & ZBX_PROBLEM_UPDATE_SUPPRESS) == ZBX_PROBLEM_UPDATE_SUPPRESS) {
+					$tasks[$k] = [
+						'type' => ZBX_TM_TASK_DATA,
+						'status' => ZBX_TM_STATUS_NEW,
+						'clock' => $time
+					];
+
+					$task_suppress[$k] = [
+						'taskid' => $id,
+						'type' => ZBX_TM_DATA_TYPE_TEMP_SUPPRESSION,
+						'data' => json_encode([
+							'eventid' => strval($suppress_eventids[$k]),
+							'action' => ZBX_PROTO_VALUE_SUPPRESSION_SUPPRESS,
+							'userid' => $acknowledgement['userid'],
+							'suppress_until' => $suppress_until
+						])
+					];
+				}
+
+				// Create tasks to unsuppress problems manually.
+				if (($acknowledgement['action'] & ZBX_PROBLEM_UPDATE_UNSUPPRESS) == ZBX_PROBLEM_UPDATE_UNSUPPRESS) {
+					$tasks[$k] = [
+						'type' => ZBX_TM_TASK_DATA,
+						'status' => ZBX_TM_STATUS_NEW,
+						'clock' => $time
+					];
+
+					$task_suppress[$k] = [
+						'taskid' => $id,
+						'type' => ZBX_TM_DATA_TYPE_TEMP_SUPPRESSION,
+						'data' => json_encode([
+							'eventid' => strval($unsuppress_eventids[$k]),
+							'action' => ZBX_PROTO_VALUE_SUPPRESSION_UNSUPPRESS,
+							'userid' => $acknowledgement['userid']
+						])
+					];
+				}
+			}
+
+			if ($tasks) {
+				$taskids = DB::insertBatch('task', $tasks);
+				$task_suppress = array_replace_recursive($task_suppress, zbx_toObject($taskids, 'taskid', true));
+				DB::insertBatch('task_data', $task_suppress, false);
+			}
+
 			// Create tasks to perform server-side acknowledgement operations.
 			$tasks = [];
 			$tasks_ack = [];
@@ -716,6 +832,114 @@ class CEvent extends CApiService {
 				$tasks_ack = array_replace_recursive($tasks_ack, zbx_toObject($taskids, 'taskid', true));
 				DB::insertBatch('task_acknowledge', $tasks_ack, false);
 			}
+
+			// Create tasks for event rank change actions - convert symptoms to cause.
+			$tasks = [];
+			$task_update_event_rank = [];
+
+			foreach ($acknowledgeids as $k => $id) {
+				$acknowledgement = $acknowledges[$k];
+
+				if (($acknowledgement['action']
+						& ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE) == ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE) {
+					$tasks[$k] = [
+						'type' => ZBX_TM_TASK_DATA,
+						'status' => ZBX_TM_STATUS_NEW,
+						'clock' => $time
+					];
+
+					$task_update_event_rank[$k] = [
+						'taskid' => $id,
+						'type' => ZBX_TM_DATA_TYPE_RANK_EVENT,
+						'data' => json_encode([
+							'acknowledgeid' => $id,
+							'action' => $acknowledgement['action'],
+							'eventid' => $tasks_update_event_rank_cause[$k]['eventid'],
+							'userid' => $acknowledgement['userid']
+						])
+					];
+				}
+			}
+
+			if ($tasks) {
+				$taskids = DB::insertBatch('task', $tasks);
+				$task_update_event_rank = array_replace_recursive($task_update_event_rank,
+					zbx_toObject($taskids, 'taskid', true)
+				);
+				DB::insertBatch('task_data', $task_update_event_rank, false);
+
+				$upd_acknowledges = [];
+
+				foreach ($acknowledgeids as $k => $id) {
+					$acknowledgement = $acknowledges[$k];
+
+					if (($acknowledgement['action']
+							& ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE) == ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE) {
+						$upd_acknowledges[] = [
+							'values' => ['taskid' => $taskids[$k]],
+							'where' => ['acknowledgeid' => $id]
+						];
+					}
+				}
+
+				DB::update('acknowledges', $upd_acknowledges);
+			}
+
+			/*
+			 * Create tasks for event rank change actions - convert cause to symptoms or update symptoms by changing
+			 * cause to a different cause.
+			 */
+			$tasks = [];
+			$task_update_event_rank = [];
+
+			foreach ($acknowledgeids as $k => $id) {
+				$acknowledgement = $acknowledges[$k];
+
+				if (($acknowledgement['action']
+						& ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) == ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) {
+					$tasks[$k] = [
+						'type' => ZBX_TM_TASK_DATA,
+						'status' => ZBX_TM_STATUS_NEW,
+						'clock' => $time
+					];
+
+					$task_update_event_rank[$k] = [
+						'taskid' => $id,
+						'type' => ZBX_TM_DATA_TYPE_RANK_EVENT,
+						'data' => json_encode([
+							'acknowledgeid' => $id,
+							'action' => $acknowledgement['action'],
+							'eventid' => $tasks_update_event_rank_symptom[$k]['eventid'],
+							'cause_eventid' => $tasks_update_event_rank_symptom[$k]['cause_eventid'],
+							'userid' => $acknowledgement['userid']
+						])
+					];
+				}
+			}
+
+			if ($tasks) {
+				$taskids = DB::insertBatch('task', $tasks);
+				$task_update_event_rank = array_replace_recursive($task_update_event_rank,
+					zbx_toObject($taskids, 'taskid', true)
+				);
+				DB::insertBatch('task_data', $task_update_event_rank, false);
+
+				$upd_acknowledges = [];
+
+				foreach ($acknowledgeids as $k => $id) {
+					$acknowledgement = $acknowledges[$k];
+
+					if (($acknowledgement['action']
+							& ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) == ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) {
+						$upd_acknowledges[] = [
+							'values' => ['taskid' => $taskids[$k]],
+							'where' => ['acknowledgeid' => $id]
+						];
+					}
+				}
+
+				DB::update('acknowledges', $upd_acknowledges);
+			}
 		}
 
 		return ['eventids' => $data['eventids']];
@@ -724,38 +948,49 @@ class CEvent extends CApiService {
 	/**
 	 * Validates the input parameters for the acknowledge() method.
 	 *
-	 * @param array         $data              And array of operation data.
-	 * @param string|array  $data['eventids']  An event ID or an array of event IDs.
-	 * @param string        $data['message']   Message if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
-	 * @param string        $data['severity']  New severity level if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
-	 * @param int           $data['action']    Flags of performed operations combined:
-	 *                                           - 0x01 - ZBX_PROBLEM_UPDATE_CLOSE
-	 *                                           - 0x02 - ZBX_PROBLEM_UPDATE_ACKNOWLEDGE
-	 *                                           - 0x04 - ZBX_PROBLEM_UPDATE_MESSAGE
-	 *                                           - 0x08 - ZBX_PROBLEM_UPDATE_SEVERITY
-	 *                                           - 0x10 - ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
+	 * @param array         $data                    And array of operation data.
+	 * @param string|array  $data['eventids']        An event ID or an array of event IDs.
+	 * @param string        $data['cause_eventid']   Cause event ID. Used if $data['action'] yields 0x100.
+	 * @param string        $data['message']         Message if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
+	 * @param string        $data['severity']        New severity level if ZBX_PROBLEM_UPDATE_SEVERITY flag is passed.
+	 * @param int           $data['suppress_until']  Suppress until time if ZBX_PROBLEM_UPDATE_SUPPRESS flag is passed.
+	 * @param int           $data['action']          Flags of performed operations combined:
+	 *                                                - 0x01  - ZBX_PROBLEM_UPDATE_CLOSE
+	 *                                                - 0x02  - ZBX_PROBLEM_UPDATE_ACKNOWLEDGE
+	 *                                                - 0x04  - ZBX_PROBLEM_UPDATE_MESSAGE
+	 *                                                - 0x08  - ZBX_PROBLEM_UPDATE_SEVERITY
+	 *                                                - 0x10  - ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
+	 *                                                - 0x20  - ZBX_PROBLEM_UPDATE_SUPPRESS
+	 *                                                - 0x40  - ZBX_PROBLEM_UPDATE_UNSUPPRESS
+	 *                                                - 0x80  - ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE
+	 *                                                - 0x100 - ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM
 	 *
-	 * @throws APIException                    If the input is invalid.
+	 * @throws APIException                          If the input is invalid.
 	 */
-	protected function validateAcknowledge(array $data) {
-		$db_fields = [
-			'eventids' => null,
-			'action' => null,
-			'message' => '',
-			'severity' => ''
+	protected function validateAcknowledge(array $data, int $time) {
+		$fields =  [
+			'eventids' =>		['type' => API_IDS, 'flags' => API_REQUIRED | API_NORMALIZE],
+			'action' =>			['type' => API_INT32, 'flags' => API_REQUIRED],
+			'message' =>		['type' => API_STRING_UTF8, 'flags' => API_ALLOW_NULL, 'default' => DB::getDefault('acknowledges', 'message'), 'length' => DB::getFieldLength('acknowledges', 'message')],
+			'severity' =>		['type' => API_INT32, 'flags' => API_ALLOW_NULL, 'default' => DB::getDefault('acknowledges', 'new_severity')],
+			'suppress_until' =>	['type' => API_TIMESTAMP, 'flags' => API_ALLOW_NULL, 'default' => null],
+			'cause_eventid' => ['type' => API_MULTIPLE, 'rules' => [
+				// "cause_eventid" should only be accessible if a cause event is converted to symptom event.
+				['if' => static function ($data) { return ($data['action'] & ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) != 0; }, 'type' => API_ID, 'flags' => API_REQUIRED],
+				['else' => true, 'type' => API_UNEXPECTED]
+			]]
 		];
 
-		if (!check_db_fields($db_fields, $data)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+		if (!CApiInputValidator::validate(['type' => API_OBJECT, 'fields' => $fields], $data, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		$data['eventids'] = zbx_toArray($data['eventids']);
-		$data['eventids'] = array_keys(array_flip($data['eventids']));
+		$action_mask = ZBX_PROBLEM_UPDATE_CLOSE | ZBX_PROBLEM_UPDATE_ACKNOWLEDGE | ZBX_PROBLEM_UPDATE_MESSAGE
+				| ZBX_PROBLEM_UPDATE_SEVERITY | ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE | ZBX_PROBLEM_UPDATE_SUPPRESS
+				| ZBX_PROBLEM_UPDATE_UNSUPPRESS | ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE
+				| ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM;
 
 		// Check that at least one valid flag is set.
-		$action_mask = ZBX_PROBLEM_UPDATE_CLOSE | ZBX_PROBLEM_UPDATE_ACKNOWLEDGE | ZBX_PROBLEM_UPDATE_MESSAGE
-				| ZBX_PROBLEM_UPDATE_SEVERITY | ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE;
-
 		if (($data['action'] & $action_mask) != $data['action']) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
 				_s('unexpected value "%1$s"', $data['action'])
@@ -763,11 +998,64 @@ class CEvent extends CApiService {
 		}
 
 		$has_close_action = (($data['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE);
+		$has_ack_action = (($data['action'] & ZBX_PROBLEM_UPDATE_ACKNOWLEDGE) == ZBX_PROBLEM_UPDATE_ACKNOWLEDGE);
 		$has_message_action = (($data['action'] & ZBX_PROBLEM_UPDATE_MESSAGE) == ZBX_PROBLEM_UPDATE_MESSAGE);
 		$has_severity_action = (($data['action'] & ZBX_PROBLEM_UPDATE_SEVERITY) == ZBX_PROBLEM_UPDATE_SEVERITY);
+		$has_unack_action = (($data['action'] & ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE) == ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE);
+		$has_suppress_action = (($data['action'] & ZBX_PROBLEM_UPDATE_SUPPRESS) == ZBX_PROBLEM_UPDATE_SUPPRESS);
+		$has_unsuppress_action = (($data['action'] & ZBX_PROBLEM_UPDATE_UNSUPPRESS) == ZBX_PROBLEM_UPDATE_UNSUPPRESS);
+		$has_rank_change_to_cause_action =
+			(($data['action'] & ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE) == ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE);
+		$has_change_rank_to_symptom_action =
+			(($data['action'] & ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM) == ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM);
 
-		if (($data['action'] & ZBX_PROBLEM_UPDATE_ACKNOWLEDGE) &&
-				($data['action'] & ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE)) {
+		// Check access rules.
+		if ($has_close_action && !self::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_('no permissions to close problems')
+			));
+		}
+
+		if ($has_message_action && !self::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_('no permissions to add problem comments')
+			));
+		}
+
+		if ($has_severity_action && !self::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_('no permissions to change problem severity')
+			));
+		}
+
+		if (($has_ack_action || $has_unack_action) && !self::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				$has_ack_action
+					? _('no permissions to acknowledge problems')
+					: _('no permissions to unacknowledge problems')
+			));
+		}
+
+		if (($has_suppress_action || $has_unsuppress_action)
+				&& !self::checkAccess(CRoleHelper::ACTIONS_SUPPRESS_PROBLEMS)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				$has_suppress_action
+					? _('no permissions to suppress problems')
+					: _('no permissions to unsuppress problems')
+			));
+		}
+
+		// Check permissions in user roles if user is allowed to change event rank.
+		if (($has_rank_change_to_cause_action || $has_change_rank_to_symptom_action)
+				&& !self::checkAccess(CRoleHelper::ACTIONS_CHANGE_PROBLEM_RANKING)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				$has_rank_change_to_cause_action
+					? _('no permissions to convert symptom problems to cause problems')
+					: _('no permissions to convert cause problems to symptom problems')
+			));
+		}
+
+		if ($has_ack_action && $has_unack_action) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
 				_s('value must be one of %1$s', implode(', ', [ZBX_PROBLEM_UPDATE_ACKNOWLEDGE,
 					ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
@@ -775,10 +1063,39 @@ class CEvent extends CApiService {
 			));
 		}
 
+		if ($has_suppress_action && $has_unsuppress_action) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_s('value must be one of %1$s', implode(', ', [ZBX_PROBLEM_UPDATE_SUPPRESS,
+					ZBX_PROBLEM_UPDATE_UNSUPPRESS
+				]))
+			));
+		}
+
+		if ($has_close_action && ($has_suppress_action || $has_unsuppress_action)) {
+			$action = $has_suppress_action ? ZBX_PROBLEM_UPDATE_SUPPRESS : ZBX_PROBLEM_UPDATE_UNSUPPRESS;
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_s('value must be one of %1$s', implode(', ', [ZBX_PROBLEM_UPDATE_CLOSE, $action]))
+			));
+		}
+
+		if ($has_rank_change_to_cause_action && $has_change_rank_to_symptom_action) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_s('value must be one of %1$s', implode(', ', [ZBX_PROBLEM_UPDATE_RANK_TO_CAUSE,
+					ZBX_PROBLEM_UPDATE_RANK_TO_SYMPTOM
+				]))
+			));
+		}
+
+		// Add the new cause ID to validate if the event exists and is still a problem.
+		$eventids = array_fill_keys($data['eventids'], true);
+		if ($has_change_rank_to_symptom_action && $data['cause_eventid'] !== null) {
+			$eventids[$data['cause_eventid']] = true;
+		}
+
 		$events = $this->get([
-			'output' => [],
+			'output' => ['r_eventid'],
 			'selectRelatedObject' => $has_close_action ? ['manual_close'] : null,
-			'eventids' => $data['eventids'],
+			'eventids' => array_keys($eventids),
 			'source' => EVENT_SOURCE_TRIGGERS,
 			'object' => EVENT_OBJECT_TRIGGER,
 			'value' => TRIGGER_VALUE_TRUE
@@ -791,13 +1108,13 @@ class CEvent extends CApiService {
 		 *   - no read rights for related trigger
 		 *   - unexisting eventid
 		 */
-		if (count($data['eventids']) != count($events)) {
+		if (count($eventids) != count($events)) {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
 		$editable_events_count = $this->get([
 			'countOutput' => true,
-			'eventids' => $data['eventids'],
+			'eventids' => array_keys($eventids),
 			'source' => EVENT_SOURCE_TRIGGERS,
 			'object' => EVENT_OBJECT_TRIGGER,
 			'editable' => true
@@ -808,13 +1125,17 @@ class CEvent extends CApiService {
 		}
 
 		if ($has_message_action && $data['message'] === '') {
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Incorrect value for field "%1$s": %2$s.', 'message', _('cannot be empty'))
-			);
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'message',
+				_('cannot be empty')
+			));
 		}
 
 		if ($has_severity_action) {
-			$this->checkCanChangeSeverity($data['eventids'], $editable_events_count, $data['severity']);
+			$this->checkCanChangeSeverity($events, $editable_events_count, $data['severity']);
+		}
+
+		if ($has_suppress_action) {
+			$this->CheckIfValidTime($data, $time);
 		}
 	}
 
@@ -873,12 +1194,48 @@ class CEvent extends CApiService {
 	}
 
 	/**
-	 * Checks if events are closed.
+	 * Checks if time is valid future time.
 	 *
-	 * @param array $event                              Event object.
-	 * @param array $event['r_eventid']                 OK event id. 0 if not resolved.
-	 * @param array $event['acknowledges']              List of problem updates.
-	 * @param array $event['acknowledges'][]['action']  Action performed in update.
+	 * @param array $data                    Input data.
+	 * @param array $data['suppress_until']  Suppress until unix time. O for Indefinite time.
+	 * @param int   $time                    Current unix time.
+	 */
+	protected function CheckIfValidTime(array $data, $time) {
+		if ($data['suppress_until'] <= $time && $data['suppress_until'] != 0) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect value for field "%1$s": %2$s.', 'suppress_until',
+					_s('unexpected value "%1$s"', $data['suppress_until'])
+				)
+			);
+		}
+	}
+
+	/**
+	 * Checks if unsuppress action can be executed for given event.
+	 *
+	 * @param array $event                                         Event object.
+	 * @param array $event['suppression_data']                     List of problem suppression data.
+	 * @param array $event['suppression_data'][]['maintenanceid']  Problem maintenanceid.
+	 *
+	 * @return bool
+	 */
+	protected function isEventSuppressed(array $event) {
+		foreach ($event['suppression_data'] as $suppression) {
+			if($suppression['maintenanceid'] == 0) {
+				return true;
+			}
+		}
+
+		return false;
+	}
+
+	/**
+	 * Checks if event is closed.
+	 *
+	 * @param array  $event                              Event object.
+	 * @param string $event['r_eventid']                 OK event id. 0 if not resolved.
+	 * @param array  $event['acknowledges']              List of problem updates.
+	 * @param int    $event['acknowledges'][]['action']  Action performed in update.
 	 *
 	 * @return bool
 	 */
@@ -894,6 +1251,8 @@ class CEvent extends CApiService {
 				}
 			}
 		}
+
+		return false;
 	}
 
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
@@ -908,16 +1267,15 @@ class CEvent extends CApiService {
 			}
 
 			// Select fields from event_recovery table using LEFT JOIN.
-			$left_join = false;
-
+			$left_join_recovery = false;
 			foreach (['c_eventid', 'correlationid', 'userid'] as $field) {
 				if ($this->outputIsRequested($field, $options['output'])) {
 					$sqlParts['select'][$field] = 'er2.'.$field;
-					$left_join = true;
+					$left_join_recovery = true;
 				}
 			}
 
-			if ($left_join) {
+			if ($left_join_recovery) {
 				$sqlParts['left_join'][] = ['alias' => 'er2', 'table' => 'event_recovery', 'using' => 'r_eventid'];
 				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 			}
@@ -926,6 +1284,17 @@ class CEvent extends CApiService {
 				$sqlParts = $this->addQuerySelect('e.object', $sqlParts);
 				$sqlParts = $this->addQuerySelect('e.objectid', $sqlParts);
 			}
+
+			$left_join_symptom = false;
+			if ($this->outputIsRequested('cause_eventid', $options['output'])) {
+				$sqlParts['select']['cause_eventid'] = 'es1.cause_eventid';
+				$left_join_symptom = true;
+			}
+
+			if ($left_join_symptom) {
+				$sqlParts['left_join'][] = ['alias' => 'es1', 'table' => 'event_symptom', 'using' => 'eventid'];
+				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
+			}
 		}
 
 		return $sqlParts;
@@ -955,6 +1324,9 @@ class CEvent extends CApiService {
 
 		// adding hosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
+			$hosts = [];
+			$relationMap = new CRelationMap();
+
 			// trigger events
 			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
 				$query = DBselect(
@@ -979,17 +1351,21 @@ class CEvent extends CApiService {
 				);
 			}
 
-			$relationMap = new CRelationMap();
 			while ($relation = DBfetch($query)) {
 				$relationMap->addRelation($relation['eventid'], $relation['hostid']);
 			}
 
-			$hosts = API::Host()->get([
-				'output' => $options['selectHosts'],
-				'hostids' => $relationMap->getRelatedIds(),
-				'nopermissions' => true,
-				'preservekeys' => true
-			]);
+			$related_ids = $relationMap->getRelatedIds();
+
+			if ($related_ids) {
+				$hosts = API::Host()->get([
+					'output' => $options['selectHosts'],
+					'hostids' => $related_ids,
+					'nopermissions' => true,
+					'preservekeys' => true
+				]);
+			}
+
 			$result = $relationMap->mapMany($result, $hosts, 'hosts');
 		}
 
@@ -1018,6 +1394,9 @@ class CEvent extends CApiService {
 				case EVENT_OBJECT_LLDRULE:
 					$api = API::DiscoveryRule();
 					break;
+				case EVENT_OBJECT_SERVICE:
+					$api = API::Service();
+					break;
 			}
 
 			$objects = $api->get([
@@ -1031,16 +1410,22 @@ class CEvent extends CApiService {
 
 		// adding alerts
 		if ($options['select_alerts'] !== null && $options['select_alerts'] != API_OUTPUT_COUNT) {
+			$alerts = [];
 			$relationMap = $this->createRelationMap($result, 'eventid', 'alertid', 'alerts');
-			$alerts = API::Alert()->get([
-				'output' => $options['select_alerts'],
-				'selectMediatypes' => API_OUTPUT_EXTEND,
-				'alertids' => $relationMap->getRelatedIds(),
-				'nopermissions' => true,
-				'preservekeys' => true,
-				'sortfield' => 'clock',
-				'sortorder' => ZBX_SORT_DOWN
-			]);
+			$related_ids = $relationMap->getRelatedIds();
+
+			if ($related_ids) {
+				$alerts = API::Alert()->get([
+					'output' => $options['select_alerts'],
+					'selectMediatypes' => API_OUTPUT_EXTEND,
+					'alertids' => $related_ids,
+					'nopermissions' => true,
+					'preservekeys' => true,
+					'sortfield' => 'clock',
+					'sortorder' => ZBX_SORT_DOWN
+				]);
+			}
+
 			$result = $relationMap->mapMany($result, $alerts, 'alerts');
 		}
 
@@ -1059,7 +1444,7 @@ class CEvent extends CApiService {
 				$acknowledges = DBFetchArrayAssoc(DBselect(self::createSelectQueryFromParts($sqlParts)), 'acknowledgeid');
 
 				// if the user data is requested via extended output or specified fields, join the users table
-				$userFields = ['alias', 'name', 'surname'];
+				$userFields = ['username', 'name', 'surname'];
 				$requestUserData = [];
 				foreach ($userFields as $userField) {
 					if ($this->outputIsRequested($userField, $options['select_acknowledges'])) {
