@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -401,7 +401,7 @@ class CHostGroup extends CApiService {
 	}
 
 	/**
-	 * @param array  $groups
+	 * @param array $groups
 	 *
 	 * @return array
 	 */
@@ -558,20 +558,23 @@ class CHostGroup extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
+		self::addUuid($groups);
+
+		self::checkUuidDuplicates($groups);
 		self::checkDuplicates($groups);
-		self::checkAndAddUuid($groups);
 	}
 
 	/**
 	 * Validates input data for update method.
 	 *
-	 * @param array $groups     [IN/OUT]
-	 * @param array $db_groups  [OUT]
+	 * @param array $groups
+	 * @param array $db_groups
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateUpdate(array &$groups, array &$db_groups = null): void {
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['groupid'], ['name']], 'fields' => [
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['groupid'], ['name']], 'fields' => [
+			'uuid' => 		['type' => API_UUID],
 			'groupid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
 			'name' =>		['type' => API_HG_NAME, 'length' => DB::getFieldLength('hstgrp', 'name')]
 		]];
@@ -581,7 +584,7 @@ class CHostGroup extends CApiService {
 		}
 
 		$db_groups = $this->get([
-			'output' => ['groupid', 'name', 'flags'],
+			'output' => ['uuid', 'groupid', 'name', 'flags'],
 			'groupids' => array_column($groups, 'groupid'),
 			'editable' => true,
 			'preservekeys' => true
@@ -592,6 +595,7 @@ class CHostGroup extends CApiService {
 		}
 
 		self::checkGroupsNotDiscovered($db_groups);
+		self::checkUuidDuplicates($groups, $db_groups);
 		self::checkDuplicates($groups, $db_groups);
 	}
 
@@ -738,31 +742,58 @@ class CHostGroup extends CApiService {
 	}
 
 	/**
-	 * Check that new UUIDs are not already used and generate UUIDs where missing.
+	 * Add the UUID to those of the given host groups that don't have the 'uuid' parameter set.
 	 *
-	 * @static
-	 *
-	 * @param array $groups_to_create
-	 *
-	 * @throws APIException
+	 * @param array $groups
 	 */
-	private static function checkAndAddUuid(array &$groups_to_create): void {
-		foreach ($groups_to_create as &$group) {
+	private static function addUuid(array &$groups): void {
+		foreach ($groups as &$group) {
 			if (!array_key_exists('uuid', $group)) {
 				$group['uuid'] = generateUuidV4();
 			}
 		}
 		unset($group);
+	}
 
-		$db_uuid = DB::select('hstgrp', [
+	/**
+	 * Verify host group UUIDs are not repeated.
+	 *
+	 * @param array      $groups
+	 * @param array|null $db_groups
+	 *
+	 * @throws APIException
+	 */
+	private static function checkUuidDuplicates(array $groups, array $db_groups = null): void {
+		$group_indexes = [];
+
+		foreach ($groups as $i => $group) {
+			if (!array_key_exists('uuid', $group)) {
+				continue;
+			}
+
+			if ($db_groups === null || $group['uuid'] !== $db_groups[$group['groupid']]['uuid']) {
+				$group_indexes[$group['uuid']] = $i;
+			}
+		}
+
+		if (!$group_indexes) {
+			return;
+		}
+
+		$duplicates = DB::select('hstgrp', [
 			'output' => ['uuid'],
-			'filter' => ['uuid' => array_column($groups_to_create, 'uuid'), 'type' => HOST_GROUP_TYPE_HOST_GROUP],
+			'filter' => [
+				'type' => HOST_GROUP_TYPE_HOST_GROUP,
+				'uuid' => array_keys($group_indexes)
+			],
 			'limit' => 1
 		]);
 
-		if ($db_uuid) {
+		if ($duplicates) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Entry with UUID "%1$s" already exists.', $db_uuid[0]['uuid'])
+				_s('Invalid parameter "%1$s": %2$s.', '/'.($group_indexes[$duplicates[0]['uuid']] + 1),
+					_('host group with the same UUID already exists')
+				)
 			);
 		}
 	}
