@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -64,15 +64,8 @@ class CRole extends CApiService {
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			// filter
 			'roleids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
-			'filter' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
-				'roleid' =>					['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'type' =>					['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [USER_TYPE_ZABBIX_USER, USER_TYPE_ZABBIX_ADMIN, USER_TYPE_SUPER_ADMIN])],
-				'readonly' =>				['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => '0,1']
-			]],
-			'search' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
-				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE]
-			]],
+			'filter' =>					['type' => API_FILTER, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => ['roleid', 'name', 'type', 'readonly']],
+			'search' =>					['type' => API_FILTER, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => ['name']],
 			'searchByAny' =>			['type' => API_BOOLEAN, 'default' => false],
 			'startSearch' =>			['type' => API_FLAG, 'default' => false],
 			'excludeSearch' =>			['type' => API_FLAG, 'default' => false],
@@ -707,7 +700,7 @@ class CRole extends CApiService {
 			return;
 		}
 
-		$unavailable_moduleids = array_diff(array_keys($moduleids), self::getEnabledModuleIds());
+		$unavailable_moduleids = array_diff(array_keys($moduleids), self::getModuleIds());
 
 		if ($unavailable_moduleids) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -1043,7 +1036,7 @@ class CRole extends CApiService {
 
 		$index = 0;
 
-		foreach (self::getEnabledModuleIds() as $moduleid) {
+		foreach (self::getModuleIds() as $moduleid) {
 			if (array_key_exists($moduleid, $new_modules_rules)) {
 				$module_status = $new_modules_rules[$moduleid]['status'];
 			}
@@ -1161,7 +1154,7 @@ class CRole extends CApiService {
 	 * @return array
 	 */
 	protected function applyQueryFilterOptions($table_name, $table_alias, array $options, array $sql_parts): array {
-		$sqlParts = parent::applyQueryFilterOptions($table_name, $table_alias, $options, $sql_parts);
+		$sql_parts = parent::applyQueryFilterOptions($table_name, $table_alias, $options, $sql_parts);
 
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
 			$sql_parts['from']['users'] = 'users u';
@@ -1169,7 +1162,7 @@ class CRole extends CApiService {
 			$sql_parts['where'][] = 'u.userid='.self::$userData['userid'];
 		}
 
-		return $sqlParts;
+		return $sql_parts;
 	}
 
 	/**
@@ -1424,7 +1417,7 @@ class CRole extends CApiService {
 		if (in_array('modules', $output, true)) {
 			$modules = [];
 
-			foreach (self::getEnabledModuleIds() as $moduleid) {
+			foreach (self::getModuleIds() as $moduleid) {
 				$modules[$moduleid] = [
 					'moduleid' => $moduleid,
 					'status' => $modules_default_access
@@ -1527,12 +1520,9 @@ class CRole extends CApiService {
 	 *
 	 * @throws APIException
 	 */
-	private static function getEnabledModuleIds(): array {
+	private static function getModuleIds(): array {
 		$modules = API::getApiService('module')->get([
 			'output' => [],
-			'filter' => [
-				'status' => MODULE_STATUS_ENABLED
-			],
 			'preservekeys' => true
 		], false);
 
