@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ class CScript extends CApiService {
 	public const ACCESS_RULES = [
 		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
 		'getscriptsbyhosts' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'getscriptsbyevents' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
 		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
 		'update' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
 		'delete' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
@@ -41,7 +42,7 @@ class CScript extends CApiService {
 	 * Fields from "actions" table. Used in get() validation and addRelatedObjects() when selecting action fields.
 	 */
 	private $action_fields = ['actionid', 'name', 'eventsource', 'status', 'esc_period', 'pause_suppressed',
-		'notify_if_canceled'
+		'notify_if_canceled', 'pause_symptoms'
 	];
 
 	/**
@@ -63,7 +64,7 @@ class CScript extends CApiService {
 	public function get(array $options) {
 		$script_fields = ['scriptid', 'name', 'command', 'host_access', 'usrgrpid', 'groupid', 'description',
 			'confirmation', 'type', 'execute_on', 'timeout', 'parameters', 'scope', 'port', 'authtype', 'username',
-			'password', 'publickey', 'privatekey', 'menu_path'
+			'password', 'publickey', 'privatekey', 'menu_path', 'url', 'new_window'
 		];
 		$group_fields = ['groupid', 'name', 'flags', 'uuid'];
 		$host_fields = ['hostid', 'host', 'name', 'description', 'status', 'proxy_hostid', 'inventory_mode', 'flags',
@@ -77,27 +78,8 @@ class CScript extends CApiService {
 			'hostids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
 			'groupids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
 			'usrgrpids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
-			'filter' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
-				'scriptid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'command' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'host_access' =>			['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [PERM_READ, PERM_READ_WRITE])],
-				'usrgrpid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'groupid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'confirmation' =>			['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'type' =>					['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT, ZBX_SCRIPT_TYPE_IPMI, ZBX_SCRIPT_TYPE_SSH, ZBX_SCRIPT_TYPE_TELNET, ZBX_SCRIPT_TYPE_WEBHOOK])],
-				'execute_on' =>				['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [ZBX_SCRIPT_EXECUTE_ON_AGENT, ZBX_SCRIPT_EXECUTE_ON_SERVER, ZBX_SCRIPT_EXECUTE_ON_PROXY])],
-				'scope' =>					['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
-				'menu_path' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE]
-			]],
-			'search' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
-				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'command' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'description' =>			['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'confirmation' =>			['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'username' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'menu_path' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE]
-			]],
+			'filter' =>					['type' => API_FILTER, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => ['scriptid', 'name', 'command', 'host_access', 'usrgrpid', 'groupid', 'confirmation', 'type', 'url', 'new_window', 'execute_on', 'scope', 'menu_path']],
+			'search' =>					['type' => API_FILTER, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => ['name', 'command', 'url', 'description', 'confirmation', 'username', 'menu_path']],
 			'searchByAny' =>			['type' => API_BOOLEAN, 'default' => false],
 			'startSearch' =>			['type' => API_FLAG, 'default' => false],
 			'excludeSearch' =>			['type' => API_FLAG, 'default' => false],
@@ -283,11 +265,26 @@ class CScript extends CApiService {
 		 */
 		foreach ($scripts as $index => $script) {
 			$type_rules = $this->getTypeValidationRules($script['type'], 'create', $type_fields);
-			$this->getScopeValidationRules($script['scope'], $scope_fields);
+			$scope_rules = $this->getScopeValidationRules($script['scope'], $scope_fields);
 
-			$type_rules['fields'] += $common_fields + $scope_fields;
+			// Temporary remove scope fields from script to validate type fields.
+			$tmp = $script;
+			$tmp_fields = array_intersect_key($scope_rules['fields'], $script);
 
-			if (!CApiInputValidator::validate($type_rules, $script, '/'.($index + 1), $error)) {
+			foreach ($tmp_fields as $field => $rules) {
+				unset($tmp[$field]);
+			}
+
+			$type_rules['fields'] += $common_fields + $type_fields;
+
+			if (!CApiInputValidator::validate($type_rules, $tmp, '/'.($index + 1), $error)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+			}
+
+			// Validate all fields together.
+			$scope_rules['fields'] += $type_rules['fields'] + $common_fields + $scope_fields;
+
+			if (!CApiInputValidator::validate($scope_rules, $script, '/'.($index + 1), $error)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 			}
 
@@ -301,7 +298,8 @@ class CScript extends CApiService {
 			}
 		}
 
-		self::checkDuplicates($scripts);
+		$this->checkUniqueness($scripts);
+		$this->checkDuplicates($scripts);
 
 		// Finally check User and Host IDs.
 		$this->checkUserGroups($scripts);
@@ -368,7 +366,7 @@ class CScript extends CApiService {
 		$db_scripts = DB::select('scripts', [
 			'output' => ['scriptid', 'name', 'command', 'host_access', 'usrgrpid', 'groupid', 'description',
 				'confirmation', 'type', 'execute_on', 'timeout', 'scope', 'port', 'authtype', 'username', 'password',
-				'publickey', 'privatekey', 'menu_path'
+				'publickey', 'privatekey', 'menu_path', 'url', 'new_window'
 			],
 			'scriptids' => array_column($scripts, 'scriptid'),
 			'preservekeys' => true
@@ -378,8 +376,7 @@ class CScript extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
-		self::checkDuplicates($scripts, $db_scripts);
-
+		$this->checkUniqueness($scripts, 'update');
 		self::addAffectedObjects($scripts, $db_scripts);
 
 		// Validate if scripts belong to actions and scope can be changed.
@@ -457,13 +454,16 @@ class CScript extends CApiService {
 		}
 
 		// Populate common and mandatory fields.
-		$scripts = $this->extendObjectsByKey($scripts, $db_scripts, 'scriptid', ['name', 'type', 'command', 'scope']);
+		$scripts = $this->extendObjectsByKey($scripts, $db_scripts, 'scriptid', ['name', 'type', 'scope']);
 
 		foreach ($scripts as $index => &$script) {
 			$db_script = $db_scripts[$script['scriptid']];
 			$method = 'update';
 
-			if (array_key_exists('type', $script) && $script['type'] != $db_script['type']) {
+			if (array_key_exists('type', $script) && $script['type'] != $db_script['type']
+					|| array_key_exists('scope', $script) && $script['scope'] == ZBX_SCRIPT_SCOPE_ACTION
+						&& $db_script['scope'] != ZBX_SCRIPT_SCOPE_ACTION
+						&& $db_script['type'] == ZBX_SCRIPT_TYPE_URL) {
 				// This means that all other fields are now required just like create method.
 				$method = 'create';
 
@@ -477,11 +477,26 @@ class CScript extends CApiService {
 			}
 
 			$type_rules = $this->getTypeValidationRules($script['type'], $method, $type_fields);
-			$this->getScopeValidationRules($script['scope'], $scope_fields);
+			$scope_rules = $this->getScopeValidationRules($script['scope'], $scope_fields);
 
-			$type_rules['fields'] += $common_fields + $scope_fields;
+			// Temporary remove scope fields from script to validate type fields.
+			$tmp = $script;
+			$tmp_fields = array_intersect_key($scope_rules['fields'], $script);
 
-			if (!CApiInputValidator::validate($type_rules, $script, '/'.($index + 1), $error)) {
+			foreach ($tmp_fields as $field => $rules) {
+				unset($tmp[$field]);
+			}
+
+			$type_rules['fields'] += $common_fields + $type_fields;
+
+			if (!CApiInputValidator::validate($type_rules, $tmp, '/'.($index + 1), $error)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+			}
+
+			// Validate all fields together.
+			$scope_rules['fields'] += $type_rules['fields'] + $common_fields + $scope_fields;
+
+			if (!CApiInputValidator::validate($scope_rules, $script, '/'.($index + 1), $error)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 			}
 
@@ -512,6 +527,8 @@ class CScript extends CApiService {
 				switch ($script['type']) {
 					case ZBX_SCRIPT_TYPE_IPMI:
 						$script['execute_on'] = DB::getDefault('scripts', 'execute_on');
+						$script['url'] = '';
+						$script['new_window'] = DB::getDefault('scripts', 'new_window');
 						// break; is not missing here
 
 					case ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT:
@@ -523,12 +540,16 @@ class CScript extends CApiService {
 						$script['privatekey'] = '';
 						$script['parameters'] = [];
 						$script['timeout'] = DB::getDefault('scripts', 'timeout');
+						$script['url'] = '';
+						$script['new_window'] = DB::getDefault('scripts', 'new_window');
 						break;
 
 					case ZBX_SCRIPT_TYPE_SSH:
 						$script['execute_on'] = DB::getDefault('scripts', 'execute_on');
 						$script['parameters'] = [];
 						$script['timeout'] = DB::getDefault('scripts', 'timeout');
+						$script['url'] = '';
+						$script['new_window'] = DB::getDefault('scripts', 'new_window');
 						break;
 
 					case ZBX_SCRIPT_TYPE_TELNET:
@@ -538,9 +559,26 @@ class CScript extends CApiService {
 						$script['execute_on'] = DB::getDefault('scripts', 'execute_on');
 						$script['parameters'] = [];
 						$script['timeout'] = DB::getDefault('scripts', 'timeout');
+						$script['url'] = '';
+						$script['new_window'] = DB::getDefault('scripts', 'new_window');
 						break;
 
 					case ZBX_SCRIPT_TYPE_WEBHOOK:
+						$script['port'] = '';
+						$script['authtype'] = DB::getDefault('scripts', 'authtype');
+						$script['username'] = '';
+						$script['password'] = '';
+						$script['publickey'] = '';
+						$script['privatekey'] = '';
+						$script['execute_on'] = DB::getDefault('scripts', 'execute_on');
+						$script['url'] = '';
+						$script['new_window'] = DB::getDefault('scripts', 'new_window');
+						break;
+
+					case ZBX_SCRIPT_TYPE_URL:
+						$script['command'] = '';
+						$script['parameters'] = [];
+						$script['timeout'] = DB::getDefault('scripts', 'timeout');
 						$script['port'] = '';
 						$script['authtype'] = DB::getDefault('scripts', 'authtype');
 						$script['username'] = '';
@@ -567,6 +605,7 @@ class CScript extends CApiService {
 		}
 		unset($script);
 
+		$this->checkDuplicates($scripts, $db_scripts);
 		$this->checkUserGroups($scripts);
 		$this->checkHostGroups($scripts);
 	}
@@ -584,22 +623,19 @@ class CScript extends CApiService {
 
 		$common_fields = [
 			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'name')],
-			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT, ZBX_SCRIPT_TYPE_IPMI, ZBX_SCRIPT_TYPE_SSH, ZBX_SCRIPT_TYPE_TELNET, ZBX_SCRIPT_TYPE_WEBHOOK])],
-			'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
-			'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')],
+			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT, ZBX_SCRIPT_TYPE_IPMI, ZBX_SCRIPT_TYPE_SSH, ZBX_SCRIPT_TYPE_TELNET, ZBX_SCRIPT_TYPE_WEBHOOK, ZBX_SCRIPT_TYPE_URL])],
+			'scope' =>			['type' => API_INT32],
 			'groupid' =>		['type' => API_ID],
 			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('scripts', 'description')]
 		];
 
 		if ($method === 'create') {
-			$common_fields['scope']['default'] = ZBX_SCRIPT_SCOPE_ACTION;
-			$api_input_rules['uniq'] = [['name']];
 			$common_fields['name']['flags'] |= API_REQUIRED;
 			$common_fields['type']['flags'] = API_REQUIRED;
-			$common_fields['command']['flags'] |= API_REQUIRED;
+			$common_fields['scope']['flags'] = API_REQUIRED;
 		}
 		else {
-			$api_input_rules['uniq'] = [['scriptid'], ['name']];
+			$api_input_rules['uniq'] = [['scriptid']];
 			$common_fields += ['scriptid' => ['type' => API_ID, 'flags' => API_REQUIRED]];
 		}
 
@@ -608,6 +644,7 @@ class CScript extends CApiService {
 		 * script types. Set only type for now. Unique parameter names, lengths and other flags are set later.
 		 */
 		$api_input_rules['fields'] += $common_fields + [
+			'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')],
 			'execute_on' =>		['type' => API_INT32],
 			'menu_path' =>		['type' => API_STRING_UTF8],
 			'usrgrpid' =>		['type' => API_ID],
@@ -623,7 +660,9 @@ class CScript extends CApiService {
 			'parameters' =>			['type' => API_OBJECTS, 'fields' => [
 				'name' =>				['type' => API_STRING_UTF8],
 				'value' =>				['type' => API_STRING_UTF8]
-			]]
+			]],
+			'url' =>			['type' => API_URL],
+			'new_window' =>		['type' => API_INT32]
 		];
 
 		return $api_input_rules;
@@ -671,12 +710,33 @@ class CScript extends CApiService {
 		switch ($type) {
 			case ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT:
 				$api_input_rules['fields'] += [
+					'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
+					'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')],
 					'execute_on' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_EXECUTE_ON_AGENT, ZBX_SCRIPT_EXECUTE_ON_SERVER, ZBX_SCRIPT_EXECUTE_ON_PROXY])]
 				];
+
+				if ($method === 'create') {
+					$api_input_rules['fields']['scope']['flags'] = API_REQUIRED;
+					$api_input_rules['fields']['command']['flags'] |= API_REQUIRED;
+				}
+				break;
+
+			case ZBX_SCRIPT_TYPE_IPMI:
+				$api_input_rules['fields'] += [
+					'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
+					'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')]
+				];
+
+				if ($method === 'create') {
+					$api_input_rules['fields']['scope']['flags'] = API_REQUIRED;
+					$api_input_rules['fields']['command']['flags'] |= API_REQUIRED;
+				}
 				break;
 
 			case ZBX_SCRIPT_TYPE_SSH:
 				$common_fields = [
+					'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
+					'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')],
 					'port' =>			['type' => API_PORT, 'flags' => API_ALLOW_USER_MACRO],
 					'authtype' =>		['type' => API_INT32, 'in' => implode(',', [ITEM_AUTHTYPE_PASSWORD, ITEM_AUTHTYPE_PUBLICKEY])],
 					'username' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'username')],
@@ -684,6 +744,8 @@ class CScript extends CApiService {
 				];
 
 				if ($method === 'create') {
+					$common_fields['scope']['flags'] = API_REQUIRED;
+					$common_fields['command']['flags'] |= API_REQUIRED;
 					$common_fields['username']['flags'] |= API_REQUIRED;
 				}
 
@@ -695,24 +757,48 @@ class CScript extends CApiService {
 
 			case ZBX_SCRIPT_TYPE_TELNET:
 				$api_input_rules['fields'] += [
+					'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
+					'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')],
 					'port' =>			['type' => API_PORT, 'flags' => API_ALLOW_USER_MACRO],
 					'username' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'username')],
 					'password' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('scripts', 'password')]
 				];
 
 				if ($method === 'create') {
+					$api_input_rules['fields']['scope']['flags'] = API_REQUIRED;
+					$api_input_rules['fields']['command']['flags'] |= API_REQUIRED;
 					$api_input_rules['fields']['username']['flags'] |= API_REQUIRED;
 				}
 				break;
 
 			case ZBX_SCRIPT_TYPE_WEBHOOK:
 				$api_input_rules['fields'] += [
-					'timeout' =>		['type' => API_TIME_UNIT, 'in' => '1:'.SEC_PER_MIN],
+					'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_ACTION, ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
+					'command' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('scripts', 'command')],
+					'timeout' =>		['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'in' => '1:'.SEC_PER_MIN],
 					'parameters' =>			['type' => API_OBJECTS, 'uniq' => [['name']], 'fields' => [
 						'name' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('script_param', 'name')],
 						'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('script_param', 'value')]
 					]]
 				];
+
+				if ($method === 'create') {
+					$api_input_rules['fields']['scope']['flags'] = API_REQUIRED;
+					$api_input_rules['fields']['command']['flags'] |= API_REQUIRED;
+				}
+				break;
+
+			case ZBX_SCRIPT_TYPE_URL:
+				$api_input_rules['fields'] += [
+					'scope' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_SCOPE_HOST, ZBX_SCRIPT_SCOPE_EVENT])],
+					'url' =>			['type' => API_URL, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO | API_ALLOW_MACRO, 'length' => DB::getFieldLength('scripts', 'url')],
+					'new_window' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_SCRIPT_URL_NEW_WINDOW_NO, ZBX_SCRIPT_URL_NEW_WINDOW_YES]), 'default' => DB::getDefault('scripts', 'new_window')]
+				];
+
+				if ($method === 'create') {
+					$api_input_rules['fields']['scope']['flags'] = API_REQUIRED;
+					$api_input_rules['fields']['url']['flags'] |= API_REQUIRED;
+				}
 				break;
 		}
 
@@ -839,7 +925,7 @@ class CScript extends CApiService {
 	}
 
 	/**
-	 * @static
+	 * Validates parameters for script.delete method.
 	 *
 	 * @param array      $scriptids
 	 * @param array|null $db_scripts
@@ -943,7 +1029,7 @@ class CScript extends CApiService {
 		}
 
 		$db_scripts = $this->get([
-			'output' => [],
+			'output' => ['type'],
 			'hostids' => $hostids,
 			'scriptids' => $data['scriptid']
 		]);
@@ -952,12 +1038,18 @@ class CScript extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
+		if ($db_scripts[0]['type'] == ZBX_SCRIPT_TYPE_URL) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Cannot execute URL type script.'));
+		}
+
 		// execute script
 		$zabbix_server = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT,
 			timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::CONNECT_TIMEOUT)),
 			timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::SCRIPT_TIMEOUT)), ZBX_SOCKET_BYTES_LIMIT
 		);
-		$result = $zabbix_server->executeScript($data['scriptid'], self::$userData['sessionid'],
+
+		$result = $zabbix_server->executeScript($data['scriptid'],
+			array_key_exists('sessionid', self::$userData) ? self::$userData['sessionid'] : self::$userData['token'],
 			$is_event ? null : $data['hostid'],
 			$is_event ? $data['eventid'] : null
 		);
@@ -976,7 +1068,8 @@ class CScript extends CApiService {
 	}
 
 	/**
-	 * Returns all the scripts that are available on each given host.
+	 * Returns all the scripts that are available on each given host. Automatically resolves macros in
+	 * confirmation and URL fields.
 	 *
 	 * @param $hostids
 	 *
@@ -995,8 +1088,16 @@ class CScript extends CApiService {
 			$scripts_by_host[$hostid] = [];
 		}
 
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $hostids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
 		$scripts = $this->get([
-			'output' => API_OUTPUT_EXTEND,
+			'output' => ['scriptid', 'name', 'command', 'host_access', 'usrgrpid', 'groupid', 'description',
+				'confirmation', 'type', 'execute_on', 'timeout', 'scope', 'port', 'authtype', 'username', 'password',
+				'publickey', 'privatekey', 'menu_path', 'url', 'new_window'
+			],
 			'hostids' => $hostids,
 			'sortfield' => 'name',
 			'preservekeys' => true
@@ -1009,38 +1110,50 @@ class CScript extends CApiService {
 		], $scripts, $hostids);
 
 		if ($scripts) {
-			// resolve macros
 			$macros_data = [];
+
 			foreach ($scripts as $scriptid => $script) {
-				if (!empty($script['confirmation'])) {
-					foreach ($script['hosts'] as $host) {
-						if (isset($scripts_by_host[$host['hostid']])) {
-							$macros_data[$host['hostid']][$scriptid] = $script['confirmation'];
+				foreach ($script['hosts'] as $host) {
+					$hostid = $host['hostid'];
+
+					if (array_key_exists($hostid, $scripts_by_host)) {
+						if (strpos($script['confirmation'], '{') !== false) {
+							$macros_data[$hostid][$scriptid]['confirmation'] = $script['confirmation'];
+						}
+
+						if (strpos($script['url'], '{') !== false) {
+							$macros_data[$hostid][$scriptid]['url'] = $script['url'];
 						}
 					}
 				}
 			}
-			if ($macros_data) {
-				$macros_data = CMacrosResolverHelper::resolve([
-					'config' => 'scriptConfirmation',
-					'data' => $macros_data
-				]);
-			}
+
+			$macros_data = CMacrosResolverHelper::resolveManualHostActionScripts($macros_data);
 
 			foreach ($scripts as $scriptid => $script) {
 				$hosts = $script['hosts'];
 				unset($script['hosts']);
-				// set script to host
+
+				// Set script to host.
 				foreach ($hosts as $host) {
 					$hostid = $host['hostid'];
 
-					if (isset($scripts_by_host[$hostid])) {
+					if (array_key_exists($hostid, $scripts_by_host)) {
 						$size = count($scripts_by_host[$hostid]);
 						$scripts_by_host[$hostid][$size] = $script;
 
-						// set confirmation text with resolved macros
-						if (isset($macros_data[$hostid][$scriptid]) && $script['confirmation']) {
-							$scripts_by_host[$hostid][$size]['confirmation'] = $macros_data[$hostid][$scriptid];
+						// Set confirmation and URL with resolved macros.
+						if (array_key_exists($hostid, $macros_data)
+								&& array_key_exists($scriptid, $macros_data[$hostid])) {
+							$macro_values = $macros_data[$hostid][$scriptid];
+
+							if (strpos($script['confirmation'], '{') !== false) {
+								$scripts_by_host[$hostid][$size]['confirmation'] = $macro_values['confirmation'];
+							}
+
+							if (strpos($script['url'], '{') !== false) {
+								$scripts_by_host[$hostid][$size]['url'] = $macro_values['url'];
+							}
 						}
 					}
 				}
@@ -1050,6 +1163,184 @@ class CScript extends CApiService {
 		return $scripts_by_host;
 	}
 
+	/**
+	 * Returns all the scripts that are available on each given event. Automatically resolves macros in
+	 * confirmation and URL fields.
+	 *
+	 * @param $eventids
+	 *
+	 * @return array
+	 */
+	public function getScriptsByEvents($eventids) {
+		zbx_value2array($eventids);
+
+		$scripts_by_events = [];
+
+		if (!$eventids) {
+			return $scripts_by_events;
+		}
+
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $eventids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		foreach ($eventids as $eventid) {
+			$scripts_by_events[$eventid] = [];
+		}
+
+		$events = API::Event()->get([
+			'output' => ['eventid', 'objectid', 'value', 'name', 'severity', 'cause_eventid'],
+			'selectHosts' => ['hostid'],
+			'object' => EVENT_OBJECT_TRIGGER,
+			'source' => EVENT_SOURCE_TRIGGERS,
+			'eventids' => $eventids,
+			'preservekeys' => true
+		]);
+
+		if (!$events) {
+			return $scripts_by_events;
+		}
+
+		$symptom_cause_eventids = [];
+
+		foreach ($events as &$event) {
+			if ($event['cause_eventid'] != 0) {
+				// There is not need to select already preselected events again.
+				if (array_key_exists($event['cause_eventid'], $events)) {
+					$event['cause'] = [
+						'eventid' => $events[$event['cause_eventid']]['eventid'],
+						'value' => $events[$event['cause_eventid']]['value'],
+						'name' => $events[$event['cause_eventid']]['name'],
+						'severity' => $events[$event['cause_eventid']]['severity']
+					];
+				}
+				else {
+					$event['cause'] = [];
+					// Collect cause event IDs for symptom events.
+					$symptom_cause_eventids[] = $event['cause_eventid'];
+				}
+			}
+		}
+		unset($event);
+
+		if ($symptom_cause_eventids) {
+			$cause_events = API::Event()->get([
+				'output' => ['eventid', 'value', 'name', 'severity'],
+				'object' => EVENT_OBJECT_TRIGGER,
+				'source' => EVENT_SOURCE_TRIGGERS,
+				'eventids' => $symptom_cause_eventids,
+				'preservekeys' => true
+			]);
+
+			if ($cause_events) {
+				foreach ($events as &$event) {
+					foreach ($cause_events as $cause_event) {
+						if (bccomp($event['cause_eventid'], $cause_event['eventid']) == 0) {
+							$event['cause'] = [
+								'eventid' => $cause_event['eventid'],
+								'value' => $cause_event['value'],
+								'name' => $cause_event['name'],
+								'severity' => $cause_event['severity']
+							];
+						}
+					}
+				}
+				unset($event);
+			}
+		}
+
+		$hostids = [];
+		foreach ($events as $event) {
+			foreach ($event['hosts'] as $host) {
+				$hostids[$host['hostid']] = true;
+			}
+		}
+
+		$scripts = $this->get([
+			'output' => ['scriptid', 'name', 'command', 'host_access', 'usrgrpid', 'groupid', 'description',
+				'confirmation', 'type', 'execute_on', 'timeout', 'scope', 'port', 'authtype', 'username', 'password',
+				'publickey', 'privatekey', 'menu_path', 'url', 'new_window'
+			],
+			'hostids' => array_keys($hostids),
+			'sortfield' => 'name',
+			'preservekeys' => true
+		]);
+
+		$scripts = $this->addRelatedGroupsAndHosts([
+			'selectGroups' => null,
+			'selectHostGroups' => null,
+			'selectHosts' => ['hostid']
+		], $scripts, array_keys($hostids));
+
+		if ($scripts) {
+			$macros_data = [];
+			$processed_scripts_per_event = [];
+
+			foreach ($scripts as $scriptid => $script) {
+				foreach ($events as $eventid => $event) {
+					foreach ($event['hosts'] as $event_host) {
+						foreach ($script['hosts'] as $host) {
+							if (bccomp($host['hostid'], $event_host['hostid']) == 0
+									&& array_key_exists($eventid, $scripts_by_events)) {
+								if (strpos($script['confirmation'], '{') !== false) {
+									$macros_data[$eventid][$scriptid]['confirmation'] = $script['confirmation'];
+								}
+
+								if (strpos($script['url'], '{') !== false) {
+									$macros_data[$eventid][$scriptid]['url'] = $script['url'];
+								}
+							}
+						}
+					}
+				}
+			}
+
+			$macros_data = CMacrosResolverHelper::resolveManualEventActionScripts($macros_data, $events);
+
+			foreach ($scripts as $scriptid => $script) {
+				$hosts = $script['hosts'];
+				unset($script['hosts']);
+
+				foreach ($events as $eventid => $event) {
+					if (!array_key_exists($eventid, $processed_scripts_per_event)) {
+						$processed_scripts_per_event[$eventid] = [];
+					}
+
+					foreach ($event['hosts'] as $event_host) {
+						foreach ($hosts as $host) {
+							if (bccomp($host['hostid'], $event_host['hostid']) == 0
+									&& array_key_exists($eventid, $scripts_by_events)
+									&& !array_key_exists($scriptid, $processed_scripts_per_event[$eventid])) {
+								$size = count($scripts_by_events[$eventid]);
+								$scripts_by_events[$eventid][$size] = $script;
+
+								// Set confirmation and URL with resolved macros.
+								if (array_key_exists($eventid, $macros_data)
+										&& array_key_exists($scriptid, $macros_data[$eventid])) {
+									$macro_values = $macros_data[$eventid][$scriptid];
+
+									if (strpos($script['confirmation'], '{') !== false) {
+										$scripts_by_events[$eventid][$size]['confirmation'] =
+											$macro_values['confirmation'];
+									}
+
+									if (strpos($script['url'], '{') !== false) {
+										$scripts_by_events[$eventid][$size]['url'] = $macro_values['url'];
+									}
+								}
+
+								$processed_scripts_per_event[$eventid][$scriptid] = true;
+							}
+						}
+					}
+				}
+			}
+		}
+
+		return $scripts_by_events;
+	}
+
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
@@ -1101,6 +1392,9 @@ class CScript extends CApiService {
 						$action_scriptids[] = $scriptid;
 					}
 				}
+
+				// Remove scope from output, since it's not requested.
+				$result = $this->unsetExtraFields($result, ['scope']);
 			}
 
 			if ($action_scriptids) {
@@ -1330,48 +1624,126 @@ class CScript extends CApiService {
 	}
 
 	/**
-	 * Check for unique script names.
+	 * Check for unique script names within menu path in the input.
 	 *
-	 * @static
+	 * @param array  $scripts  Array of scripts.
+	 * @param string $method   API method "create" or "update". Default "create".
 	 *
-	 * @param array      $scripts
-	 * @param array|null $db_scripts
+	 * $scripts = [[
+	 *     'name' =>      (string)  Script name (optional for update method).
+	 *     'menu_path' => (string)  Script menu path (optional).
+	 * ]]
 	 *
-	 * @throws APIException if script names are not unique.
+	 * @throws APIException if script names within menu paths are not unique.
 	 */
-	private static function checkDuplicates(array $scripts, array $db_scripts = null): void {
-		$names = [];
+	private function checkUniqueness(array $scripts, string $method = 'create'): void {
+		if ($method === 'update') {
+			$scripts = array_filter($scripts,
+				static fn($script) => array_key_exists('name', $script) || array_key_exists('menu_path', $script)
+			);
 
-		foreach ($scripts as $script) {
-			if (!array_key_exists('name', $script)) {
-				continue;
-			}
-
-			if ($db_scripts === null || $script['name'] !== $db_scripts[$script['scriptid']]['name']) {
-				$names[] = $script['name'];
+			if (!$scripts) {
+				return;
 			}
 		}
 
-		if (!$names) {
+		foreach ($scripts as &$script) {
+			$menu_path = '';
+
+			if (array_key_exists('menu_path', $script)) {
+				$menu_path = trimPath($script['menu_path']);
+			}
+
+			// Trim preceeding and trailing slashes for comparison.
+			$menu_path = trim($menu_path, '/');
+			$script['menu_path'] = $menu_path;
+		}
+		unset($script);
+
+		$api_input_rules = $this->getValidationRules($method);
+		$api_input_rules['uniq'] = [['name', 'menu_path']];
+		$api_input_rules['fields'] = array_intersect_key($api_input_rules['fields'], array_flip(['name', 'menu_path']));
+		$api_input_rules['flags'] |= API_ALLOW_UNEXPECTED;
+
+		if (!CApiInputValidator::validate($api_input_rules, $scripts, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+	}
+
+	/**
+	 * Check for duplicate script names within menu path.
+	 *
+	 * @param array       $scripts     Array of scripts.
+	 * @param array|null  $db_scripts  Array of scripts from database.
+	 *
+	 * $scripts = [[
+	 *     'scriptid' =>  (string)  Script ID.
+	 *     'name' =>      (string)  Script name.
+	 *     'menu_path' => (string)  Script menu path (exists if scope = 1 for update method).
+	 *     'scope' =>     (string)  Script scope.
+	 * ]]
+	 *
+	 * $db_scripts = [
+	 *     <scriptid> => [
+	 *         'name' =>      (string)  Script name.
+	 *         'menu_path' => (string)  Script menu path.
+	 *         'scope' =>     (string)  Script scope.
+	 *     ]
+	 * ]
+	 *
+	 * @throws APIException if script names within menu paths have duplicates in DB.
+	 */
+	private function checkDuplicates(array $scripts, ?array $db_scripts = null): void {
+		if ($db_scripts !== null) {
+			$scripts = $this->extendFromObjects(zbx_toHash($scripts, 'scriptid'), $db_scripts, ['menu_path']);
+
+			/*
+			 * Remove unchanged scripts and continue validation only for scripts that have changed name, menu path or
+			 * scope. If scope is changed to action, menu_path will be reset to empty string and that is a change.
+			 */
+			$scripts = array_filter($scripts,
+				static fn($script) => $script['name'] !== $db_scripts[$script['scriptid']]['name']
+					|| $script['menu_path'] !== $db_scripts[$script['scriptid']]['menu_path']
+					|| ($script['scope'] !== $db_scripts[$script['scriptid']]['scope']
+						&& $script['scope'] == ZBX_SCRIPT_SCOPE_ACTION)
+			);
+
+			if (!$scripts) {
+				return;
+			}
+		}
+
+		$scripts_ex = DB::select('scripts', [
+			'output' => ['scriptid', 'name', 'menu_path'],
+			'filter' => ['name' => array_column($scripts, 'name')]
+		]);
+
+		if (!$scripts_ex) {
 			return;
 		}
 
-		$duplicates = DB::select('scripts', [
-			'output' => ['name'],
-			'filter' => ['name' => $names],
-			'limit' => 1
-		]);
+		$db_scriptids = [];
 
-		if ($duplicates) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Script "%1$s" already exists.', $duplicates[0]['name']));
+		foreach ($scripts_ex as $script) {
+			$name = self::getScriptNameAndPath($script);
+			$db_scriptids[$name] = $script['scriptid'];
+		}
+
+		foreach ($scripts as $script) {
+			$name = self::getScriptNameAndPath($script);
+
+			if ($db_scripts === null && array_key_exists($name, $db_scriptids)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Script "%1$s" already exists.', $script['name']));
+			}
+			elseif (array_key_exists($name, $db_scriptids) && bccomp($script['scriptid'], $db_scriptids[$name]) != 0) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Script "%1$s" already exists.', $script['name']));
+			}
 		}
 	}
 
 	/**
 	 * Update "script_param" table and populate script.parameters by "script_paramid" property.
 	 *
-	 * @static
-	 *
 	 * @param array      $scripts
 	 * @param string     $method
 	 * @param array|null $db_scripts
@@ -1445,8 +1817,6 @@ class CScript extends CApiService {
 	/**
 	 * Add the existing parameters to $db_scripts whether these are affected by the update.
 	 *
-	 * @static
-	 *
 	 * @param array $scripts
 	 * @param array $db_scripts
 	 */
@@ -1473,4 +1843,35 @@ class CScript extends CApiService {
 				array_diff_key($db_parameter, array_flip(['scriptid']));
 		}
 	}
+
+	/**
+	 * Helper function to combine trimmed menu path with name.
+	 *
+	 * @param array  $script  Script data.
+	 *
+	 * $script = [
+	 *     'name' =>      (string)  Script name.
+	 *     'menu_path' => (string)  Script menu path (optional).
+	 * ]
+	 *
+	 * Example:
+	 *   $script = [
+	 *       'name' =>      'ABC'
+	 *       'menu_path' => '/a/b'
+	 *   ]
+	 * Output: a/b/ABC
+	 *
+	 * @return string
+	 */
+	private static function getScriptNameAndPath(array $script): string {
+		$menu_path = '';
+
+		if (array_key_exists('menu_path', $script)) {
+			$menu_path = trimPath($script['menu_path']);
+		}
+
+		$menu_path = trim($menu_path, '/');
+
+		return $menu_path === '' ? $script['name'] : $menu_path.'/'.$script['name'];
+	}
 }
