@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,32 +29,32 @@ class CProfiler {
 	protected $slowSqlQueryTime = 0.01;
 
 	/**
-	 * Determines time for sum of all script sql query times to be considered slow.
+	 * Determines time for single Elasticsearch query to be considered slow.
 	 *
 	 * @var float
 	 */
-	protected $slowTotalSqlTime = 0.5;
-
-	/**
-	 * Determines time for script execution time to be considered slow.
-	 *
-	 * @var float
-	 */
-	protected $slowScriptTime = 1.0;
+	protected $slowElasticQueryTime = 0.01;
 
 	/**
 	 * Contains all api requests info.
 	 *
 	 * @var array
 	 */
-	protected $apiLog = array();
+	protected $apiLog = [];
 
 	/**
 	 * Contains SQL queries info.
 	 *
 	 * @var array
 	 */
-	protected $sqlQueryLog = array();
+	protected $sqlQueryLog = [];
+
+	/**
+	 * Contains Elasticsearch queries info.
+	 *
+	 * @var array
+	 */
+	protected $elasticQueryLog = [];
 
 	/**
 	 * Total time of all performed sql queries.
@@ -63,6 +63,13 @@ class CProfiler {
 	 */
 	protected $sqlTotalTime = 0.0;
 
+	/**
+	 * Total time of all performed Elasticsearch queries.
+	 *
+	 * @var float
+	 */
+	protected $elasticTotalTime = 0.0;
+
 	/**
 	 * Timestamp of profiling start.
 	 *
@@ -84,6 +91,13 @@ class CProfiler {
 	 */
 	private static $instance;
 
+	/**
+	 * Root directory path
+	 *
+	 * @var string
+	 */
+	private $root_dir;
+
 	/**
 	 * @static
 	 *
@@ -100,7 +114,9 @@ class CProfiler {
 	/**
 	 * Private constructor.
 	 */
-	private function __construct() {}
+	private function __construct() {
+		$this->root_dir = realpath(dirname(__FILE__).'/../../..');
+	}
 
 	/**
 	 * Start script profiling.
@@ -117,82 +133,116 @@ class CProfiler {
 	}
 
 	/**
-	 * Output profiling data.
+	 * Make profiling data.
+	 *
+	 * @return CPre
 	 */
-	public function show() {
+	public function make() {
 		global $DB;
 
-		$debug_str = '<a name="debug"></a>';
-		$debug_str .= '******************** '._('Script profiler').' ********************'.'<br>';
+		$debug = [];
+		$debug[] = (new CLink())->setAttribute('name', 'debug');
+		$debug[] = '******************** '._('Script profiler').' ********************';
+		$debug[] = BR();
+		$debug[] = _s('Total time: %1$s', round($this->stopTime - $this->startTime, 6));
+		$debug[] = BR();
+		$debug[] = _s('Total SQL time: %1$s', $this->sqlTotalTime);
+		$debug[] = BR();
 
-		$totalScriptTime = $this->stopTime - $this->startTime;
-		$totalTimeStr = _s('Total time: %s', round($totalScriptTime, 6));
-		if ($totalTimeStr > $this->slowScriptTime) {
-			$totalTimeStr = '<b>'.$totalTimeStr.'</b>';
+		if ($this->elasticQueryLog) {
+			$debug[] = _s('Total Elasticsearch time: %1$s', $this->elasticTotalTime);
+			$debug[] = BR();
 		}
-		$debug_str .= $totalTimeStr.'<br>';
-
-		$sqlTotalTimeStr = _s('Total SQL time: %s', $this->sqlTotalTime);
-		if ($sqlTotalTimeStr > $this->slowTotalSqlTime) {
-			$sqlTotalTimeStr = '<b>'.$sqlTotalTimeStr.'</b>';
-		}
-		$debug_str .= $sqlTotalTimeStr.'<br>';
 
 		if (isset($DB) && isset($DB['SELECT_COUNT'])) {
-			$debug_str .= _s('SQL count: %s (selects: %s | executes: %s)',
-				count($this->sqlQueryLog), $DB['SELECT_COUNT'], $DB['EXECUTE_COUNT']).'<br>';
+			$debug[] = _s('SQL count: %1$s (selects: %2$s | executes: %3$s)',
+				count($this->sqlQueryLog), $DB['SELECT_COUNT'], $DB['EXECUTE_COUNT']);
+			$debug[] = BR();
 		}
 
-		$debug_str .= _s('Peak memory usage: %s', mem2str($this->getMemoryPeak())).'<br>';
-		$debug_str .= _s('Memory limit: %s', ini_get('memory_limit')).'<br>';
-
-		$debug_str .= '<br>';
+		$debug[] = _s('Peak memory usage: %1$s', mem2str($this->getMemoryPeak()));
+		$debug[] = BR();
+		$debug[] = _s('Memory limit: %1$s', ini_get('memory_limit'));
+		$debug[] = BR();
+		$debug[] = BR();
 
 		foreach ($this->apiLog as $i => $apiCall) {
-			$debug_str .= '<div style="border-bottom: 1px dotted gray; margin-bottom: 20px;">';
 			list($class, $method, $params, $result, $file, $line) = $apiCall;
-			// api method
-			$debug_str .= '<div style="padding-bottom: 10px;">';
-			$debug_str .= ($i + 1).'. <b>'.$class.'.'.$method.'</b>'.(($file !== null) ? ' ['.$file.':'.$line.']' : '');
-			$debug_str .= '</div>';
-			// parameters
-			$debug_str .= '<table><tr><td style="width: 300px" valign="top">Parameters:';
-			$debug_str .= '<pre>'.print_r(CHtml::encode($params), true).'</pre>';
-			$debug_str .= '</td>';
-			// result
-			$debug_str .= '<td valign="top">Result:<pre>'.print_r(CHtml::encode($result), true).'</pre></td>';
 
-			$debug_str .= '</tr></table>';
-			$debug_str .= '</div>';
+			// api method
+			$debug[] = ($i + 1).'. ';
+			$debug[] = bold($class.'.'.$method);
+			$debug[] = ($file !== null ? ' ['.$file.':'.$line.']' : null);
+			$debug[] = BR();
+			$debug[] = BR();
+
+			// parameters, result
+			$debug[] = (new CTable())
+				->addRow([
+					[_('Parameters').':', BR(), print_r($params, true)],
+					[_('Result').':', BR(), print_r($result, true)]
+				]);
+
+			$debug[] = BR();
 		}
 
-		$debug_str .= '<br>';
+		$debug[] = BR();
 
 		foreach ($this->sqlQueryLog as $query) {
 			$time = $query[0];
-			$sql = htmlspecialchars($query[1], ENT_QUOTES, 'UTF-8');
 
-			if (strpos($sql, 'SELECT ') !== false) {
-				$sqlString = '<span style="color: green; font-size: 1.2em;">'.$sql.'</span>';
-			}
-			else {
-				$sqlString = '<span style="color: blue; font-size: 1.2em;">'.$sql.'</span>';
-			}
-			$sqlString = 'SQL ('.$time.'): '.$sqlString.'<br>';
+			$sql = [
+				'SQL ('.$time.'): ',
+				(new CSpan($query[1]))
+					->addClass(substr($query[1], 0, 6) === 'SELECT' ? ZBX_STYLE_GREEN : ZBX_STYLE_BLUE),
+				BR()
+			];
+
 			if ($time > $this->slowSqlQueryTime) {
-				$sqlString = '<b>'.$sqlString.'</b>';
+				$sql = bold($sql);
 			}
-			$debug_str .= $sqlString;
+			$debug[] = $sql;
 
-			$callStackString = '<span style="font-style: italic;">'.$this->formatCallStack($query[2]).'</span>'.'<br>'.'<br>';
-			$debug_str .= rtrim($callStackString, '-> ').'</span>'.'<br>'.'<br>';
+			$debug[] = $this->formatCallStack($query[2]);
+			$debug[] = BR();
+			$debug[] = BR();
 		}
 
-		$debug = new CDiv(null, 'textcolorstyles');
-		$debug->attr('name', 'zbx_debug_info');
-		$debug->attr('style', 'display: none; overflow: auto; width: 95%; border: 1px #777777 solid; margin: 4px; padding: 4px;');
-		$debug->addItem(array(BR(), new CJsScript($debug_str), BR()));
-		$debug->show();
+		$debug[] = BR();
+
+		foreach ($this->elasticQueryLog as $query) {
+			$time = $query[0];
+
+			$record = [
+				'Elasticsearch ('.$time.'): ',
+				$query[1].' ',
+				(new CSpan($query[2]))->addClass(ZBX_STYLE_BLUE),
+				BR(),
+				'Request: ',
+				(new CSpan($query[3]))->addClass(ZBX_STYLE_GREEN),
+				BR()
+			];
+
+			if ($time > $this->slowElasticQueryTime) {
+				$sql = bold($record);
+			}
+			$debug[] = $record;
+
+			$debug[] = $this->formatCallStack($query[4]);
+			$debug[] = BR();
+			$debug[] = BR();
+		}
+
+		return (new CPre())
+			->addClass(ZBX_STYLE_DEBUG_OUTPUT)
+			->addItem($debug);
+	}
+
+	/**
+	 * Output profiling data.
+	 */
+	public function show() {
+		return $this->make()->show();
 	}
 
 	/**
@@ -202,19 +252,14 @@ class CProfiler {
 	 * @param string $sql
 	 */
 	public function profileSql($time, $sql) {
-		if (!is_null(CWebUser::$data) && isset(CWebUser::$data['debug_mode'])
-				&& CWebUser::$data['debug_mode'] == GROUP_DEBUG_MODE_DISABLED) {
-			return;
-		}
-
 		$time = round($time, 6);
 
 		$this->sqlTotalTime += $time;
-		$this->sqlQueryLog[] = array(
+		$this->sqlQueryLog[] = [
 			$time,
 			$sql,
-			array_slice(debug_backtrace(), 1)
-		);
+			array_slice(debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS), 1)
+		];
 	}
 
 	/**
@@ -225,13 +270,8 @@ class CProfiler {
 	 * @param array  $params
 	 * @param array  $result
 	 */
-	public function profileApiCall($class, $method, $params, $result) {
-		if (!is_null(CWebUser::$data) && isset(CWebUser::$data['debug_mode'])
-				&& CWebUser::$data['debug_mode'] == GROUP_DEBUG_MODE_DISABLED) {
-			return;
-		}
-
-		$backtrace = debug_backtrace();
+	public function profileApiCall($class, $method, array $params, $result) {
+		$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
 
 		// Use the file name and line number from the first call to the API wrapper object.
 		// Due to a bug earlier versions of PHP 5.3 did not provide the file name and line number
@@ -245,14 +285,40 @@ class CProfiler {
 			$line = null;
 		}
 
-		$this->apiLog[] = array(
+		$this->apiLog[] = [
 			$class,
 			$method,
 			$params,
 			$result,
 			$file,
 			$line
-		);
+		];
+	}
+
+	/**
+	 * Store Elasticsearch query data.
+	 *
+	 * @param float  $time
+	 * @param string $method
+	 * @param string $endpoint
+	 * @param string $query
+	 */
+	public function profileElasticsearch($time, $method, $endpoint, $query) {
+		if (!is_null(CWebUser::$data) && isset(CWebUser::$data['debug_mode'])
+				&& CWebUser::$data['debug_mode'] == GROUP_DEBUG_MODE_DISABLED) {
+			return;
+		}
+
+		$time = round($time, 6);
+
+		$this->elasticTotalTime += $time;
+		$this->elasticQueryLog[] = [
+			$time,
+			$method,
+			$endpoint,
+			$query,
+			array_slice(debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS), 1)
+		];
 	}
 
 	/**
@@ -276,14 +342,14 @@ class CProfiler {
 	 */
 	public function formatCallStack(array $callStack = null) {
 		if (!$callStack) {
-			$callStack = debug_backtrace(false);
+			$callStack = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
 
 			// never show the call to this method
 			array_shift($callStack);
 		}
 
-		$callStackString = '';
-		$callWithFile = array();
+		$functions = [];
+		$callWithFile = [];
 
 		$callStack = array_reverse($callStack);
 		$firstCall = reset($callStack);
@@ -291,11 +357,12 @@ class CProfiler {
 		foreach ($callStack as $call) {
 			// do not show the call to the error handler function
 			if ($call['function'] != 'zbx_err_handler') {
-				if (isset($call['class'])) {
-					$callStackString .= $call['class'].$call['type'];
+				if (array_key_exists('class', $call)) {
+					$functions[] = $call['class'].$call['type'].$call['function'].'()';
+				}
+				else {
+					$functions[] = $call['function'].'()';
 				}
-
-				$callStackString .= $call['function'].'() &rarr; ';
 			}
 
 			// if the error is caused by an incorrect function call - the location of that call is contained in
@@ -303,18 +370,25 @@ class CProfiler {
 			// if it's caused by something else (like an undefined index) - the location of the call is contained in the
 			// call to the error handler function
 			// to display the location we use the last call where this information is present
-			if (isset($call['file'])) {
+			if (array_key_exists('file', $call)) {
 				$callWithFile = $call;
 			}
 		}
 
-		if ($callStackString) {
-			$path = pathinfo($firstCall['file']);
-			$callStackString = $path['basename'].':'.$firstCall['line'] . ' &rarr; '.rtrim($callStackString, '&rarr; ');
+		$callStackString = '';
+
+		if ($functions) {
+			$callStackString .= pathinfo($firstCall['file'], PATHINFO_BASENAME).':'.$firstCall['line'].' → '.
+				implode(' → ', $functions);
 		}
 
 		if ($callWithFile) {
-			$callStackString .= ' in '.$callWithFile['file'].':'.$callWithFile['line'];
+			$file_name = $callWithFile['file'];
+
+			if (substr_compare($file_name, $this->root_dir, 0, strlen($this->root_dir)) === 0) {
+				$file_name = substr($file_name, strlen($this->root_dir) + 1);
+			}
+			$callStackString .= ' in '.$file_name.':'.$callWithFile['line'];
 		}
 
 		return $callStackString;
