@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,26 +19,23 @@
 **/
 
 
+use Zabbix\Core\{
+	CModule,
+	CWidget
+};
+
 class CDashboardHelper {
 
 	/**
 	 * Get dashboard owner name.
-	 *
-	 * @static
-	 *
-	 * @param string $userid
-	 *
-	 * @return string
 	 */
-	public static function getOwnerName($userid): string {
+	public static function getOwnerName(string $userid): string {
 		$users = API::User()->get([
 			'output' => ['name', 'surname', 'username'],
 			'userids' => $userid
 		]);
 
-		$name = $users ? getUserFullname($users[0]) : _('Inaccessible user');
-
-		return $name;
+		return $users ? getUserFullname($users[0]) : _('Inaccessible user');
 	}
 
 	/**
@@ -64,14 +61,6 @@ class CDashboardHelper {
 
 	/**
 	 * Prepare widget pages for dashboard grid.
-	 *
-	 * @static
-	 *
-	 * @param array  $pages
-	 * @param string $templateid
-	 * @param bool   $with_rf_rate
-	 *
-	 * @return array
 	 */
 	public static function preparePagesForGrid(array $pages, ?string $templateid, bool $with_rf_rate): array {
 		if (!$pages) {
@@ -80,63 +69,58 @@ class CDashboardHelper {
 
 		$grid_pages = [];
 
-		$context = ($templateid === null)
-			? CWidgetConfig::CONTEXT_DASHBOARD
-			: CWidgetConfig::CONTEXT_TEMPLATE_DASHBOARD;
-
-		$known_widget_types = array_keys(CWidgetConfig::getKnownWidgetTypes($context));
-
 		foreach ($pages as $page) {
 			$grid_page_widgets = [];
 
 			CArrayHelper::sort($page['widgets'], ['y', 'x']);
 
-			foreach ($page['widgets'] as $widget) {
-				if (!in_array($widget['type'], $known_widget_types)) {
-					continue;
-				}
+			foreach ($page['widgets'] as $widget_data) {
+				$grid_page_widget = [
+					'widgetid' => $widget_data['widgetid'],
+					'type' => $widget_data['type'],
+					'name' => $widget_data['name'],
+					'view_mode' => $widget_data['view_mode'],
+					'pos' => [
+						'x' => (int) $widget_data['x'],
+						'y' => (int) $widget_data['y'],
+						'width' => (int) $widget_data['width'],
+						'height' => (int) $widget_data['height']
+					],
+					'rf_rate' => 0,
+					'fields' => []
+				];
 
-				$widgetid = $widget['widgetid'];
-				$fields_orig = self::convertWidgetFields($widget['fields']);
+				/** @var CWidget $widget */
+				$widget = APP::ModuleManager()->getModule($widget_data['type']);
 
-				// Transforms corrupted data to default values.
-				$widget_form = CWidgetConfig::getForm($widget['type'], json_encode($fields_orig), $templateid);
-				$widget_form->validate();
-				$fields = $widget_form->getFieldsData();
+				if ($widget !== null && $widget->getType() === CModule::TYPE_WIDGET
+						&& ($templateid === null || $widget->hasTemplateSupport())) {
+					$grid_page_widget['fields'] = self::convertWidgetFields($widget_data['fields']);
 
-				if ($with_rf_rate) {
-					$rf_rate = (int) CProfile::get('web.dashboard.widget.rf_rate', -1, $widgetid);
+					if ($with_rf_rate) {
+						$rf_rate = (int) CProfile::get('web.dashboard.widget.rf_rate', -1, $widget_data['widgetid']);
 
-					if ($rf_rate == -1) {
-						if ($context === CWidgetConfig::CONTEXT_DASHBOARD) {
-							$rf_rate = ($fields['rf_rate'] == -1)
-								? CWidgetConfig::getDefaultRfRate($widget['type'])
-								: $fields['rf_rate'];
-						}
-						else {
-							$rf_rate = CWidgetConfig::getDefaultRfRate($widget['type']);
+						if ($rf_rate == -1) {
+							if ($templateid === null) {
+								// Transforms corrupted data to default values.
+								$widget_form = $widget->getForm($grid_page_widget['fields'], $templateid);
+								$widget_form->validate();
+								$values = $widget_form->getFieldsValues();
+
+								$rf_rate = $values['rf_rate'] == -1
+									? $widget->getDefaultRefreshRate()
+									: $values['rf_rate'];
+							}
+							else {
+								$rf_rate = $widget->getDefaultRefreshRate();
+							}
 						}
+
+						$grid_page_widget['rf_rate'] = $rf_rate;
 					}
 				}
-				else {
-					$rf_rate = 0;
-				}
 
-				$grid_page_widgets[] = [
-					'widgetid' => $widgetid,
-					'type' => $widget['type'],
-					'name' => $widget['name'],
-					'view_mode' => $widget['view_mode'],
-					'pos' => [
-						'x' => (int) $widget['x'],
-						'y' => (int) $widget['y'],
-						'width' => (int) $widget['width'],
-						'height' => (int) $widget['height']
-					],
-					'rf_rate' => $rf_rate,
-					'fields' => $fields_orig,
-					'configuration' => CWidgetConfig::getConfiguration($widget['type'], $fields, $widget['view_mode'])
-				];
+				$grid_page_widgets[] = $grid_page_widget;
 			}
 
 			$grid_pages[] = [
@@ -173,7 +157,10 @@ class CDashboardHelper {
 			ZBX_WIDGET_FIELD_TYPE_GRAPH_PROTOTYPE => [],
 			ZBX_WIDGET_FIELD_TYPE_MAP => [],
 			ZBX_WIDGET_FIELD_TYPE_SERVICE => [],
-			ZBX_WIDGET_FIELD_TYPE_SLA => []
+			ZBX_WIDGET_FIELD_TYPE_SLA => [],
+			ZBX_WIDGET_FIELD_TYPE_USER => [],
+			ZBX_WIDGET_FIELD_TYPE_ACTION => [],
+			ZBX_WIDGET_FIELD_TYPE_MEDIA_TYPE => []
 		];
 
 		foreach ($pages as $p_index => $page) {
@@ -315,6 +302,48 @@ class CDashboardHelper {
 			}
 		}
 
+		if ($ids[ZBX_WIDGET_FIELD_TYPE_USER]) {
+			$db_users = API::User()->get([
+				'output' => [],
+				'userids' => array_keys($ids[ZBX_WIDGET_FIELD_TYPE_USER]),
+				'preservekeys' => true
+			]);
+
+			foreach ($ids[ZBX_WIDGET_FIELD_TYPE_USER] as $userid => $indexes) {
+				if (!array_key_exists($userid, $db_users)) {
+					$inaccessible_indexes = array_merge($inaccessible_indexes, $indexes);
+				}
+			}
+		}
+
+		if ($ids[ZBX_WIDGET_FIELD_TYPE_ACTION]) {
+			$db_actions = API::Action()->get([
+				'output' => [],
+				'actionids' => array_keys($ids[ZBX_WIDGET_FIELD_TYPE_ACTION]),
+				'preservekeys' => true
+			]);
+
+			foreach ($ids[ZBX_WIDGET_FIELD_TYPE_ACTION] as $actionid => $indexes) {
+				if (!array_key_exists($actionid, $db_actions)) {
+					$inaccessible_indexes = array_merge($inaccessible_indexes, $indexes);
+				}
+			}
+		}
+
+		if ($ids[ZBX_WIDGET_FIELD_TYPE_MEDIA_TYPE]) {
+			$db_media_types = API::MediaType()->get([
+				'output' => [],
+				'mediatypeids' => array_keys($ids[ZBX_WIDGET_FIELD_TYPE_MEDIA_TYPE]),
+				'preservekeys' => true
+			]);
+
+			foreach ($ids[ZBX_WIDGET_FIELD_TYPE_MEDIA_TYPE] as $mediatypeid => $indexes) {
+				if (!array_key_exists($mediatypeid, $db_media_types)) {
+					$inaccessible_indexes = array_merge($inaccessible_indexes, $indexes);
+				}
+			}
+		}
+
 		foreach ($inaccessible_indexes as $index) {
 			unset($pages[$index['p']]['widgets'][$index['w']]['fields'][$index['f']]);
 		}
@@ -358,8 +387,11 @@ class CDashboardHelper {
 	 */
 	public static function hasTimeSelector(array $pages): bool {
 		foreach ($pages as $page) {
-			foreach ($page['widgets'] as $widget) {
-				if (CWidgetConfig::usesTimeSelector($widget['type'], $widget['fields'])) {
+			foreach ($page['widgets'] as $widget_data) {
+				$widget = App::ModuleManager()->getModule($widget_data['type']);
+
+				if ($widget !== null && $widget->getType() === CModule::TYPE_WIDGET
+						&& $widget->usesTimeSelector($widget_data['fields'])) {
 					return true;
 				}
 			}
@@ -411,10 +443,10 @@ class CDashboardHelper {
 				$dashboard_page['widgets'] = [];
 			}
 
-			foreach ($dashboard_page['widgets'] as $widget_index => &$widget) {
+			foreach ($dashboard_page['widgets'] as $widget_index => &$widget_data) {
 				$widget_errors = [];
 
-				if (!array_key_exists('pos', $widget)) {
+				if (!array_key_exists('pos', $widget_data)) {
 					$widget_errors[] = _s('Invalid parameter "%1$s": %2$s.',
 						'pages['.$dashboard_page_index.'][widgets]['.$widget_index.']',
 						_s('the parameter "%1$s" is missing', 'pos')
@@ -422,7 +454,7 @@ class CDashboardHelper {
 				}
 				else {
 					foreach (['x', 'y', 'width', 'height'] as $field) {
-						if (!is_array($widget['pos']) || !array_key_exists($field, $widget['pos'])) {
+						if (!is_array($widget_data['pos']) || !array_key_exists($field, $widget_data['pos'])) {
 							$widget_errors[] = _s('Invalid parameter "%1$s": %2$s.',
 								'pages['.$dashboard_page_index.'][widgets]['.$widget_index.'][pos]',
 								_s('the parameter "%1$s" is missing', $field)
@@ -432,7 +464,7 @@ class CDashboardHelper {
 				}
 
 				foreach (['type', 'name', 'view_mode'] as $field) {
-					if (!array_key_exists($field, $widget)) {
+					if (!array_key_exists($field, $widget_data)) {
 						$widget_errors[] = _s('Invalid parameter "%1$s": %2$s.',
 							'pages['.$dashboard_page_index.'][widgets]['.$widget_index.']',
 							_s('the parameter "%1$s" is missing', $field)
@@ -446,28 +478,45 @@ class CDashboardHelper {
 					break 2;
 				}
 
-				$widget_fields = array_key_exists('fields', $widget) ? $widget['fields'] : '{}';
-				$widget['form'] = CWidgetConfig::getForm($widget['type'], $widget_fields, $templateid);
-				unset($widget['fields']);
+				$widget_fields = array_key_exists('fields', $widget_data) ? $widget_data['fields'] : [];
+				unset($widget_data['fields']);
 
-				if ($widget_errors = $widget['form']->validate()) {
-					if ($widget['name'] === '') {
-						$context = $templateid !== null
-							? CWidgetConfig::CONTEXT_TEMPLATE_DASHBOARD
-							: CWidgetConfig::CONTEXT_DASHBOARD;
+				if ($widget_data['type'] === ZBX_WIDGET_INACCESSIBLE) {
+					continue;
+				}
 
-						$widget_name = CWidgetConfig::getKnownWidgetTypes($context)[$widget['type']];
+				$widget = APP::ModuleManager()->getModule($widget_data['type']);
+
+				if ($widget === null || $widget->getType() !== CModule::TYPE_WIDGET) {
+					if ($widget_data['name'] !== '') {
+						$widget_name = $widget_data['name'];
 					}
 					else {
-						$widget_name = $widget['name'];
+						$widget_name = 'pages['.$dashboard_page_index.'][widgets]['.$widget_index.']';
 					}
 
+					$errors[] = _s('Cannot save widget "%1$s".', $widget_name).' '._('Inaccessible widget type.');
+
+					continue;
+				}
+
+				$widget_name = $widget_data['name'] !== '' ? $widget_data['name'] : $widget->getDefaultName();
+
+				if ($templateid !== null && !$widget->hasTemplateSupport()) {
+					$errors[] = _s('Cannot save widget "%1$s".', $widget_name).' '._('Inaccessible widget type.');
+
+					continue;
+				}
+
+				$widget_data['form'] = $widget->getForm($widget_fields, $templateid);
+
+				if ($widget_errors = $widget_data['form']->validate()) {
 					foreach ($widget_errors as $error) {
 						$errors[] = _s('Cannot save widget "%1$s".', $widget_name).' '.$error;
 					}
 				}
 			}
-			unset($widget);
+			unset($widget_data);
 		}
 		unset($dashboard_page);
 
@@ -568,4 +617,52 @@ class CDashboardHelper {
 
 		return $dashboards;
 	}
+
+	public static function getWidgetLastType(bool $for_template_dashboard_only = false): ?string {
+		$known_widgets = APP::ModuleManager()->getWidgets($for_template_dashboard_only);
+
+		$widget_last_type = CProfile::get('web.dashboard.last_widget_type');
+
+		if (!array_key_exists($widget_last_type, $known_widgets)) {
+			$current_types = [];
+			$deprecated_types = [];
+
+			/** @var CWidget $widget */
+			foreach ($known_widgets as $widget) {
+				if (!$widget->isDeprecated()) {
+					$current_types[$widget->getId()] = $widget->getDefaultName();
+				}
+				else {
+					$deprecated_types[$widget->getId()] = $widget->getDefaultName();
+				}
+			}
+
+			natcasesort($current_types);
+			natcasesort($deprecated_types);
+
+			if ($current_types) {
+				$widget_last_type = array_key_first($current_types);
+			}
+			elseif ($deprecated_types) {
+				$widget_last_type = array_key_first($deprecated_types);
+			}
+			else {
+				$widget_last_type = null;
+			}
+		}
+
+		return $widget_last_type;
+	}
+
+	/**
+	 * @throws JsonException
+	 */
+	public static function getConfigurationHash(array $dashboard, array $widget_defaults): string {
+		ksort($widget_defaults);
+
+		return md5(json_encode([
+			array_intersect_key($dashboard, array_flip(['name', 'display_period', 'auto_start', 'pages'])),
+			$widget_defaults
+		], JSON_THROW_ON_ERROR));
+	}
 }
