@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -76,10 +76,8 @@ class CTimezoneHelper {
 	 *
 	 * @return bool
 	 */
-	public static function isSupported(?string $timezone): bool {
-		return $timezone === null
-			? array_key_exists(ZBX_DEFAULT_TIMEZONE, self::getList())
-			: array_key_exists($timezone, self::getList());
+	public static function isSupported(string $timezone): bool {
+		return array_key_exists($timezone, self::getList());
 	}
 
 	/**
