@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -100,7 +100,6 @@ class CFilter extends CDiv {
 			->setId(uniqid('filter_'));
 
 		$this->form = (new CForm('get'))
-			->cleanItems()
 			->setAttribute('name', self::FORM_NAME);
 
 		$this->reset_url = new CUrl();
@@ -122,8 +121,8 @@ class CFilter extends CDiv {
 	 *
 	 * @return CFilter
 	 */
-	public function addVar($name, $value) {
-		$this->form->addVar($name, $value);
+	public function addVar($name, $value, $id = null) {
+		$this->form->addVar($name, $value, $id);
 
 		return $this;
 	}
