@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,13 +21,6 @@
 
 class CTextArea extends CTag {
 
-	/**
-	 * The "&" symbol in the textarea should be encoded.
-	 *
-	 * @var int
-	 */
-	protected $encStrategy = self::ENC_ALL;
-
 	/**
 	 * Init textarea.
 	 *
@@ -84,6 +77,12 @@ class CTextArea extends CTag {
 		return $this;
 	}
 
+	public function disableSpellcheck(): self {
+		$this->setAttribute('spellcheck', 'false');
+
+		return $this;
+	}
+
 	public function setWidth($value) {
 		$this->addStyle('width: '.$value.'px;');
 		return $this;
