@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -56,8 +56,6 @@ class C10ImportConverter extends CConverter {
 
 		$content = $this->convertSysmaps($content);
 
-		$content = $this->convertScreens($content);
-
 		$content = $this->filterDuplicateGroups($content);
 		$content = $this->filterDuplicateTriggers($content);
 		$content = $this->filterDuplicateGraphs($content);
@@ -140,7 +138,7 @@ class C10ImportConverter extends CConverter {
 		}
 
 		foreach ($content['hosts'] as &$host) {
-			$host = CArrayHelper::renameKeys($host, ['name' => 'host']);
+			$host['host'] = $host['name'];
 			$host = $this->convertHostInterfaces($host);
 			$host = $this->convertHostProfiles($host);
 			$host = $this->convertHostApplications($host);
@@ -148,10 +146,12 @@ class C10ImportConverter extends CConverter {
 			$host = $this->convertHostTriggers($host, $host['host']);
 			$host = $this->convertHostGraphs($host, $host['host']);
 			$host = $this->convertHostMacros($host);
+			$host = $this->convertProxy($host);
 
 			$host = $this->wrapChildren($host, 'templates', 'name');
 			$host = $this->wrapChildren($host, 'groups', 'name');
 
+			unset($host['useipmi']);
 			unset($host['useip']);
 			unset($host['ip']);
 			unset($host['dns']);
@@ -188,7 +188,7 @@ class C10ImportConverter extends CConverter {
 		}
 
 		foreach ($content['templates'] as &$template) {
-			$template = CArrayHelper::renameKeys($template, ['name' => 'template']);
+			$template['template'] = $template['name'];
 			$template = $this->convertHostApplications($template);
 			$template = $this->convertHostItems($template);
 			$template = $this->convertHostTriggers($template, $template['template']);
@@ -226,12 +226,12 @@ class C10ImportConverter extends CConverter {
 		// create an agent interface from the host properties
 		if (isset($host['useip']) && isset($host['ip']) && isset($host['dns']) && isset($host['port'])) {
 			$agentInterface = [
-				'type' => INTERFACE_TYPE_AGENT,
+				'type' => (string) INTERFACE_TYPE_AGENT,
 				'useip' => $host['useip'],
 				'ip' => $host['ip'],
 				'dns' => $host['dns'],
 				'port' => $host['port'],
-				'default' => INTERFACE_PRIMARY,
+				'default' => (string) INTERFACE_PRIMARY,
 				'interface_ref' => 'if'.$i
 			];
 			$interfaces[] = $agentInterface;
@@ -256,38 +256,58 @@ class C10ImportConverter extends CConverter {
 				}
 			}
 
-			// if a least one IPMI item exists on a host, create an IPMI interface
+			// If a least one IPMI item exists on a host, create an IPMI interface.
 			if ($hasIpmiItem) {
+				if (array_key_exists('ipmi_ip', $host) && $host['ipmi_ip'] !== '') {
+					$ip_parser = new CIPParser(['v6' => ZBX_HAVE_IPV6]);
+
+					if ($ip_parser->parse($host['ipmi_ip']) == CParser::PARSE_SUCCESS) {
+						$ip = $host['ipmi_ip'];
+						$dns = '';
+						$useip = (string) INTERFACE_USE_IP;
+					}
+					else {
+						$ip = '';
+						$dns = $host['ipmi_ip'];
+						$useip = (string) INTERFACE_USE_DNS;
+					}
+				}
+				else {
+					$ip = $host['ip'];
+					$dns = '';
+					$useip = (string) INTERFACE_USE_IP;
+				}
+
 				$ipmiInterface = [
-					'type' => INTERFACE_TYPE_IPMI,
-					'useip' => INTERFACE_USE_IP,
-					'ip' => (array_key_exists('ipmi_ip', $host) && $host['ipmi_ip'] !== '')
-						? $host['ipmi_ip'] : $host['ip'],
-					'dns' => '',
+					'type' => (string) INTERFACE_TYPE_IPMI,
+					'useip' => $useip,
+					'ip' => $ip,
+					'dns' => $dns,
 					'port' => (array_key_exists('ipmi_port', $host) && $host['ipmi_port'] !== '')
 						? $host['ipmi_port'] : '623',
-					'default' => INTERFACE_PRIMARY,
+					'default' => (string) INTERFACE_PRIMARY,
 					'interface_ref' => 'if'.$i
 				];
 				$interfaces[] = $ipmiInterface;
 				$i++;
 			}
 
-			// if SNMP item exist, create an SNMP interface for each SNMP item port.
+			// If SNMP item exist, create an SNMP interface for each SNMP item port.
 			if ($snmpItems) {
 				$snmpInterfaces = [];
+
 				foreach ($snmpItems as $item) {
 					if (!isset($item['snmp_port']) || isset($snmpInterfaces[$item['snmp_port']])) {
 						continue;
 					}
 
 					$snmpInterface = [
-						'type' => INTERFACE_TYPE_SNMP,
+						'type' => (string) INTERFACE_TYPE_SNMP,
 						'useip' => $host['useip'],
 						'ip' => $host['ip'],
 						'dns' => $host['dns'],
 						'port' => $item['snmp_port'],
-						'default' => (count($snmpInterfaces)) ? INTERFACE_SECONDARY : INTERFACE_PRIMARY,
+						'default' => (count($snmpInterfaces)) ? (string) INTERFACE_SECONDARY : (string) INTERFACE_PRIMARY,
 						'interface_ref' => 'if'.$i
 					];
 					$snmpInterfaces[$item['snmp_port']] = $snmpInterface;
@@ -304,26 +324,32 @@ class C10ImportConverter extends CConverter {
 		// map items to new interfaces
 		if (isset($host['items']) && $host['items']) {
 			foreach ($host['items'] as &$item) {
-				if (!isset($item['type'])) {
+				if (!array_key_exists('type', $item)) {
 					continue;
 				}
 
-				$interfaceType = itemTypeInterface($item['type']);
-				switch ($interfaceType) {
-					case INTERFACE_TYPE_AGENT:
-					case INTERFACE_TYPE_ANY:
-						$item['interface_ref'] = $agentInterface['interface_ref'];
+				// 1.8 till 4.4 uses the old item types.
+				if (in_array($item['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C, ITEM_TYPE_SNMPV3])) {
+					$interface_type = INTERFACE_TYPE_SNMP;
+				}
+				else {
+					$interface_type = itemTypeInterface($item['type']);
+				}
 
+				switch ($interface_type) {
+					case INTERFACE_TYPE_AGENT:
+					case INTERFACE_TYPE_OPT:
+						$item['interface_ref'] = $agentInterface['interface_ref'];
 						break;
+
 					case INTERFACE_TYPE_IPMI:
 						$item['interface_ref'] = $ipmiInterface['interface_ref'];
-
 						break;
+
 					case INTERFACE_TYPE_SNMP:
 						if (isset($item['snmp_port'])) {
 							$item['interface_ref'] = $snmpInterfaces[$item['snmp_port']]['interface_ref'];
 						}
-
 						break;
 				}
 			}
@@ -341,14 +367,14 @@ class C10ImportConverter extends CConverter {
 	 * @return array
 	 */
 	protected function convertHostProfiles(array $host) {
-		$host['inventory'] = ['inventory_mode' => HOST_INVENTORY_DISABLED];
+		$host['inventory'] = ['inventory_mode' => (string) HOST_INVENTORY_DISABLED];
 
 		// rename and merge profile fields
 		if (array_key_exists('host_profile', $host)) {
 			foreach ($host['host_profile'] as $key => $value) {
 				$host['inventory'][$this->getNewProfileName($key)] = $value;
 			}
-			$host['inventory']['inventory_mode'] = HOST_INVENTORY_MANUAL;
+			$host['inventory']['inventory_mode'] = (string) HOST_INVENTORY_MANUAL;
 		}
 
 		if (array_key_exists('host_profiles_ext', $host)) {
@@ -364,7 +390,7 @@ class C10ImportConverter extends CConverter {
 					$host['inventory'][$key] = $value;
 				}
 			}
-			$host['inventory']['inventory_mode'] = HOST_INVENTORY_MANUAL;
+			$host['inventory']['inventory_mode'] = (string) HOST_INVENTORY_MANUAL;
 		}
 
 		return $host;
@@ -420,7 +446,7 @@ class C10ImportConverter extends CConverter {
 			'poc_1_phone_2' => 'poc_1_phone_b',
 			'poc_2_phone_1' => 'poc_2_phone_a',
 			'poc_2_phone_2' => 'poc_2_phone_b',
-			'device_notes' => 'notes',
+			'device_notes' => 'notes'
 		];
 
 		return array_key_exists($key, $map) ? $map[$key] : $key;
@@ -652,13 +678,22 @@ class C10ImportConverter extends CConverter {
 		}
 
 		foreach ($host['items'] as &$item) {
-			$item = CArrayHelper::renameKeys($item, ['description' => 'name']);
+			$item = CArrayHelper::renameKeys($item, ['trapper_hosts' => 'allowed_hosts', 'snmp_port' => 'port']);
+			$item['name'] = $item['description'];
 
 			// convert simple check keys
 			$item['key'] = $this->itemKeyConverter->convert($item['key']);
 
 			$item = $this->wrapChildren($item, 'applications', 'name');
 
+			$item = $this->convertItemValueMap($item);
+
+			// Add tag inventory_link.
+			$item['inventory_link'] = '';
+			$item['snmpv3_contextname'] = '';
+			$item['snmpv3_authprotocol'] = '';
+			$item['snmpv3_privprotocol'] = '';
+
 			unset($item['lastlogsize']);
 		}
 		unset($item);
@@ -691,16 +726,14 @@ class C10ImportConverter extends CConverter {
 			$graph = $this->convertGraphItemReference($graph, 'ymin_item_1');
 			$graph = $this->convertGraphItemReference($graph, 'ymax_item_1');
 
-			if ($graph['type'] == GRAPH_TYPE_NORMAL || $graph['type'] == GRAPH_TYPE_STACKED) {
-				unset($graph['show_legend']);
-			}
-
 			foreach ($graph['graph_items'] as &$graphItem) {
 				$graphItem = $this->convertGraphItemReference($graphItem, 'item', $hostName);
 
 				unset($graphItem['periods_cnt']);
 			}
 			unset($graphItem);
+
+			$graph['graph_items'] = array_values($graph['graph_items']);
 		}
 		unset($graph);
 
@@ -721,8 +754,6 @@ class C10ImportConverter extends CConverter {
 			$array[$key] = [];
 		}
 		else {
-			$colon_pos = strpos($array[$key], ':');
-
 			list ($host, $item_key) = explode(':', $array[$key], 2);
 
 			// replace host macros with the host name
@@ -850,6 +881,10 @@ class C10ImportConverter extends CConverter {
 				unset($link);
 			}
 
+			if (!array_key_exists('backgroundid', $map)) {
+				$map['backgroundid'] = '';
+			}
+
 			$map = CArrayHelper::renameKeys($map, ['backgroundid' => 'background']);
 		}
 		unset($map);
@@ -857,37 +892,6 @@ class C10ImportConverter extends CConverter {
 		return $content;
 	}
 
-	/**
-	 * Convert screen elements.
-	 *
-	 * @param array $content
-	 *
-	 * @return array
-	 */
-	protected function convertScreens(array $content) {
-		if (!isset($content['screens']) || !$content['screens']) {
-			return $content;
-		}
-
-		foreach ($content['screens'] as &$screen) {
-			$screen = CArrayHelper::renameKeys($screen, ['screenitems' => 'screen_items']);
-
-			if (isset($screen['screen_items']) && $screen['screen_items']) {
-				foreach ($screen['screen_items'] as &$screenItem) {
-					$screenItem = CArrayHelper::renameKeys($screenItem, ['resourceid' => 'resource']);
-
-					if (isset($screenItem['resource']) && $screenItem['resource'] !== '0') {
-						$screenItem['resource'] = CArrayHelper::renameKeys($screenItem['resource'], ['key_' => 'key']);
-					}
-				}
-				unset($screenItem);
-			}
-		}
-		unset($screen);
-
-		return $content;
-	}
-
 	/**
 	 * Merges all of the values from each element of $source stored in the $key property to the $key property of $target.
 	 *
@@ -937,4 +941,45 @@ class C10ImportConverter extends CConverter {
 
 		return $array;
 	}
+
+	/**
+	 * Convert old proxy format to new proxy format.
+	 *
+	 * @param array $host  Import data.
+	 *
+	 * @return array
+	 */
+	protected function convertProxy(array $host) {
+		$host['proxy'] = [];
+
+		// If ID is zero, no proxy was assigned to host. Thus it is not name.
+		if ($host['proxy_hostid'] != 0) {
+			$host['proxy']['name'] = $host['proxy_hostid'];
+		}
+
+		unset($host['proxy_hostid']);
+
+		return $host;
+	}
+
+	/**
+	 * Convert valuemap for items.
+	 *
+	 * @param array $item  Import data.
+	 *
+	 * @return array
+	 */
+	protected function convertItemValueMap(array $item) {
+		$item['valuemap'] = [];
+		if (array_key_exists('valuemapid', $item)) {
+			// If ID is zero, no value maps were assigned to item. Thus it is not name.
+			if ($item['valuemapid'] != 0) {
+				$item['valuemap']['name'] = $item['valuemapid'];
+			}
+
+			unset($item['valuemapid']);
+		}
+
+		return $item;
+	}
 }
