@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,10 +27,6 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 * @var array
 	 */
 	protected $configs = [
-		'scriptConfirmation' => [
-			'types' => ['host', 'interfaceWithoutPort', 'user', 'user_data'],
-			'method' => 'resolveTexts'
-		],
 		'httpTestName' => [
 			'types' => ['host', 'interfaceWithoutPort', 'user'],
 			'method' => 'resolveTexts'
@@ -351,7 +347,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		// Replace macros to value.
 		foreach (array_keys($macros) as $hostid) {
 			foreach ($data[$hostid] as &$text) {
-				$matched_macros = $this->getMacroPositions($text, $types);
+				$matched_macros = self::getMacroPositions($text, $types);
 
 				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 					$text = substr_replace($text, $macros[$hostid][$macro], $pos, strlen($macro));
@@ -451,8 +447,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					if (array_key_exists($data['f_num'], $functionids)) {
 						$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
 							'token' => $token,
-							'function' => $data['function'],
-							'parameters' => $data['parameters']
+							'macrofunc' => $data['macrofunc']
 						];
 					}
 				}
@@ -511,7 +506,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		foreach ($macro_values as $triggerid => $foo) {
 			$trigger = &$triggers[$triggerid];
 
-			$matched_macros = $this->getMacroPositions($trigger['description'], $types);
+			$matched_macros = self::getMacroPositions($trigger['description'], $types);
 
 			foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 				if (array_key_exists($macro, $macro_values[$triggerid])) {
@@ -615,8 +610,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				if (array_key_exists($data['f_num'], $functionids)) {
 					$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
 						'token' => $token,
-						'function' => $data['function'],
-						'parameters' => $data['parameters']
+						'macrofunc' => $data['macrofunc']
 					];
 				}
 			}
@@ -663,7 +657,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			$trigger = &$triggers[$triggerid];
 
 			foreach ($options['sources'] as $source) {
-				$matched_macros = $this->getMacroPositions($trigger[$source], $types);
+				$matched_macros = self::getMacroPositions($trigger[$source], $types);
 
 				if ($options['html']) {
 					$macro_string = [];
@@ -788,6 +782,154 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			}
 		}
 
+		foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
+					'token' => $token,
+					'macrofunc' => $data['macrofunc']
+				];
+			}
+		}
+
+		if ($matched_macros['usermacros']) {
+			$usermacros[$triggerid] = ['hostids' => [], 'macros' => $matched_macros['usermacros']];
+		}
+
+		// Get macro value.
+		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
+		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
+		$macro_values = $this->getItemMacros($macros['item'], $macro_values);
+		$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
+
+		if ($usermacros) {
+			// Get hosts for triggers.
+			$db_triggers = API::Trigger()->get([
+				'output' => [],
+				'selectHosts' => ['hostid'],
+				'triggerids' => array_keys($usermacros),
+				'preservekeys' => true
+			]);
+
+			foreach ($usermacros as $triggerid => &$usermacros_data) {
+				if (array_key_exists($triggerid, $db_triggers)) {
+					$usermacros_data['hostids'] = zbx_objectValues($db_triggers[$triggerid]['hosts'], 'hostid');
+				}
+			}
+			unset($usermacros_data);
+
+			// Get user macros values.
+			foreach ($this->getUserMacros($usermacros) as $triggerid => $usermacros_data) {
+				$macro_values[$triggerid] = array_key_exists($triggerid, $macro_values)
+					? array_merge($macro_values[$triggerid], $usermacros_data['macros'])
+					: $usermacros_data['macros'];
+			}
+		}
+
+		$types = $this->transformToPositionTypes($types);
+
+		$matched_macros = self::getMacroPositions($trigger['url'], $types);
+
+		$url = $trigger['url'];
+		foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+			$url = substr_replace($url, $macro_values[$triggerid][$macro], $pos, strlen($macro));
+		}
+
+		return true;
+	}
+
+	/**
+	 * Resolve macros in trigger URL name.
+	 *
+	 * @param array  $trigger
+	 * @param string $trigger['triggerid']
+	 * @param string $trigger['expression']
+	 * @param string $trigger['url_name']
+	 * @param string $trigger['eventid']     (optional)
+	 * @param string $url
+	 *
+	 * @return bool
+	 */
+	public function resolveTriggerUrlName(array $trigger, &$url_name) {
+		$macros = [
+			'host' => [],
+			'interface' => [],
+			'item' => [],
+			'event' => [],
+			'log' => []
+		];
+		$usermacros = [];
+		$macro_values = [];
+
+		$types = [
+			'macros' => [
+				'trigger' => ['{TRIGGER.ID}'],
+				'event' => ['{EVENT.ID}']
+			],
+			'macros_n' => [
+				'host' => ['{HOST.ID}', '{HOST.HOST}', '{HOST.NAME}'],
+				'interface' => ['{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
+			],
+			'macro_funcs_n' => [
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+			],
+			'usermacros' => true
+		];
+
+		$triggerid = $trigger['triggerid'];
+
+		// Find macros.
+		$functionids = $this->findFunctions($trigger['expression']);
+		$matched_macros = self::extractMacros([$trigger['url_name']], $types);
+
+		foreach ($matched_macros['macros']['trigger'] as $macro) {
+			$macro_values[$triggerid][$macro] = $triggerid;
+		}
+
+		foreach ($matched_macros['macros']['event'] as $macro) {
+			if (!array_key_exists('eventid', $trigger) && $macro === '{EVENT.ID}') {
+				return false;
+			}
+			$macro_values[$triggerid][$macro] = $trigger['eventid'];
+		}
+
+		foreach ($matched_macros['macros_n']['host'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['host'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+			}
+		}
+
+		foreach ($matched_macros['macros_n']['interface'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['interface'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+			}
+		}
+
+		foreach ($matched_macros['macros_n']['item'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+			}
+		}
+
+		foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+			}
+		}
+
 		foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -836,11 +978,11 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 		$types = $this->transformToPositionTypes($types);
 
-		$matched_macros = $this->getMacroPositions($trigger['url'], $types);
+		$matched_macros = self::getMacroPositions($trigger['url_name'], $types);
 
-		$url = $trigger['url'];
+		$url_name = $trigger['url_name'];
 		foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-			$url = substr_replace($url, $macro_values[$triggerid][$macro], $pos, strlen($macro));
+			$url_name = substr_replace($url_name, $macro_values[$triggerid][$macro], $pos, strlen($macro));
 		}
 
 		return true;
@@ -1238,7 +1380,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	}
 
 	/**
-	 * Resolve {HOST.HOST<1-9} and empty plaseholders in the expression macros.
+	 * Resolve {HOST.HOST<1-9} and empty placeholders in the expression macros.
 	 *   For example:
 	 *     {$last(/ /key)} => {$last(/Zabbix server/key)}
 	 *     {$last(/MySQL server/key)} => {$last(/MySQL server/key)}
@@ -1773,7 +1915,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		// Replace macros to value.
 		foreach (array_keys($macro_values) as $key) {
 			foreach ($options['sources'] as $source) {
-				$matched_macros = $this->getMacroPositions($data[$key][$source], $types);
+				$matched_macros = self::getMacroPositions($data[$key][$source], $types);
 
 				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 					$data[$key][$source] =
@@ -1821,11 +1963,9 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				: $usermacros_data['macros'];
 		}
 
-		$types = $this->transformToPositionTypes($types);
-
 		// Replace macros to value.
 		foreach ($macro_values as $key => $macros) {
-			$function = $this->resolveFunctionMacros($functions[$key]['function_string'], $macros, $types);
+			$function = $this->resolveFunctionMacros($functions[$key]['function_string'], $macros);
 			$function = substr_replace($function, TRIGGER_QUERY_PLACEHOLDER, $functions[$key]['function_query_pos'], 8);
 			$functions[$key]['parameter'] = substr($function, strlen($functions[$key]['function']) + 1, -1);
 		}
@@ -2332,6 +2472,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			if ($step['params'] !== '') {
 				$texts[] = $step['params'];
 			}
+
 			if ($step['error_handler_params'] !== '') {
 				$texts[] = $step['error_handler_params'];
 			}
@@ -2486,4 +2627,412 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 		return $events;
 	}
+
+	/**
+	 * Resolve macros for manual host action scripts. Resolves host macros, interface macros, inventory, user macros
+	 * and user data macros.
+	 *
+	 * @param array $data                        Array of unersolved macros.
+	 * @param array $data[<hostid>]              Array of scripts. Contains script ID as keys.
+	 * @param array $data[<hostid>][<scriptid>]  Script fields to resolve macros for.
+	 *
+	 * Example input:
+	 *     array (
+	 *         10084 => array (
+	 *             57 => array (
+	 *                 'confirmation' => 'Are you sure you want to edit {HOST.HOST} now?',
+	 *                 'url' => 'http://zabbix/ui/zabbix.php?action=host.edit&hostid={HOST.ID}'
+	 *             ),
+	 *             61 => array(
+	 *                 'confirmation' => 'Hello, {USER.FULLNAME}! Execute script?'
+	 *             )
+	 *         )
+	 *     )
+	 *
+	 * Output:
+	 *     array (
+	 *         10084 => array (
+	 *             57 => array (
+	 *                 'confirmation' => 'Are you sure you want to edit Zabbix server now?',
+	 *                 'url' => 'http://zabbix/ui/zabbix.php?action=host.edit&hostid=10084'
+	 *             ),
+	 *             61 => array (
+	 *                 'confirmation' => 'Hello, Zabbix Administrator! Execute script?'
+	 *             )
+	 *         )
+	 *     )
+	 *
+	 * @return array
+	 */
+	public function resolveManualHostActionScripts(array $data): array {
+		$types = [
+			'macros' => [
+				'host' => ['{HOSTNAME}', '{HOST.ID}', '{HOST.NAME}', '{HOST.HOST}'],
+				'interface' => ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}'],
+				'user_data' => ['{USER.ALIAS}', '{USER.USERNAME}', '{USER.FULLNAME}', '{USER.NAME}', '{USER.SURNAME}'],
+				'inventory' => array_keys(self::getSupportedHostInventoryMacrosMap())
+			],
+			'usermacros' => true
+		];
+		$macro_values = [];
+		$macros = ['host' => [], 'interface' => [], 'user_data' => [], 'inventory' => []];
+		$usermacros = [];
+
+		foreach ($data as $hostid => $script) {
+			// Reset matched macros for each host.
+			$matched_macros = [
+				'macros' => ['host' => [], 'interface' => [], 'user_data' => [], 'inventory' => []],
+				'usermacros' => []
+			];
+
+			foreach ($script as $fields) {
+				$exctracted_macros = self::extractMacros($fields, $types);
+
+				// $type = 'macros' and 'usermacros'
+				foreach ($exctracted_macros as $type => $_macros) {
+					if ($type === 'usermacros') {
+						foreach ($_macros as $macro => $value) {
+							$matched_macros[$type][$macro] = null;
+						}
+					}
+					else {
+						foreach ($_macros as $key => $__macros) {
+							// $key = 'host', 'interface', 'user_data' and 'inventory'
+							foreach ($__macros as $macro) {
+								if (!in_array($macro, $matched_macros[$type][$key])) {
+									$matched_macros[$type][$key][] = $macro;
+								}
+							}
+						}
+					}
+				}
+			}
+
+			foreach ($matched_macros['macros']['host'] as $macro) {
+				if ($macro === '{HOST.ID}') {
+					$macro_values[$hostid][$macro] = $hostid;
+				}
+				else {
+					$macro_values[$hostid][$macro] = UNRESOLVED_MACRO_STRING;
+					$macros['host'][$hostid][$hostid] = true;
+				}
+			}
+
+			foreach ($matched_macros['macros']['interface'] as $macro) {
+				$macro_values[$hostid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['interface'][$hostid][$hostid] = true;
+			}
+
+			foreach ($matched_macros['macros']['inventory'] as $macro) {
+				$macro_values[$hostid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['inventory'][$hostid][$hostid] = true;
+			}
+
+			foreach ($matched_macros['macros']['user_data'] as $macro) {
+				$macro_values[$hostid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['user_data'][$hostid] = true;
+			}
+
+			if ($matched_macros['usermacros']) {
+				$usermacros[$hostid] = ['hostids' => [$hostid], 'macros' => $matched_macros['usermacros']];
+			}
+		}
+
+		$macro_values = self::getHostMacrosByHostId($macros['host'], $macro_values);
+		$macro_values = self::getInterfaceMacrosByHostId($macros['interface'], $macro_values);
+		$macro_values = self::getInventoryMacrosByHostId($macros['inventory'], $macro_values);
+		$macro_values = self::getUserDataMacros($macro_values);
+
+		foreach ($this->getUserMacros($usermacros) as $hostid => $usermacros_data) {
+			$macro_values[$hostid] = array_key_exists($hostid, $macro_values)
+				? array_merge($macro_values[$hostid], $usermacros_data['macros'])
+				: $usermacros_data['macros'];
+		}
+
+		foreach ($data as $hostid => &$scripts) {
+			if (array_key_exists($hostid, $macro_values)) {
+				foreach ($scripts as &$fields) {
+					foreach ($fields as &$value) {
+						$value = strtr($value, $macro_values[$hostid]);
+					}
+					unset($value);
+				}
+				unset($fields);
+			}
+		}
+		unset($scripts);
+
+		return $data;
+	}
+
+	/**
+	 * Resolve macros for manual event action scripts. Resolves host<1-9> macros, interface<1-9> macros,
+	 * inventory<1-9> macros, user macros, event macros and user data macros.
+	 *
+	 * @param array $data                                  Array of unersolved macros.
+	 * @param array $data[<eventid>]                       Array of scripts. Contains script ID as keys.
+	 * @param array $data[<eventid>][<scriptid>]           Script fields to resolve macros for.
+	 * @param array $events                                Array of events.
+	 * @param array $events[<eventid>]                     Event fields.
+	 * @param array $events[<eventid>][hosts]              Array of hosts that created the event.
+	 * @param array $events[<eventid>][hosts][][<hostid>]  Host ID.
+	 * @param array $events[<eventid>][objectid]           Trigger ID.
+	 *
+	 * Example input:
+	 *     array (
+	 *         19 => array (
+	 *             57 => array (
+	 *                 'confirmation' => 'Responsible hosts {HOST.HOST1}, {HOST.HOST2}! Navigate to triggers?',
+	 *                 'url' => 'http://zabbix/ui/triggers.php?context=host&filter_hostids[]={HOST.ID1}&filter_hostids[]={HOST.ID2}&filter_set=1'
+	 *             ),
+	 *             61 => array(
+	 *                 'confirmation' => 'Hello, {USER.FULLNAME}! Execute script?'
+	 *             )
+	 *         )
+	 *     )
+	 *
+	 * Output:
+	 *     array (
+	 *         19 => array (
+	 *             57 => array (
+	 *                 'confirmation' => 'Responsible hosts Zabbix server, Zabbix PC! Navigate to triggers?',
+	 *                 'url' => 'http://zabbix/ui/triggers.php?context=host&filter_hostids[]=10084&filter_hostids[]=10134&filter_set=1'
+	 *             ),
+	 *             61 => array (
+	 *                 'confirmation' => 'Hello, Zabbix Administrator! Execute script?'
+	 *             )
+	 *         )
+	 *     )
+	 *
+	 * @return array
+	 */
+	public function resolveManualEventActionScripts(array $data, array $events): array {
+		$types = [
+			'macros' => [
+				'event' => ['{EVENT.ID}', '{EVENT.NAME}', '{EVENT.NSEVERITY}', '{EVENT.SEVERITY}', '{EVENT.STATUS}',
+					'{EVENT.VALUE}', '{EVENT.CAUSE.ID}', '{EVENT.CAUSE.NAME}', '{EVENT.CAUSE.NSEVERITY}',
+					'{EVENT.CAUSE.SEVERITY}', '{EVENT.CAUSE.STATUS}', '{EVENT.CAUSE.VALUE}'
+				],
+				'user_data' => ['{USER.ALIAS}', '{USER.USERNAME}', '{USER.FULLNAME}', '{USER.NAME}', '{USER.SURNAME}']
+			],
+			'macros_n' => [
+				'host' => ['{HOSTNAME}', '{HOST.ID}', '{HOST.HOST}', '{HOST.NAME}'],
+				'interface' => ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}'],
+				'inventory' => array_keys(self::getSupportedHostInventoryMacrosMap())
+			],
+			'usermacros' => true
+		];
+		$macro_values = [];
+		$macros = ['host' => [], 'interface' => [], 'inventory' => [], 'event' => [], 'user_data' => [],
+			'host_n' => [], 'interface_n' => [], 'inventory_n' => []
+		];
+		$usermacros = [];
+
+		foreach ($data as $eventid => $script) {
+			$event = $events[$eventid];
+			$triggerid = $event['objectid'];
+			$matched_macros = [
+				'macros' => ['host' => [], 'interface' => [], 'inventory' => [], 'event' => [], 'user_data' => []],
+				'macros_n' => ['host' => [], 'interface' => [], 'inventory' => []],
+				'usermacros' => []
+			];
+
+			foreach ($script as $fields) {
+				$exctracted_macros = self::extractMacros($fields, $types);
+
+				// $type = 'macros' and 'usermacros'
+				foreach ($exctracted_macros as $type => $_macros) {
+					switch ($type) {
+						case 'usermacros':
+							foreach ($_macros as $macro => $value) {
+								$matched_macros[$type][$macro] = null;
+							}
+							break;
+
+						case 'macros':
+							foreach ($_macros as $key => $__macros) {
+								// $key = 'host', 'interface', 'inventory', 'event' and 'user_data'
+								foreach ($__macros as $macro) {
+									if (!in_array($macro, $matched_macros[$type][$key])) {
+										$matched_macros[$type][$key][] = $macro;
+									}
+								}
+							}
+							break;
+
+						case 'macros_n':
+							// $key = 'host', 'interface', 'inventory'
+							foreach ($_macros as $key => $__macros) {
+								foreach ($__macros as $macro => $_macro) {
+									$matched_macros[$type][$key][$macro] = $_macro;
+								}
+							}
+							break;
+					}
+				}
+			}
+
+			$hostid = $event['hosts'][0]['hostid'];
+
+			foreach ($matched_macros['macros']['host'] as $macro) {
+				if ($macro === '{HOST.ID}') {
+					$macro_values[$eventid][$macro] = $hostid;
+				}
+				else {
+					$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+					$macros['host'][$hostid][$eventid] = true;
+				}
+			}
+
+			foreach ($matched_macros['macros']['interface'] as $macro) {
+				$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['interface'][$hostid][$eventid] = true;
+			}
+
+			foreach ($matched_macros['macros']['inventory'] as $macro) {
+				$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['inventory'][$hostid][$eventid] = true;
+			}
+
+			foreach ($matched_macros['macros']['user_data'] as $macro) {
+				$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['user_data'][$hostid] = true;
+			}
+
+			// Numeric index macros.
+			foreach ($matched_macros['macros_n']['host'] as $macro => $_data) {
+				$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['host_n'][$triggerid][$eventid][$macro] = $_data;
+			}
+
+			foreach ($matched_macros['macros_n']['interface'] as $macro => $_data) {
+				$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['interface_n'][$triggerid][$eventid][$macro] = $_data;
+			}
+
+			foreach ($matched_macros['macros_n']['inventory'] as $macro => $_data) {
+				$macro_values[$eventid][$macro] = UNRESOLVED_MACRO_STRING;
+				$macros['inventory_n'][$triggerid][$eventid][$macro] = $_data;
+			}
+
+			// Event macros.
+			foreach ($matched_macros['macros']['event'] as $macro) {
+				switch ($macro) {
+					case '{EVENT.ID}':
+						$macro_values[$eventid][$macro] = $eventid;
+						break;
+
+					case '{EVENT.NAME}':
+						$macro_values[$eventid][$macro] = $event['name'];
+						break;
+
+					case '{EVENT.NSEVERITY}':
+						$macro_values[$eventid][$macro] = $event['severity'];
+						break;
+
+					case '{EVENT.SEVERITY}':
+						$macro_values[$eventid][$macro] = CSeverityHelper::getName($event['severity']);
+						break;
+
+					case '{EVENT.STATUS}':
+						$macro_values[$eventid][$macro] = trigger_value2str($event['value']);
+						break;
+
+					case '{EVENT.VALUE}':
+						$macro_values[$eventid][$macro] = $event['value'];
+						break;
+
+					/*
+					 * If event is already cause event, $event['cause'] does not exist or is empty, macros resolve to
+					 * *UNKNOWN*.
+					 */
+					case '{EVENT.CAUSE.ID}':
+						$macro_values[$eventid][$macro] = $event['cause_eventid'] == 0
+							? UNRESOLVED_MACRO_STRING
+							: $event['cause_eventid'];
+						break;
+
+					case '{EVENT.CAUSE.NAME}':
+						$macro_values[$eventid][$macro] = array_key_exists('cause', $event) && $event['cause']
+							? $event['cause']['name']
+							: UNRESOLVED_MACRO_STRING;
+						break;
+
+					case '{EVENT.CAUSE.NSEVERITY}':
+						$macro_values[$eventid][$macro] = array_key_exists('cause', $event) && $event['cause']
+							? $event['cause']['severity']
+							: UNRESOLVED_MACRO_STRING;
+						break;
+
+					case '{EVENT.CAUSE.SEVERITY}':
+						$macro_values[$eventid][$macro] = array_key_exists('cause', $event) && $event['cause']
+							? CSeverityHelper::getName($event['cause']['severity'])
+							: UNRESOLVED_MACRO_STRING;
+						break;
+
+					case '{EVENT.CAUSE.STATUS}':
+						$macro_values[$eventid][$macro] = array_key_exists('cause', $event) && $event['cause']
+							? trigger_value2str($event['cause']['value'])
+							: UNRESOLVED_MACRO_STRING;
+						break;
+
+					case '{EVENT.CAUSE.VALUE}':
+						$macro_values[$eventid][$macro] = array_key_exists('cause', $event) && $event['cause']
+							? $event['cause']['value']
+							: UNRESOLVED_MACRO_STRING;
+						break;
+				}
+			}
+
+			if ($matched_macros['usermacros']) {
+				$usermacros[$eventid] = [
+					'hostids' => array_column($event['hosts'], 'hostid'),
+					'macros' => $matched_macros['usermacros']
+				];
+			}
+		}
+
+		$macro_values = self::getHostMacrosByHostId($macros['host'], $macro_values);
+		$macro_values = self::getInterfaceMacrosByHostId($macros['interface'], $macro_values);
+		$macro_values = self::getInventoryMacrosByHostId($macros['inventory'], $macro_values);
+		$macro_values = self::getUserDataMacros($macro_values);
+
+		$trigger_hosts_by_f_num = self::getExpressionHosts(
+			array_keys($macros['host_n'] + $macros['interface_n'] + $macros['inventory_n'])
+		);
+		$macro_values = self::getHostNMacros($macros['host_n'], $macro_values, $trigger_hosts_by_f_num);
+		$macro_values = self::getInterfaceNMacros($macros['interface_n'], $macro_values, $trigger_hosts_by_f_num);
+		$macro_values = self::getInventoryNMacros($macros['inventory_n'], $macro_values, $trigger_hosts_by_f_num);
+
+		foreach ($this->getUserMacros($usermacros) as $eventid => $usermacros_data) {
+			$macro_values[$eventid] = array_key_exists($eventid, $macro_values)
+				? array_merge($macro_values[$eventid], $usermacros_data['macros'])
+				: $usermacros_data['macros'];
+		}
+
+		$types = $this->transformToPositionTypes($types);
+
+		// Replace macros to value.
+		foreach ($data as $eventid => &$scripts) {
+			$event = $events[$eventid];
+			$triggerid = $event['objectid'];
+
+			foreach ($scripts as &$fields) {
+				foreach ($fields as &$value) {
+					$matched_macros = self::getMacroPositions($value, $types);
+
+					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+						if (array_key_exists($macro, $macro_values[$eventid])) {
+							$value = substr_replace($value, $macro_values[$eventid][$macro], $pos, strlen($macro));
+						}
+					}
+				}
+				unset($value);
+			}
+			unset($fields);
+		}
+		unset($scripts);
+
+		return $data;
+	}
 }
