@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ abstract class CControllerResponse {
 			CMessageHelper::restoreScheduleMessages();
 		}
 
-		(new CPageHeader(_('Loading...'), CWebUser::getLang()))->display();
+		(new CHtmlPageHeader(_('Loading...'), CWebUser::getLang()))->show();
 
 		echo '<body>';
 
@@ -58,7 +58,6 @@ abstract class CControllerResponse {
 
 	protected function getForm(): CForm {
 		$form = (new CForm())
-				->cleanItems()
 				->setAction($this->getLocation())
 				->setEnctype('multipart/form-data')
 				->setId('form-data');
