@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,13 +26,26 @@ class CProfile {
 	private static $update = [];
 	private static $insert = [];
 	private static $stringProfileMaxLength;
+	private static $is_initialized = false;
 
 	public static function init() {
 		self::$userDetails = CWebUser::$data;
 		self::$profiles = [];
 
-		$profilesTableSchema = DB::getSchema('profiles');
-		self::$stringProfileMaxLength = $profilesTableSchema['fields']['value_str']['length'];
+		self::$stringProfileMaxLength = DB::getFieldLength('profiles', 'value_str');
+		DBselect('SELECT NULL FROM users u WHERE '.dbConditionId('u.userid', (array) self::$userDetails['userid']).
+			' FOR UPDATE'
+		);
+
+		if (!self::$is_initialized) {
+			register_shutdown_function(function() {
+				DBstart();
+				$result = self::flush();
+				DBend($result);
+			});
+		}
+
+		self::$is_initialized = true;
 	}
 
 	/**
@@ -52,7 +65,9 @@ class CProfile {
 
 			foreach (self::$insert as $idx => $profile) {
 				foreach ($profile as $idx2 => $data) {
-					$result &= self::insertDB($idx, $data['value'], $data['type'], $idx2);
+					if (!self::insertDB($idx, $data['value'], $data['type'], $idx2)) {
+						$result = false;
+					}
 				}
 			}
 
@@ -60,9 +75,13 @@ class CProfile {
 			foreach (self::$update as $idx => $profile) {
 				ksort($profile);
 				foreach ($profile as $idx2 => $data) {
-					$result &= self::updateDB($idx, $data['value'], $data['type'], $idx2);
+					if (!self::updateDB($idx, $data['value'], $data['type'], $idx2)) {
+						$result = false;
+					}
 				}
 			}
+
+			self::clear();
 		}
 
 		return $result;
@@ -86,7 +105,7 @@ class CProfile {
 			return null;
 		}
 
-		if (is_null(self::$profiles)) {
+		if (self::$profiles === null) {
 			self::init();
 		}
 
@@ -107,7 +126,7 @@ class CProfile {
 
 		// Aggressive caching, cache all items matched $idx key.
 		$query = DBselect(
-			'SELECT type, value_id, value_int, value_str, idx, idx2'.
+			'SELECT type,value_id,value_int,value_str,idx,idx2'.
 			' FROM profiles'.
 			' WHERE userid='.self::$userDetails['userid'].
 				' AND idx LIKE '.zbx_dbstr($idx_pattern)
@@ -170,6 +189,8 @@ class CProfile {
 			self::$profiles[$idx][$row['idx2']] = $row[$value_type];
 		}
 
+		ksort(self::$profiles[$idx], SORT_NUMERIC);
+
 		return array_key_exists($idx2, self::$profiles[$idx]) ? self::$profiles[$idx][$idx2] : $default_value;
 	}
 
@@ -196,7 +217,7 @@ class CProfile {
 	 * @param int|array  	$idx2	second identifier, which can be list of identifiers as well
 	 */
 	public static function delete($idx, $idx2 = 0) {
-		if (is_null(self::$profiles)) {
+		if (self::$profiles === null) {
 			self::init();
 		}
 
@@ -244,10 +265,14 @@ class CProfile {
 	 * @param int		$idx2
 	 */
 	public static function update($idx, $value, $type, $idx2 = 0) {
-		if (is_null(self::$profiles)) {
+		if (self::$profiles === null) {
 			self::init();
 		}
 
+		if (is_array($value)) {
+			return;
+		}
+
 		if (!self::checkValueType($value, $type)) {
 			return;
 		}
@@ -321,14 +346,14 @@ class CProfile {
 			$value_type => zbx_dbstr($value),
 			'type' => $type,
 			'idx2' => zbx_dbstr($idx2)
+		] + [
+			'value_str' => zbx_dbstr('')
 		];
 
 		return DBexecute('INSERT INTO profiles ('.implode(', ', array_keys($values)).') VALUES ('.implode(', ', $values).')');
 	}
 
 	private static function updateDB($idx, $value, $type, $idx2) {
-		$sqlIdx2 = ($idx2 > 0) ? ' AND idx2='.zbx_dbstr($idx2) : '';
-
 		$valueType = self::getFieldByType($type);
 
 		return DBexecute(
@@ -336,8 +361,8 @@ class CProfile {
 			$valueType.'='.zbx_dbstr($value).','.
 			' type='.$type.
 			' WHERE userid='.self::$userDetails['userid'].
-			' AND idx='.zbx_dbstr($idx).
-			$sqlIdx2
+				' AND idx='.zbx_dbstr($idx).
+				' AND idx2='.zbx_dbstr($idx2)
 		);
 	}
 
