@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ class CColorValidator extends CStringValidator {
 	public $regex = '/^[0-9a-f]{6}$/i';
 
 	public function __construct(array $options = []) {
-		$this->messageRegex = _('Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).');
+		$this->messageRegex = _('Color "%1$s" is not correct: expecting hexadecimal color code (6 symbols).');
 		$this->messageEmpty = _('Empty color.');
 
 		parent::__construct($options);
