@@ -1,7 +1,7 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,39 +19,23 @@
 **/
 
 
+namespace Zabbix\Widgets\Fields;
+
+use Zabbix\Widgets\CWidgetField;
+
 class CWidgetFieldLatLng extends CWidgetField {
 
-	/**
-	 * @var string
-	 */
-	private $placeholder;
-
-	/**
-	 * @var int
-	 */
-	private $width;
+	public const DEFAULT_VALUE = '';
 
 	/**
 	 * Latitude, longitude and zoom level input text box widget field.
-	 *
-	 * @param string $name  field name in form
-	 * @param string $label  label for the field in form
 	 */
-	public function __construct($name, $label) {
+	public function __construct(string $name, string $label = null) {
 		parent::__construct($name, $label);
 
-		$this->setSaveType(ZBX_WIDGET_FIELD_TYPE_STR);
-		$this->setValidationRules(['type' => API_LAT_LNG_ZOOM, 'length' => 255]);
-		$this->placeholder = '40.6892494,-74.0466891';
-		$this->width = ZBX_TEXTAREA_MEDIUM_WIDTH;
-		$this->setDefault('');
-	}
-
-	public function getPlaceholder() {
-		return $this->placeholder;
-	}
-
-	public function getWidth() {
-		return $this->width;
+		$this
+			->setDefault(self::DEFAULT_VALUE)
+			->setSaveType(ZBX_WIDGET_FIELD_TYPE_STR)
+			->setValidationRules(['type' => API_LAT_LNG_ZOOM, 'length' => 255]);
 	}
 }
