@@ -1,7 +1,7 @@
-<?php
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,65 +19,48 @@
 **/
 
 
-/**
- * Widget Field for numeric box
- */
+namespace Zabbix\Widgets\Fields;
+
+use Zabbix\Widgets\CWidgetField;
+
 class CWidgetFieldRangeControl extends CWidgetField {
 
-	/**
-	 * Allowed min value
-	 *
-	 * @var int
-	 */
-	private $min;
+	private int $min;
+	private int $max;
+	private int $step;
 
 	/**
-	 * Allowed max value
-	 *
-	 * @var int
+	 * @param int $min  Minimal allowed value.
+	 * @param int $max  Maximal allowed value.
 	 */
-	private $max;
-
-	/**
-	 * Step value
-	 *
-	 * @var int
-	 */
-	private $step;
-
-	/**
-	 * A numeric box widget field.
-	 *
-	 * @param string $name   field name in form
-	 * @param string $label  label for the field in form
-	 * @param int    $min    minimal allowed value (this included)
-	 * @param int    $max    maximal allowed value (this included)
-	 * @param int    $step   step value
-	 */
-	public function __construct($name, $label, $min = 0, $max = ZBX_MAX_INT32, $step = 1) {
+	public function __construct(string $name, string $label = null, int $min = 0, int $max = ZBX_MAX_INT32,
+			int $step = 1) {
 		parent::__construct($name, $label);
 
-		$this->setSaveType(ZBX_WIDGET_FIELD_TYPE_INT32);
 		$this->min = $min;
 		$this->max = $max;
 		$this->step = $step;
-		$this->setExValidationRules(['in' => $this->min.':'.$this->max]);
+
+		$this
+			->setSaveType(ZBX_WIDGET_FIELD_TYPE_INT32)
+			->setExValidationRules(['in' => $this->min.':'.$this->max]);
 	}
 
-	public function setValue($value) {
+	public function setValue($value): self {
 		$this->value = (int) $value;
+
 		return $this;
 	}
 
-	public function getMin() {
+	public function getMin(): int {
 		return $this->min;
 	}
 
-	public function getMax() {
+	public function getMax(): int {
 		return $this->max;
 	}
 
-	public function getStep() {
+	public function getStep(): int {
 		return $this->step;
 	}
 }
