@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ function zbx_is_callable(array $names) {
 
 /************ REQUEST ************/
 function redirect($url) {
-	$curl = (new CUrl($url))->removeArgument('sid');
+	$curl = (new CUrl($url))->removeArgument(CCsrfTokenHelper::CSRF_TOKEN_NAME);
 	header('Location: '.$curl->getUrl());
 	exit;
 }
@@ -396,32 +396,6 @@ function getColorVariations($color, $variations_requested = 1) {
 	return $variations;
 }
 
-function zbx_num2bitstr($num, $rev = false) {
-	if (!is_numeric($num)) {
-		return 0;
-	}
-
-	$strbin = '';
-
-	$len = 32;
-	if ($num > ZBX_MAX_INT32) {
-		$len = 64;
-	}
-
-	for ($i = 0; $i < $len; $i++) {
-		$sbin = 1 << $i;
-		$bit = ($sbin & $num) ? '1' : '0';
-		if ($rev) {
-			$strbin .= $bit;
-		}
-		else {
-			$strbin = $bit.$strbin;
-		}
-	}
-
-	return $strbin;
-}
-
 /**
  * Convert suffixed string to decimal bytes ('10K' => 10240).
  * Note: this function must not depend on optional PHP libraries, since it is used in Zabbix setup.
@@ -580,7 +554,7 @@ function convertUnitsS($value, $ignore_millisec = false) {
 					$v = fmod($value_abs, 1) * 1000;
 
 					if ($v > 0) {
-						$parts['milliseconds'] = formatFloat($v, null, ZBX_UNITS_ROUNDOFF_SUFFIXED);
+						$parts['milliseconds'] = formatFloat($v, ['decimals' => ZBX_UNITS_ROUNDOFF_SUFFIXED]);
 					}
 				}
 			}
@@ -600,7 +574,7 @@ function convertUnitsS($value, $ignore_millisec = false) {
 	$result = [];
 
 	foreach (array_filter($parts) as $part_unit => $part_value) {
-		$result[] = formatFloat($part_value, null, ZBX_UNITS_ROUNDOFF_SUFFIXED).$units[$part_unit];
+		$result[] = formatFloat($part_value, ['decimals' => ZBX_UNITS_ROUNDOFF_SUFFIXED]).$units[$part_unit];
 	}
 
 	return $result ? ($value < 0 ? '-' : '').implode(' ', $result) : '0';
@@ -674,23 +648,15 @@ function convertSecondsToTimeUnits(int $value): string {
 
 /**
  * Converts a raw value to a user-friendly representation based on unit and other parameters.
- * Example:
- * 	6442450944 B convert to 6 GB
+ * Example: 6442450944 B => 6 GB.
  *
- * @param array  $options
- * @param string $options['value']
- * @param string $options['units']
- * @param int    $options['convert']
- * @param int    $options['power']
- * @param string $options['unit_base']
- * @param bool   $options['ignore_milliseconds']
- * @param int    $options['precision']
- * @param int    $options['decimals']
- * @param bool   $options['decimals_exact']
+ * @see convertUnitsRaw
+ *
+ * @param array $options
  *
  * @return string
  */
-function convertUnits(array $options) {
+function convertUnits(array $options): string {
 	[
 		'value' => $value,
 		'units' => $units
@@ -707,23 +673,33 @@ function convertUnits(array $options) {
 
 /**
  * Converts a raw value to a user-friendly representation based on unit and other parameters.
- * Example:
- * 	6442450944 B convert to 6 GB
+ * Example: 6442450944 B => 6 GB.
  *
- * @param array  $options
- * @param string $options['value']
- * @param string $options['units']
- * @param int    $options['convert']
- * @param int    $options['power']
- * @param string $options['unit_base']
- * @param bool   $options['ignore_milliseconds']
- * @param int    $options['precision']
- * @param int    $options['decimals']
- * @param bool   $options['decimals_exact']
+ * @param array $options
+ *
+ * $options = [
+ *     'value' =>               (string)    Value to convert.
+ *     'units' =>               (string)    Units to base the conversion on. Default: ''.
+ *     'convert' =>             (int)       Default: ITEM_CONVERT_WITH_UNITS. Set to ITEM_CONVERT_NO_UNITS to
+ *                                          force-convert a value with empty units.
+ *     'power' =>               (int)       Convert to the specified power of "unit_base" (0 => '', 1 => K, 2 => M, ..).
+ *                                          By default, the power will be calculated automatically.
+ *     'unit_base' =>           (string)    1000 or 1024. By default, will only use 1024 for "B" and "Bps" units.
+ *     'ignore_milliseconds' => (bool)      Ignore milliseconds in time conversion ("s" units).
+ *     'precision' =>           (int)       Max number of significant digits to take into account.
+ *                                          Default: ZBX_FLOAT_DIG.
+ *     'decimals' =>            (int|null)  Max number of first non-zero decimals to display. If null is specified,
+ *                                          ZBX_UNITS_ROUNDOFF_SUFFIXED or ZBX_UNITS_ROUNDOFF_UNSUFFIXED will be used,
+ *                                          depending on whether the units have been prefixed.
+ *     'decimals_exact' =>      (bool)      Display exactly this number of decimals instead of first non-zeros.
+ *                                          Default: false.
+ *     'small_scientific' =>    (bool)      Allow scientific notation for small numbers. Default: true.
+ *     'zero_as_zero' =>        (bool)      Return zero as '0', regardless of other options. Default: true.
+ * ]
  *
  * @return array
  */
-function convertUnitsRaw(array $options) {
+function convertUnitsRaw(array $options): array {
 	static $power_table = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'];
 
 	$options += [
@@ -733,9 +709,11 @@ function convertUnitsRaw(array $options) {
 		'power' => null,
 		'unit_base' => null,
 		'ignore_milliseconds' => false,
-		'precision' => null,
+		'precision' => ZBX_FLOAT_DIG,
 		'decimals' => null,
-		'decimals_exact' => false
+		'decimals_exact' => false,
+		'small_scientific' => true,
+		'zero_as_zero' => true
 	];
 
 	$value = $options['value'] !== null ? $options['value'] : '';
@@ -788,9 +766,13 @@ function convertUnitsRaw(array $options) {
 
 	if (in_array($units, $blacklist) || !$do_convert || $value_abs < 1) {
 		return [
-			'value' => formatFloat($value, $options['precision'], $options['decimals'] ?? ZBX_UNITS_ROUNDOFF_UNSUFFIXED,
-				$options['decimals_exact']
-			),
+			'value' => formatFloat($value, [
+				'precision' => $options['precision'],
+				'decimals' => $options['decimals'] !== null ? $options['decimals'] : ZBX_UNITS_ROUNDOFF_UNSUFFIXED,
+				'decimals_exact' => $options['decimals_exact'],
+				'small_scientific' => $options['small_scientific'],
+				'zero_as_zero' => $options['zero_as_zero']
+			]),
 			'units' => $units,
 			'is_numeric' => true
 		];
@@ -799,7 +781,7 @@ function convertUnitsRaw(array $options) {
 	$unit_base = $options['unit_base'];
 
 	if ($unit_base != 1000 && $unit_base != ZBX_KIBIBYTE) {
-		$unit_base = ($units === 'B' || $units === 'Bps') ? ZBX_KIBIBYTE : 1000;
+		$unit_base = isBinaryUnits($units) ? ZBX_KIBIBYTE : 1000;
 	}
 
 	if ($options['power'] === null) {
@@ -807,10 +789,16 @@ function convertUnitsRaw(array $options) {
 		$unit_prefix = null;
 
 		foreach ($power_table as $power => $prefix) {
-			$result = formatFloat($value / pow($unit_base, $power), $options['precision'],
-				$options['decimals'] ?? ($prefix === '' ? ZBX_UNITS_ROUNDOFF_UNSUFFIXED : ZBX_UNITS_ROUNDOFF_SUFFIXED),
-				$options['decimals_exact']
-			);
+			$result = formatFloat($value / pow($unit_base, $power), [
+				'precision' => $options['precision'],
+				'decimals' => $options['decimals'] !== null
+					? $options['decimals']
+					: ($prefix === '' ? ZBX_UNITS_ROUNDOFF_UNSUFFIXED : ZBX_UNITS_ROUNDOFF_SUFFIXED),
+				'decimals_exact' => $options['decimals_exact'],
+				'small_scientific' => $options['small_scientific'],
+				'zero_as_zero' => $options['zero_as_zero']
+			]);
+
 			$unit_prefix = $prefix;
 
 			if (abs($result) < $unit_base) {
@@ -819,19 +807,18 @@ function convertUnitsRaw(array $options) {
 		}
 	}
 	else {
-		if (array_key_exists($options['power'], $power_table) && $value_abs != 0) {
-			$unit_power = $options['power'];
-			$unit_prefix = $power_table[$unit_power];
-		}
-		else {
-			$unit_power = count($power_table) - 1;
-			$unit_prefix = $power_table[$unit_power];
-		}
+		$unit_power = array_key_exists($options['power'], $power_table) ? $options['power'] : count($power_table) - 1;
+		$unit_prefix = $power_table[$unit_power];
 
-		$result = formatFloat($value / pow($unit_base, $unit_power), $options['precision'],
-			$options['decimals'] ?? ($unit_prefix === '' ? ZBX_UNITS_ROUNDOFF_UNSUFFIXED : ZBX_UNITS_ROUNDOFF_SUFFIXED),
-			$options['decimals_exact']
-		);
+		$result = formatFloat($value / pow($unit_base, $unit_power), [
+			'precision' => $options['precision'],
+			'decimals' => $options['decimals'] !== null
+				? $options['decimals']
+				: ($unit_prefix === '' ? ZBX_UNITS_ROUNDOFF_UNSUFFIXED : ZBX_UNITS_ROUNDOFF_SUFFIXED),
+			'decimals_exact' => $options['decimals_exact'],
+			'small_scientific' => $options['small_scientific'],
+			'zero_as_zero' => $options['zero_as_zero']
+		]);
 	}
 
 	$result_units = ($result == 0 ? '' : $unit_prefix).$units;
@@ -901,6 +888,10 @@ function zbx_empty($value) {
 }
 
 function zbx_is_int($var) {
+	if (is_array($var)) {
+		return false;
+	}
+
 	if (is_int($var)) {
 		return true;
 	}
@@ -1384,7 +1375,7 @@ function zbx_str2links($text) {
 				if ($pos != $start) {
 					$result[] = mb_substr($line, $start, $pos - $start);
 				}
-				$result[] = new CLink(CHtml::encode($match), $match);
+				$result[] = new CLink($match, $match);
 				$start = $pos + mb_strlen($match);
 			}
 		}
@@ -1450,22 +1441,39 @@ function make_sorting_header($obj, $tabfield, $sortField, $sortOrder, $link = nu
 }
 
 /**
- * Format floating-point number in best possible way for displaying.
+ * Get decimal point and thousands separator for number formatting according to the current locale.
  *
- * @param string   $number     Valid number in decimal or scientific notation.
- * @param int|null $precision  Max number of significant digits to take into account. Default: ZBX_FLOAT_DIG.
- * @param int|null $decimals   Max number of first non-zero decimals to display. Default: 0.
- * @param bool     $exact      Display exactly this number of decimals instead of first non-zeros.
+ * @return array  'decimal_point' and 'thousands_sep' values.
+ */
+function getNumericFormatting(): array {
+	static $numeric_formatting = null;
+
+	if ($numeric_formatting === null) {
+		$numeric_formatting = array_intersect_key(localeconv(), array_flip(['decimal_point', 'thousands_sep']));
+	}
+
+	return $numeric_formatting;
+}
+
+/**
+ * Format floating-point number in the best possible way for displaying.
+ *
+ * @param float $number   Valid number in decimal or scientific notation.
+ * @param array $options  Formatting options.
+ *
+ * $options = [
+ *     'precision' =>        (int)   Max number of significant digits to take into account. Default: ZBX_FLOAT_DIG.
+ *     'decimals' =>         (int)   Max number of first non-zero decimals to display. Default: 0.
+ *     'decimals_exact' =>   (bool)  Display exactly this number of decimals instead of first non-zeros. Default: false.
+ *     'small_scientific' => (bool)  Allow scientific notation for small numbers. Default: true.
+ *     'zero_as_zero' =>     (bool)  Return zero as '0', regardless of other options. Default: true.
+ * ]
  *
  * Note: $decimals must be less than $precision.
  *
  * @return string
  */
-function formatFloat(float $number, int $precision = null, int $decimals = null, bool $exact = false): string {
-	if ($number == 0) {
-		return '0';
-	}
-
+function formatFloat(float $number, array $options = []): string {
 	if ($number == INF) {
 		return _('Infinity');
 	}
@@ -1474,12 +1482,24 @@ function formatFloat(float $number, int $precision = null, int $decimals = null,
 		return '-'._('Infinity');
 	}
 
-	if ($precision === null) {
-		$precision = ZBX_FLOAT_DIG;
-	}
+	$defaults = [
+		'precision' => ZBX_FLOAT_DIG,
+		'decimals' => 0,
+		'decimals_exact' => false,
+		'small_scientific' => true,
+		'zero_as_zero' => true
+	];
 
-	if ($decimals === null) {
-		$decimals = 0;
+	[
+		'precision' => $precision,
+		'decimals' => $decimals,
+		'decimals_exact' => $decimals_exact,
+		'small_scientific' => $small_scientific,
+		'zero_as_zero' => $zero_as_zero
+	] = $options + $defaults;
+
+	if ($zero_as_zero && $number == 0) {
+		return '0';
 	}
 
 	$number_original = $number;
@@ -1500,11 +1520,11 @@ function formatFloat(float $number, int $precision = null, int $decimals = null,
 			}
 
 			$test_number = sprintf('%.'.($precision - 1).'E', $test);
-			$test_digits = ($precision == 1)
+			$test_digits = $precision == 1
 				? 1
 				: strlen(rtrim(explode('E', $test_number)[0], '0')) - ($test_number[0] === '-' ? 2 : 1);
 
-			if ($test_digits - $exponent <= $precision) {
+			if (!$small_scientific || $test_digits - $exponent < $precision) {
 				break;
 			}
 		}
@@ -1523,29 +1543,43 @@ function formatFloat(float $number, int $precision = null, int $decimals = null,
 		}
 
 		$number = sprintf('%.'.($precision - 1).'E', $number);
-		$digits = ($precision == 1) ? 1 : strlen(rtrim(explode('E', $number)[0], '0')) - ($number[0] === '-' ? 2 : 1);
+		$digits = $precision == 1 ? 1 : strlen(rtrim(explode('E', $number)[0], '0')) - ($number[0] === '-' ? 2 : 1);
 	}
 
-	if ($number == 0) {
+	if ($zero_as_zero && $number == 0) {
 		return '0';
 	}
 
+	[
+		'decimal_point' => $decimal_point,
+		'thousands_sep' => $thousands_sep
+	] = getNumericFormatting();
+
 	$exponent = (int) explode('E', sprintf('%.'.($precision - 1).'E', $number))[1];
 
 	if ($exponent < 0) {
-		if ($digits - $exponent <= ($exact ? min($decimals + 1, $precision) : $precision)) {
-			return sprintf('%.'.($exact ? $decimals : $digits - $exponent - 1).'f', $number);
+		if (!$small_scientific
+				|| $digits - $exponent <= ($decimals_exact ? min($decimals + 1, $precision) : $precision)) {
+			return number_format($number, $decimals_exact ? $decimals : $digits - $exponent - 1, $decimal_point,
+				$thousands_sep
+			);
 		}
 		else {
-			return sprintf('%.'.($exact ? $decimals : min($digits - 1, $decimals)).'E', $number);
+			return str_replace('.', $decimal_point,
+				sprintf('%.'.($decimals_exact ? $decimals : min($digits - 1, $decimals)).'E', $number)
+			);
 		}
 	}
 	elseif ($exponent >= min(PHP_FLOAT_DIG, $precision + 3)
 			|| ($exponent >= $precision && $number != $number_original)) {
-		return sprintf('%.'.($exact ? $decimals : min($digits - 1, $decimals)).'E', $number);
+		return str_replace('.', $decimal_point,
+			sprintf('%.'.($decimals_exact ? $decimals : min($digits - 1, $decimals)).'E', $number)
+		);
 	}
 	else {
-		return sprintf('%.'.($exact ? $decimals : max(0, min($digits - $exponent - 1, $decimals))).'f', $number);
+		return number_format($number, $decimals_exact ? $decimals : max(0, min($digits - $exponent - 1, $decimals)),
+			$decimal_point, $thousands_sep
+		);
 	}
 }
 
@@ -1557,8 +1591,8 @@ function formatFloat(float $number, int $precision = null, int $decimals = null,
 * @return float
 */
 function truncateFloat(float $number): float {
-	if ($number == INF) {
-		return INF;
+	if (is_infinite($number)) {
+		return $number;
 	}
 
 	return (float) sprintf('%.'.(ZBX_FLOAT_DIG - 1).'E', $number);
@@ -1625,13 +1659,14 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 		show_error_message(_('No permissions to referred object or it does not exist!'));
 
 		require_once dirname(__FILE__).'/page_header.php';
-		(new CWidget())->show();
+		(new CHtmlPage())->show();
 		require_once dirname(__FILE__).'/page_footer.php';
 	}
 	// deny access to a page
 	else {
 		// url to redirect the user to after he logs in
-		$url = (new CUrl(!empty($_REQUEST['request']) ? $_REQUEST['request'] : ''))->removeArgument('sid');
+		$url = (new CUrl(!empty($_REQUEST['request']) ? $_REQUEST['request'] : ''))
+			->removeArgument(CCsrfTokenHelper::CSRF_TOKEN_NAME);
 
 		if (CAuthenticationHelper::get(CAuthenticationHelper::HTTP_LOGIN_FORM) == ZBX_AUTH_FORM_HTTP
 				&& CAuthenticationHelper::get(CAuthenticationHelper::HTTP_AUTH_ENABLED) == ZBX_AUTH_HTTP_ENABLED
@@ -1806,8 +1841,20 @@ function makeMessageBox(string $class, array $messages, string $title = null, bo
 function filter_messages(): array {
 	if (!CSettingsHelper::getGlobal(CSettingsHelper::SHOW_TECHNICAL_ERRORS)
 			&& CWebUser::getType() != USER_TYPE_SUPER_ADMIN && !CWebUser::getDebugMode()) {
+
+		$type = CMessageHelper::getType();
+		$title = CMessageHelper::getTitle();
 		$messages = CMessageHelper::getMessages();
-		CMessageHelper::clear(false);
+		CMessageHelper::clear();
+
+		if ($title !== null) {
+			if ($type === CMessageHelper::MESSAGE_TYPE_SUCCESS) {
+				CMessageHelper::setSuccessTitle($title);
+			}
+			else {
+				CMessageHelper::setErrorTitle($title);
+			}
+		}
 
 		$generic_exists = false;
 		foreach ($messages as $message) {
