@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -59,9 +59,9 @@ function make_decoration($haystack, $needle, $class = null) {
 	$pos = mb_strpos($tmpHaystack, $tmpNeedle);
 
 	if ($pos !== false) {
-		$start = CHtml::encode(mb_substr($haystack, 0, $pos));
-		$end = CHtml::encode(mb_substr($haystack, $pos + mb_strlen($needle)));
-		$found = CHtml::encode(mb_substr($haystack, $pos, mb_strlen($needle)));
+		$start = mb_substr($haystack, 0, $pos);
+		$end = mb_substr($haystack, $pos + mb_strlen($needle));
+		$found = mb_substr($haystack, $pos, mb_strlen($needle));
 
 		if (is_null($class)) {
 			$result = [$start, bold($found), $end];
@@ -138,17 +138,49 @@ function BR() {
 	return new CTag('br');
 }
 
+function BULLET() {
+	return new CHtmlEntity('&bullet;');
+}
+
+function COPYR() {
+	return new CHtmlEntity('&copy;');
+}
+
+function HELLIP() {
+	return new CHtmlEntity('&hellip;');
+}
+
+function LARR() {
+	return new CHtmlEntity('&lArr;');
+}
+
+function NBSP() {
+	return new CHtmlEntity('&nbsp;');
+}
+
+function NDASH() {
+	return new CHtmlEntity('&ndash;');
+}
+
+function RARR() {
+	return new CHtmlEntity('&rArr;');
+}
+
+function ZWSPACE() {
+	return new CHtmlEntity('&#8203;');
+}
+
 function get_icon($type, $params = []) {
 	switch ($type) {
-		case 'favourite':
+		case 'favorite':
 			if (CFavorite::exists($params['fav'], $params['elid'], $params['elname'])) {
-				$icon = (new CRedirectButton(SPACE, null))
+				$icon = (new CRedirectButton(NBSP(), null))
 					->addClass(ZBX_STYLE_BTN_REMOVE_FAV)
 					->setTitle(_('Remove from favorites'))
 					->onClick('rm4favorites("'.$params['elname'].'", "'.$params['elid'].'");');
 			}
 			else {
-				$icon = (new CRedirectButton(SPACE, null))
+				$icon = (new CRedirectButton(NBSP(), null))
 					->addClass(ZBX_STYLE_BTN_ADD_FAV)
 					->setTitle(_('Add to favorites'))
 					->onClick('add2favorites("'.$params['elname'].'", "'.$params['elid'].'");');
@@ -159,7 +191,7 @@ function get_icon($type, $params = []) {
 
 		case 'kioskmode':
 			if ($params['mode'] == ZBX_LAYOUT_KIOSKMODE) {
-				$icon = (new CButton(null, '&nbsp;'))
+				$icon = (new CButton(null, NBSP()))
 					->setTitle(_('Normal view'))
 					->setAttribute('data-layout-mode', ZBX_LAYOUT_NORMAL)
 					->addClass(ZBX_LAYOUT_MODE)
@@ -167,7 +199,7 @@ function get_icon($type, $params = []) {
 					->addClass(ZBX_STYLE_BTN_MIN);
 			}
 			else {
-				$icon = (new CButton(null, '&nbsp;'))
+				$icon = (new CButton(null, NBSP()))
 					->setTitle(_('Kiosk mode'))
 					->setAttribute('data-layout-mode', ZBX_LAYOUT_KIOSKMODE)
 					->addClass(ZBX_LAYOUT_MODE)
@@ -308,7 +340,7 @@ function getHostNavigation($current_element, $hostid, $lld_ruleid = 0) {
 		}
 
 		$host = new CSpan(
-			(new CLink(CHtml::encode($db_host['name']),
+			(new CLink($db_host['name'],
 				(new CUrl('zabbix.php'))
 					->setArgument('action', 'host.edit')
 					->setArgument('hostid', $db_host['hostid'])
@@ -437,11 +469,11 @@ function getHostNavigation($current_element, $hostid, $lld_ruleid = 0) {
 	else {
 		$discovery_rule = (new CSpan())->addItem(
 			new CLink(
-				CHtml::encode($db_discovery_rule['name']),
-					(new CUrl('host_discovery.php'))
-						->setArgument('form', 'update')
-						->setArgument('itemid', $db_discovery_rule['itemid'])
-						->setArgument('context', $context)
+				$db_discovery_rule['name'],
+				(new CUrl('host_discovery.php'))
+					->setArgument('form', 'update')
+					->setArgument('itemid', $db_discovery_rule['itemid'])
+					->setArgument('context', $context)
 			)
 		);
 
@@ -631,7 +663,7 @@ function getHostAvailabilityTable(array $host_interfaces): CHostAvailability {
  * @param string $current_time	current Unix timestamp
  * @param array  $ts_delete		deletion timestamp of the host group
  *
- * @return CDiv
+ * @return CLink
  */
 function getHostGroupLifetimeIndicator($current_time, $ts_delete) {
 	// Check if the element should've been deleted in the past.
@@ -658,7 +690,7 @@ function getHostGroupLifetimeIndicator($current_time, $ts_delete) {
  * @param string $current_time	current Unix timestamp
  * @param array  $ts_delete		deletion timestamp of the host
  *
- * @return CDiv
+ * @return CLink
  */
 function getHostLifetimeIndicator($current_time, $ts_delete) {
 	// Check if the element should've been deleted in the past.
@@ -685,7 +717,7 @@ function getHostLifetimeIndicator($current_time, $ts_delete) {
  * @param string $current_time	current Unix timestamp
  * @param array  $ts_delete		deletion timestamp of the graph
  *
- * @return CDiv
+ * @return CLink
  */
 function getGraphLifetimeIndicator($current_time, $ts_delete) {
 	// Check if the element should've been deleted in the past.
@@ -712,7 +744,7 @@ function getGraphLifetimeIndicator($current_time, $ts_delete) {
  * @param string $current_time	current Unix timestamp
  * @param array  $ts_delete		deletion timestamp of the trigger
  *
- * @return CDiv
+ * @return CLink
  */
 function getTriggerLifetimeIndicator($current_time, $ts_delete) {
 	// Check if the element should've been deleted in the past.
@@ -739,7 +771,7 @@ function getTriggerLifetimeIndicator($current_time, $ts_delete) {
  * @param string $current_time	current Unix timestamp
  * @param array  $ts_delete		deletion timestamp of the item
  *
- * @return CDiv
+ * @return CLink
  */
 function getItemLifetimeIndicator($current_time, $ts_delete) {
 	// Check if the element should've been deleted in the past.
@@ -806,11 +838,11 @@ function makePageFooter($with_version = true) {
 /**
  * Get drop-down submenu item list for the User settings section.
  *
- * @return array|null  Menu definition for CWidget::setTitleSubmenu.
+ * @return array  Menu definition for CHtmlPage::setTitleSubmenu.
  */
-function getUserSettingsSubmenu(): ?array {
+function getUserSettingsSubmenu(): array {
 	if (!CWebUser::checkAccess(CRoleHelper::ACTIONS_MANAGE_API_TOKENS)) {
-		return null;
+		return [];
 	}
 
 	$profile_url = (new CUrl('zabbix.php'))
@@ -834,7 +866,7 @@ function getUserSettingsSubmenu(): ?array {
 /**
  * Get drop-down submenu item list for the Administration->General section.
  *
- * @return array  Menu definition for CWidget::setTitleSubmenu.
+ * @return array  Menu definition for CHtmlPage::setTitleSubmenu.
  */
 function getAdministrationGeneralSubmenu() {
 	$gui_url = (new CUrl('zabbix.php'))
@@ -845,14 +877,6 @@ function getAdministrationGeneralSubmenu() {
 		->setArgument('action', 'autoreg.edit')
 		->getUrl();
 
-	$housekeeping_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'housekeeping.edit')
-		->getUrl();
-
-	$audit_settings_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'audit.settings.edit')
-		->getUrl();
-
 	$image_url = (new CUrl('zabbix.php'))
 		->setArgument('action', 'image.list')
 		->getUrl();
@@ -865,10 +889,6 @@ function getAdministrationGeneralSubmenu() {
 		->setArgument('action', 'regex.list')
 		->getUrl();
 
-	$macros_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'macros.edit')
-		->getUrl();
-
 	$trigdisplay_url = (new CUrl('zabbix.php'))
 		->setArgument('action', 'trigdisplay.edit')
 		->getUrl();
@@ -881,31 +901,26 @@ function getAdministrationGeneralSubmenu() {
 		->setArgument('action', 'module.list')
 		->getUrl();
 
-	$tokens_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'token.list')
+	$connectors_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'connector.list')
 		->getUrl();
 
 	$miscconfig_url = (new CUrl('zabbix.php'))
 		->setArgument('action', 'miscconfig.edit')
 		->getUrl();
 
-	$can_access_tokens = (!CWebUser::isGuest() && CWebUser::checkAccess(CRoleHelper::ACTIONS_MANAGE_API_TOKENS));
-
 	return [
 		'main_section' => [
 			'items' => array_filter([
 				$gui_url            => _('GUI'),
 				$autoreg_url        => _('Autoregistration'),
-				$housekeeping_url   => _('Housekeeping'),
-				$audit_settings_url => _('Audit log'),
 				$image_url          => _('Images'),
 				$iconmap_url        => _('Icon mapping'),
 				$regex_url          => _('Regular expressions'),
-				$macros_url         => _('Macros'),
 				$trigdisplay_url    => _('Trigger displaying options'),
 				$geomap_url			=> _('Geographical maps'),
 				$modules_url        => _('Modules'),
-				$tokens_url         => $can_access_tokens ? _('API tokens') : null,
+				$connectors_url     => _('Connectors'),
 				$miscconfig_url     => _('Other')
 			])
 		]
@@ -1023,7 +1038,6 @@ function makeSuppressedProblemIcon(array $icon_data, bool $blink = false) {
  * @return CTag  Returns CSpan or CButton depending on boolean $icon_data['button'] parameter
  */
 function makeActionIcon(array $icon_data): CTag {
-
 	if (array_key_exists('button', $icon_data) && $icon_data['button']) {
 		$icon = (new CButton(null))->addClass($icon_data['icon']);
 	}
@@ -1047,6 +1061,10 @@ function makeActionIcon(array $icon_data): CTag {
 		$icon->setTitle($icon_data['title']);
 	}
 
+	if (array_key_exists('style', $icon_data)) {
+		$icon->addStyle($icon_data['style']);
+	}
+
 	if (array_key_exists('aria-label', $icon_data)) {
 		$icon
 			->addItem($icon_data['aria-label'])
