@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,54 +21,68 @@
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
 $this->includeJsFile('configuration.graph.list.js.php');
 
 if (!empty($this->data['parent_discoveryid'])) {
-	$widget = (new CWidget())
+	$html_page = (new CHtmlPage())
 		->setTitle(_('Graph prototypes'))
+		->setDocUrl(CDocHelper::getUrl($data['context'] === 'host'
+			? CDocHelper::DATA_COLLECTION_HOST_GRAPH_PROTOTYPE_LIST
+			: CDocHelper::DATA_COLLECTION_TEMPLATES_GRAPH_PROTOTYPE_LIST
+		))
 		->setControls(
 			(new CTag('nav', true,
-				(new CList())->addItem(new CRedirectButton(_('Create graph prototype'),
-					(new CUrl('graphs.php'))
-						->setArgument('form', 'create')
-						->setArgument('parent_discoveryid', $data['parent_discoveryid'])
-						->setArgument('context', $data['context'])
-						->getUrl()
-				))
+				(new CList())
+					->addItem(
+						new CRedirectButton(_('Create graph prototype'),
+							(new CUrl('graphs.php'))
+								->setArgument('form', 'create')
+								->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+								->setArgument('context', $data['context'])
+						)
+					)
 			))->setAttribute('aria-label', _('Content controls'))
 		)
 		->setNavigation(getHostNavigation('graphs', $this->data['hostid'], $this->data['parent_discoveryid']));
 }
 else {
-	$widget = (new CWidget())
+	$html_page = (new CHtmlPage())
 		->setTitle(_('Graphs'))
+		->setDocUrl(CDocHelper::getUrl($data['context'] === 'host'
+			? CDocHelper::DATA_COLLECTION_HOST_GRAPH_LIST
+			: CDocHelper::DATA_COLLECTION_TEMPLATE_GRAPH_LIST
+		))
 		->setControls(
-			(new CTag('nav', true, ($data['hostid'] == 0)
-				? (new CButton('form',
-					($data['context'] === 'host')
-						? _('Create graph (select host first)')
-						: _('Create graph (select template first)')
-				))->setEnabled(false)
-				: new CRedirectButton(_('Create graph'), (new CUrl('graphs.php'))
-					->setArgument('hostid', $data['hostid'])
-					->setArgument('form', 'create')
-					->setArgument('context', $data['context'])
-					->getUrl()
-				)
-			))
-				->setAttribute('aria-label', _('Content controls'))
+			(new CTag('nav', true,
+				(new CList())
+					->addItem(
+						$data['hostid'] != 0
+							? new CRedirectButton(_('Create graph'),
+								(new CUrl('graphs.php'))
+									->setArgument('hostid', $data['hostid'])
+									->setArgument('form', 'create')
+									->setArgument('context', $data['context'])
+							)
+							: (new CButton('form',
+								$data['context'] === 'host'
+									? _('Create graph (select host first)')
+									: _('Create graph (select template first)')
+							))->setEnabled(false)
+					)
+			))->setAttribute('aria-label', _('Content controls'))
 		);
 
 	if (!empty($this->data['hostid'])) {
-		$widget->setNavigation(getHostNavigation('graphs', $this->data['hostid']));
+		$html_page->setNavigation(getHostNavigation('graphs', $this->data['hostid']));
 	}
 
 	// Add filter tab.
-	$hg_ms_params = ($data['context'] === 'host') ? ['real_hosts' => 1] : ['templated_hosts' => 1];
+	$hg_ms_params = $data['context'] === 'host' ? ['with_hosts' => true] : ['with_templates' => true];
 
-	$widget->addItem(
+	$html_page->addItem(
 		(new CFilter())
 			->setResetUrl((new CUrl('graphs.php'))->setArgument('context', $data['context']))
 			->setProfile($data['profileIdx'])
@@ -77,36 +91,38 @@ else {
 			->addFilterTab(_('Filter'), [
 				(new CFormList())
 					->addRow(
-						(new CLabel(_('Host groups'), 'filter_groups__ms')),
+						new CLabel($data['context'] === 'host' ? _('Host groups') : _('Template groups'),
+							'filter_groupids__ms'
+						),
 						(new CMultiSelect([
 							'name' => 'filter_groupids[]',
-							'object_name' => 'hostGroup',
+							'object_name' => $data['context'] === 'host' ? 'hostGroup' : 'templateGroup',
 							'data' => $data['filter']['groups'],
 							'popup' => [
 								'parameters' => [
-									'srctbl' => 'host_groups',
+									'srctbl' =>  $data['context'] === 'host' ? 'host_groups' : 'template_groups',
 									'srcfld1' => 'groupid',
 									'dstfrm' => 'zbx_filter',
 									'dstfld1' => 'filter_groupids_',
-									'with_hosts_and_templates' => 1,
-									'editable' => 1,
+									'editable' => true,
 									'enrich_parent_groups' => true
 								] + $hg_ms_params
 							]
 						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 					)
 					->addRow(
-						(new CLabel(($data['context'] === 'host') ? _('Hosts') : _('Templates'), 'filter_hosts__ms')),
+						(new CLabel(($data['context'] === 'host') ? _('Hosts') : _('Templates'), 'filter_hostids__ms')),
 						(new CMultiSelect([
 							'name' => 'filter_hostids[]',
-							'object_name' => ($data['context'] === 'host') ? 'hosts' : 'templates',
+							'object_name' => $data['context'] === 'host' ? 'hosts' : 'templates',
 							'data' => $data['filter']['hosts'],
 							'popup' => [
-								'filter_preselect_fields' => [
-									'hostgroups' => 'filter_groupids_'
+								'filter_preselect' => [
+									'id' => 'filter_groupids_',
+									'submit_as' => 'groupid'
 								],
 								'parameters' => [
-									'srctbl' => ($data['context'] === 'host') ? 'hosts' : 'templates',
+									'srctbl' => $data['context'] === 'host' ? 'hosts' : 'templates',
 									'srcfld1' => 'hostid',
 									'dstfrm' => 'zbx_filter',
 									'dstfld1' => 'filter_hostids_',
@@ -126,7 +142,8 @@ $url = (new CUrl('graphs.php'))
 // create form
 $graphForm = (new CForm('post', $url))
 	->setName('graphForm')
-	->addVar('hostid', $data['hostid']);
+	->addVar('hostid', $data['hostid'])
+	->addVar('context', $data['context'], 'form_context');
 
 if (!empty($this->data['parent_discoveryid'])) {
 	$graphForm->addVar('parent_discoveryid', $this->data['parent_discoveryid']);
@@ -157,6 +174,8 @@ $graphTable = (new CTableInfo())
 		$info_column
 	]);
 
+$csrf_token = CCsrfTokenHelper::get('graphs.php');
+
 foreach ($data['graphs'] as $graph) {
 	$graphid = $graph['graphid'];
 
@@ -178,7 +197,7 @@ foreach ($data['graphs'] as $graph) {
 	$name[] = makeGraphTemplatePrefix($graphid, $data['parent_templates'], $flag, $data['allowed_ui_conf_templates']);
 
 	if ($graph['discoveryRule'] && $data['parent_discoveryid'] === null) {
-		$name[] = (new CLink(CHtml::encode($graph['discoveryRule']['name']),
+		$name[] = (new CLink($graph['discoveryRule']['name'],
 			(new CUrl('host_discovery.php'))
 				->setArgument('form', 'update')
 				->setArgument('itemid', $graph['discoveryRule']['itemid'])
@@ -199,7 +218,7 @@ foreach ($data['graphs'] as $graph) {
 		$url->setArgument('filter_hostids', [$data['hostid']]);
 	}
 
-	$name[] = new CLink(CHtml::encode($graph['name']), $url);
+	$name[] = new CLink($graph['name'], $url);
 	$info_icons = [];
 	$discover = null;
 
@@ -214,7 +233,7 @@ foreach ($data['graphs'] as $graph) {
 					->setArgument('context', $data['context'])
 					->getUrl()
 			))
-				->addSID()
+				->addCsrfToken($csrf_token)
 				->addClass(ZBX_STYLE_LINK_ACTION)
 				->addClass($nodiscover ? ZBX_STYLE_RED : ZBX_STYLE_GREEN);
 	}
@@ -235,27 +254,39 @@ foreach ($data['graphs'] as $graph) {
 }
 
 // buttons
-$buttonsArray = [];
+$buttons = [];
 if (!$this->data['parent_discoveryid']) {
-	$buttonsArray['graph.masscopyto'] = ['name' => _('Copy')];
+	$buttons['graph.masscopyto'] = [
+		'content' => (new CSimpleButton(_('Copy')))
+			->addClass('js-copy')
+			->addClass(ZBX_STYLE_BTN_ALT)
+			->removeId()
+	];
 }
-$buttonsArray['graph.massdelete'] = ['name' => _('Delete'), 'confirm' => $this->data['parent_discoveryid']
+$buttons['graph.massdelete'] = ['name' => _('Delete'), 'confirm' => $this->data['parent_discoveryid']
 	? _('Delete selected graph prototypes?')
-	: _('Delete selected graphs?')
+	: _('Delete selected graphs?'),
+	'csrf_token' => $csrf_token
 ];
 
 // append table to form
 $graphForm->addItem([
 	$graphTable,
 	$data['paging'],
-	new CActionButtonList('action', 'group_graphid', $buttonsArray,
-		$data['parent_discoveryid']
-			? $data['parent_discoveryid']
-			: $data['hostid']
+	new CActionButtonList('action', 'group_graphid', $buttons,
+		$data['parent_discoveryid'] ?: $data['hostid']
 	)
 ]);
 
-// append form to widget
-$widget->addItem($graphForm);
+(new CScriptTag('
+	view.init('.json_encode([
+		'checkbox_hash' => $data['hostid'],
+		'checkbox_object' => 'group_graphid'
+	]).');
+'))
+	->setOnDocumentReady()
+	->show();
 
-$widget->show();
+$html_page
+	->addItem($graphForm)
+	->show();
