@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,28 +23,40 @@
  * @var CView $this
  */
 
-$widget = (new CWidget())
+require_once dirname(__FILE__).'/js/configuration.trigger.prototype.list.js.php';
+
+$html_page = (new CHtmlPage())
 	->setTitle(_('Trigger prototypes'))
+	->setDocUrl(CDocHelper::getUrl($data['context'] === 'host'
+		? CDocHelper::DATA_COLLECTION_HOST_TRIGGER_PROTOTYPE_LIST
+		: CDocHelper::DATA_COLLECTION_TEMPLATES_TRIGGER_PROTOTYPE_LIST
+	))
 	->setControls(
 		(new CTag('nav', true,
-			(new CList())->addItem(new CRedirectButton(_('Create trigger prototype'),
-				(new CUrl('trigger_prototypes.php'))
-					->setArgument('parent_discoveryid', $data['parent_discoveryid'])
-					->setArgument('form', 'create')
-			))
+			(new CList())
+				->addItem(
+					new CRedirectButton(_('Create trigger prototype'),
+						(new CUrl('trigger_prototypes.php'))
+							->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+							->setArgument('form', 'create')
+							->setArgument('context', $data['context'])
+					)
+				)
 		))->setAttribute('aria-label', _('Content controls'))
 	)
-	->addItem(get_header_host_table('triggers', $this->data['hostid'], $this->data['parent_discoveryid']));
-
-// create form
-$triggersForm = (new CForm())
-	->setName('triggersForm')
-	->addVar('parent_discoveryid', $this->data['parent_discoveryid']);
+	->setNavigation(getHostNavigation('triggers', $this->data['hostid'], $this->data['parent_discoveryid']));
 
 $url = (new CUrl('trigger_prototypes.php'))
 	->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+	->setArgument('context', $data['context'])
 	->getUrl();
 
+// create form
+$triggersForm = (new CForm('post', $url))
+	->setName('triggersForm')
+	->addVar('parent_discoveryid', $data['parent_discoveryid'], 'form_parent_discoveryid')
+	->addVar('context', $data['context'], 'form_context');
+
 // create table
 $triggersTable = (new CTableInfo())
 	->setHeader([
@@ -60,27 +72,31 @@ $triggersTable = (new CTableInfo())
 		_('Tags')
 	]);
 
-$this->data['triggers'] = CMacrosResolverHelper::resolveTriggerExpressions($this->data['triggers'], [
+$data['triggers'] = CMacrosResolverHelper::resolveTriggerExpressions($data['triggers'], [
 	'html' => true,
-	'sources' => ['expression', 'recovery_expression']
+	'sources' => ['expression', 'recovery_expression'],
+	'context' => $data['context']
 ]);
 
-foreach ($this->data['triggers'] as $trigger) {
+$csrf_token = CCsrfTokenHelper::get('trigger_prototypes.php');
+
+foreach ($data['triggers'] as $trigger) {
 	$triggerid = $trigger['triggerid'];
-	$trigger['discoveryRuleid'] = $this->data['parent_discoveryid'];
+	$trigger['discoveryRuleid'] = $data['parent_discoveryid'];
 
 	// description
 	$description = [];
 	$description[] = makeTriggerTemplatePrefix($trigger['triggerid'], $data['parent_templates'],
-		ZBX_FLAG_DISCOVERY_PROTOTYPE
+		ZBX_FLAG_DISCOVERY_PROTOTYPE, $data['allowed_ui_conf_templates']
 	);
 
 	$description[] = new CLink(
-		CHtml::encode($trigger['description']),
-		'trigger_prototypes.php?'.
-			'form=update'.
-			'&parent_discoveryid='.$this->data['parent_discoveryid'].
-			'&triggerid='.$triggerid
+		$trigger['description'],
+		(new CUrl('trigger_prototypes.php'))
+			->setArgument('form', 'update')
+			->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+			->setArgument('triggerid', $triggerid)
+			->setArgument('context', $data['context'])
 	);
 
 	if ($trigger['dependencies']) {
@@ -90,22 +106,27 @@ foreach ($this->data['triggers'] as $trigger) {
 		foreach ($trigger['dependencies'] as $dependency) {
 			$depTrigger = $data['dependencyTriggers'][$dependency['triggerid']];
 
-			$depTriggerDescription = CHtml::encode(
-				implode(', ', zbx_objectValues($depTrigger['hosts'], 'name')).NAME_DELIMITER.$depTrigger['description']
-			);
+			$depTriggerDescription =
+				implode(', ', zbx_objectValues($depTrigger['hosts'], 'name')).NAME_DELIMITER.$depTrigger['description'];
 
 			if ($depTrigger['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 				$triggerDependencies[] = (new CLink(
 					$depTriggerDescription,
-					'trigger_prototypes.php?form=update'.url_param('parent_discoveryid').
-						'&triggerid='.$depTrigger['triggerid']))
-					->addClass(triggerIndicatorStyle($depTrigger['status']));
+					(new CUrl('trigger_prototypes.php'))
+						->setArgument('form', 'update')
+						->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+						->setArgument('triggerid', $depTrigger['triggerid'])
+						->setArgument('context', $data['context'])
+				))->addClass(triggerIndicatorStyle($depTrigger['status']));
 			}
 			elseif ($depTrigger['flags'] == ZBX_FLAG_DISCOVERY_NORMAL) {
 				$triggerDependencies[] = (new CLink(
 					$depTriggerDescription,
-					'triggers.php?form=update&triggerid='.$depTrigger['triggerid']))
-					->addClass(triggerIndicatorStyle($depTrigger['status']));
+					(new CUrl('triggers.php'))
+						->setArgument('form', 'update')
+						->setArgument('triggerid', $depTrigger['triggerid'])
+						->setArgument('context', $data['context'])
+				))->addClass(triggerIndicatorStyle($depTrigger['status']));
 			}
 
 			$triggerDependencies[] = BR();
@@ -118,17 +139,19 @@ foreach ($this->data['triggers'] as $trigger) {
 	// status
 	$status = (new CLink(
 		($trigger['status'] == TRIGGER_STATUS_DISABLED) ? _('No') : _('Yes'),
-		'trigger_prototypes.php?'.
-			'action='.(($trigger['status'] == TRIGGER_STATUS_DISABLED)
+		(new CUrl('trigger_prototypes.php'))
+			->setArgument('action', ($trigger['status'] == TRIGGER_STATUS_DISABLED)
 				? 'triggerprototype.massenable'
 				: 'triggerprototype.massdisable'
-			).
-			'&g_triggerid='.$triggerid.
-			'&parent_discoveryid='.$this->data['parent_discoveryid']
+			)
+			->setArgument('g_triggerid[]', $triggerid)
+			->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+			->setArgument('context', $data['context'])
+			->getUrl()
 	))
+		->addCsrfToken($csrf_token)
 		->addClass(ZBX_STYLE_LINK_ACTION)
-		->addClass(triggerIndicatorStyle($trigger['status']))
-		->addSID();
+		->addClass(triggerIndicatorStyle($trigger['status']));
 
 
 	$nodiscover = ($trigger['discover'] == ZBX_PROTOTYPE_NO_DISCOVER);
@@ -136,14 +159,14 @@ foreach ($this->data['triggers'] as $trigger) {
 			(new CUrl('trigger_prototypes.php'))
 				->setArgument('g_triggerid[]', $triggerid)
 				->setArgument('parent_discoveryid', $data['parent_discoveryid'])
-				->setArgument('action', 'triggerprototype.massupdate')
-				->setArgument('visible[discover]', '1')
-				->setArgument('massupdate', 'discover')
-				->setArgument('discover', $nodiscover ? ZBX_PROTOTYPE_DISCOVER : ZBX_PROTOTYPE_NO_DISCOVER)
-				->setArgumentSID()
+				->setArgument('action',  $nodiscover
+					? 'triggerprototype.discover.enable'
+					: 'triggerprototype.discover.disable'
+				)
+				->setArgument('context', $data['context'])
 				->getUrl()
 		))
-			->addSID()
+			->addCsrfToken($csrf_token)
 			->addClass(ZBX_STYLE_LINK_ACTION)
 			->addClass($nodiscover ? ZBX_STYLE_RED : ZBX_STYLE_GREEN);
 
@@ -162,7 +185,7 @@ foreach ($this->data['triggers'] as $trigger) {
 
 	$triggersTable->addRow([
 		$checkBox,
-		getSeverityCell($trigger['priority'], $this->data['config']),
+		CSeverityHelper::makeSeverityCell((int) $trigger['priority']),
 		$description,
 		$trigger['opdata'],
 		(new CDiv($expression))->addClass(ZBX_STYLE_WORDWRAP),
@@ -175,25 +198,36 @@ foreach ($this->data['triggers'] as $trigger) {
 // append table to form
 $triggersForm->addItem([
 	$triggersTable,
-	$this->data['paging'],
+	$data['paging'],
 	new CActionButtonList('action', 'g_triggerid',
 		[
 			'triggerprototype.massenable' => ['name' => _('Create enabled'),
-				'confirm' => _('Create triggers from selected prototypes as enabled?')
+				'confirm' => _('Create triggers from selected prototypes as enabled?'), 'csrf_token' => $csrf_token
 			],
 			'triggerprototype.massdisable' => ['name' => _('Create disabled'),
-				'confirm' => _('Create triggers from selected prototypes as disabled?')
+				'confirm' => _('Create triggers from selected prototypes as disabled?'), 'csrf_token' => $csrf_token
+			],
+			'popup.massupdate.triggerprototype' => [
+				'content' => (new CButton('', _('Mass update')))
+					->onClick(
+						"openMassupdatePopup('popup.massupdate.triggerprototype', {".
+							CCsrfTokenHelper::CSRF_TOKEN_NAME.": '".CCsrfTokenHelper::get('triggerprototype').
+						"'}, {
+							dialogue_class: 'modal-popup-static',
+							trigger_element: this
+						});"
+					)
+					->addClass(ZBX_STYLE_BTN_ALT)
+					->removeAttribute('id')
 			],
-			'triggerprototype.massupdateform' => ['name' => _('Mass update')],
 			'triggerprototype.massdelete' => ['name' => _('Delete'),
-				'confirm' => _('Delete selected trigger prototypes?')
-			],
+				'confirm' => _('Delete selected trigger prototypes?'), 'csrf_token' => $csrf_token
+			]
 		],
 		$this->data['parent_discoveryid']
 	)
 ]);
 
-// append form to widget
-$widget->addItem($triggersForm);
-
-$widget->show();
+$html_page
+	->addItem($triggersForm)
+	->show();
