@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 	const view = {
 		init() {
 			$.subscribe("acknowledge.create", function(event, response, overlay) {
-				postMessageOk(response.message);
+				postMessageOk(response.success.title);
 				location.href = location.href;
 			});
 		},
