@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,12 +26,11 @@ $this->includeJsFile('monitoring.history.js.php');
 
 $web_layout_mode = CViewHelper::loadLayoutMode();
 
-$historyWidget = (new CWidget())->setWebLayoutMode($web_layout_mode);
+$html_page = (new CHtmlPage())->setWebLayoutMode($web_layout_mode);
 
 $header = [
 	'left' => _n('%1$s item', '%1$s items', count($data['items'])),
 	'right' => (new CForm('get'))
-		->cleanItems()
 		->setName('filter_view_as')
 		->addVar('itemids', $data['itemids'])
 ];
@@ -52,12 +51,8 @@ if ($data['items']) {
 }
 
 if ((count($data['items']) == 1 || $same_host) && $data['itemids']) {
-	$header['left'] = [
-		$host_name,
-		NAME_DELIMITER,
-		count($data['items']) == 1 ? $item['name'] : $header['left']
-	];
-	$header_row[] = implode('', $header['left']);
+	$header['left'] = $host_name.NAME_DELIMITER.(count($data['items']) == 1 ? $item['name'] : $header['left']);
+	$header_row[] = $header['left'];
 }
 else {
 	$header_row[] = $header['left'];
@@ -103,7 +98,7 @@ if ($data['action'] !== HISTORY_GRAPH && $data['action'] !== HISTORY_BATCH_GRAPH
 }
 
 if ($data['action'] == HISTORY_GRAPH && count($data['items']) == 1) {
-	$action_list->addItem(get_icon('favourite', [
+	$action_list->addItem(get_icon('favorite', [
 		'fav' => 'web.favorite.graphids',
 		'elid' => $item['itemid'],
 		'elname' => 'itemid'
@@ -222,7 +217,7 @@ if ($data['itemids']) {
 // append plaintext to widget
 if ($data['plaintext']) {
 	foreach ($header_row as $text) {
-		$historyWidget->addItem([new CSpan($text), BR()]);
+		$html_page->addItem([new CSpan($text), BR()]);
 	}
 
 	if ($data['itemids']) {
@@ -231,11 +226,11 @@ if ($data['plaintext']) {
 		foreach ($screen as $text) {
 			$pre->addItem([$text, BR()]);
 		}
-		$historyWidget->addItem($pre);
+		$html_page->addItem($pre);
 	}
 }
 else {
-	$historyWidget
+	$html_page
 		->setTitle($header['left'])
 		->setDocUrl(CDocHelper::getUrl(CDocHelper::MONITORING_HISTORY))
 		->setControls((new CTag('nav', true, $header['right']))->setAttribute('aria-label', _('Content controls')));
@@ -272,10 +267,10 @@ else {
 
 	if ($data['itemids']) {
 		if ($data['action'] !== HISTORY_LATEST) {
-			$historyWidget->addItem($filter_form);
+			$html_page->addItem($filter_form);
 		}
 
-		$historyWidget->addItem($screen->get());
+		$html_page->addItem($screen->get());
 
 		if ($data['action'] !== HISTORY_LATEST) {
 			CScreenBuilder::insertScreenStandardJs($screen->timeline);
@@ -283,10 +278,10 @@ else {
 	}
 	else {
 		if ($filter_tab) {
-			$historyWidget->addItem($filter_form);
+			$html_page->addItem($filter_form);
 		}
 
-		$historyWidget->addItem(
+		$html_page->addItem(
 			(new CTableInfo())
 				->setHeader([
 					(new CColHeader(_('Timestamp')))->addClass(ZBX_STYLE_CELL_WIDTH),
@@ -298,4 +293,4 @@ else {
 	}
 }
 
-$historyWidget->show();
+$html_page->show();
