@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * @var CView $this
  */
 
-$widget = (new CWidget())
+$html_page = (new CHtmlPage())
 	->setTitle(_('Maps'))
 	->setDocUrl(CDocHelper::getUrl(CDocHelper::MONITORING_SYSMAP_LIST))
 	->setControls(
@@ -31,7 +31,6 @@ $widget = (new CWidget())
 			(new CList())
 				->addItem(
 					(new CForm('get'))
-						->cleanItems()
 						->addItem(
 							(new CSubmit('form', _('Create map')))->setEnabled($data['allowed_edit'])
 						)
@@ -39,9 +38,14 @@ $widget = (new CWidget())
 				->addItem(
 					(new CButton('form', _('Import')))
 						->onClick(
-							'return PopUp("popup.import", {rules_preset: "map"},
-								{dialogue_class: "modal-popup-generic"}
-							);'
+							'return PopUp("popup.import", '.
+								json_encode([ 'rules_preset' => 'map',
+									CCsrfTokenHelper::CSRF_TOKEN_NAME => CCsrfTokenHelper::get('import')
+								]).
+						', {
+								dialogueid: "popup_import",
+								dialogue_class: "modal-popup-generic"
+							});'
 						)
 						->setEnabled($data['allowed_edit'])
 						->removeId()
@@ -118,12 +122,12 @@ $sysmapForm->addItem([
 			)
 		],
 		'map.massdelete' => ['name' => _('Delete'), 'confirm' => _('Delete selected maps?'),
-			'disabled' => $data['allowed_edit'] ? null : 'disabled'
+			'disabled' => $data['allowed_edit'] ? null : 'disabled',
+			'csrf_token' => CCsrfTokenHelper::get('sysmaps.php')
 		]
 	])
 ]);
 
-// append form to widget
-$widget->addItem($sysmapForm);
-
-$widget->show();
+$html_page
+	->addItem($sysmapForm)
+	->show();
