@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,18 +20,16 @@
 
 
 require_once dirname(__FILE__).'/include/classes/user/CWebUser.php';
-CWebUser::disableSessionCookie();
 
 require_once dirname(__FILE__).'/include/config.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
 
-$page['title'] = _('ZABBIX');
 $page['file'] = 'index.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
 	'name' =>		[T_ZBX_STR, O_NO,	null,	null,	'isset({enter}) && {enter} != "'.ZBX_GUEST_USER.'"', _('Username')],
-	'password' =>	[T_ZBX_STR, O_OPT, null,	null,	'isset({enter}) && {enter} != "'.ZBX_GUEST_USER.'"'],
+	'password' =>	[T_ZBX_STR, O_OPT, P_NO_TRIM,	null,	'isset({enter}) && {enter} != "'.ZBX_GUEST_USER.'"'],
 	'sessionid' =>	[T_ZBX_STR, O_OPT, null,	null,	null],
 	'reconnect' =>	[T_ZBX_INT, O_OPT, P_SYS,	null,	null],
 	'enter' =>		[T_ZBX_STR, O_OPT, P_SYS,	null,	null],
@@ -42,31 +40,36 @@ $fields = [
 check_fields($fields);
 
 if (hasRequest('reconnect') && CWebUser::isLoggedIn()) {
+	if (CAuthenticationHelper::get(CAuthenticationHelper::SAML_AUTH_ENABLED) == ZBX_AUTH_SAML_ENABLED) {
+		$provisioning = CProvisioning::forUserDirectoryId(CAuthenticationHelper::getSamlUserdirectoryid());
+		$saml_config = $provisioning->getIdpConfig();
+
+		if ($saml_config['slo_url'] !== '' && CSessionHelper::has('saml_data')) {
+			redirect('index_sso.php?slo');
+		}
+	}
+
 	CWebUser::logout();
+	redirect('index.php');
 }
 
-$config = select_config();
 $autologin = hasRequest('enter') ? getRequest('autologin', 0) : getRequest('autologin', 1);
 $request = getRequest('request', '');
 
-if ($request) {
-	$test_request = [];
-	preg_match('/^\/?(?<filename>[a-z0-9\_\.]+\.php)(?<request>\?.*)?$/i', $request, $test_request);
-
-	$request = (array_key_exists('filename', $test_request) && file_exists('./'.$test_request['filename']))
-		? $test_request['filename'].(array_key_exists('request', $test_request) ? $test_request['request'] : '')
-		: '';
+if ($request !== '' && !CHtmlUrlValidator::validateSameSite($request)) {
+	$request = '';
 }
 
-if (!hasRequest('form') && $config['http_auth_enabled'] == ZBX_AUTH_HTTP_ENABLED
-		&& $config['http_login_form'] == ZBX_AUTH_FORM_HTTP && !hasRequest('enter')) {
+if (!hasRequest('form') && CAuthenticationHelper::get(CAuthenticationHelper::HTTP_AUTH_ENABLED) == ZBX_AUTH_HTTP_ENABLED
+		&& CAuthenticationHelper::get(CAuthenticationHelper::HTTP_LOGIN_FORM) == ZBX_AUTH_FORM_HTTP
+		&& !hasRequest('enter')) {
 	redirect('index_http.php');
-
-	exit;
 }
 
 // login via form
 if (hasRequest('enter') && CWebUser::login(getRequest('name', ZBX_GUEST_USER), getRequest('password', ''))) {
+	CSessionHelper::set('sessionid', CWebUser::$data['sessionid']);
+
 	if (CWebUser::$data['autologin'] != $autologin) {
 		API::User()->update([
 			'userid' => CWebUser::$data['userid'],
@@ -74,23 +77,26 @@ if (hasRequest('enter') && CWebUser::login(getRequest('name', ZBX_GUEST_USER), g
 		]);
 	}
 
-	$redirect = array_filter([CWebUser::isGuest() ? '' : $request, CWebUser::$data['url'], ZBX_DEFAULT_URL]);
+	$redirect = array_filter([CWebUser::isGuest() ? '' : $request, CWebUser::$data['url'], CMenuHelper::getFirstUrl()]);
 	redirect(reset($redirect));
-
-	exit;
 }
 
 if (CWebUser::isLoggedIn() && !CWebUser::isGuest()) {
-	redirect(CWebUser::$data['url'] ? CWebUser::$data['url'] : ZBX_DEFAULT_URL);
+	redirect(CWebUser::$data['url'] ? : CMenuHelper::getFirstUrl());
 }
 
-$messages = clear_messages();
+$messages = get_and_clear_messages();
 
-(new CView('general.login', [
-	'http_login_url' => $config['http_auth_enabled'] == ZBX_AUTH_HTTP_ENABLED
+echo (new CView('general.login', [
+	'http_login_url' => (CAuthenticationHelper::get(CAuthenticationHelper::HTTP_AUTH_ENABLED) == ZBX_AUTH_HTTP_ENABLED)
 		? (new CUrl('index_http.php'))->setArgument('request', getRequest('request'))
 		: '',
+	'saml_login_url' => (CAuthenticationHelper::get(CAuthenticationHelper::SAML_AUTH_ENABLED) == ZBX_AUTH_SAML_ENABLED)
+		? (new CUrl('index_sso.php'))->setArgument('request', getRequest('request'))
+		: '',
 	'guest_login_url' => CWebUser::isGuestAllowed() ? (new CUrl())->setArgument('enter', ZBX_GUEST_USER) : '',
 	'autologin' => $autologin == 1,
-	'error' => hasRequest('enter') && $messages ? array_pop($messages) : null
-]))->render();
+	'error' => (hasRequest('enter') && $messages) ? array_pop($messages) : null
+]))->getOutput();
+
+session_write_close();
