@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,8 +17,96 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+/**
+ * WARNING: the class doesn't support parsing query strings with multi-dimensional arrays.
+ *
+ * @param url
+ */
+var Curl = function(url) {
+	url = url || location.href;
+
+	this.url = url;
+	this.args = {};
+
+	this.query = (this.url.indexOf('?') >= 0) ? this.url.substring(this.url.indexOf('?') + 1) : '';
+	if (this.query.indexOf('#') >= 0) {
+		this.query = this.query.substring(0, this.query.indexOf('#'));
+	}
+
+	var protocolSepIndex = this.url.indexOf('://');
+	if (protocolSepIndex >= 0) {
+		this.protocol = this.url.substring(0, protocolSepIndex).toLowerCase();
+		this.host = this.url.substring(protocolSepIndex + 3);
+
+		if (this.host.indexOf('/') >= 0) {
+			this.host = this.host.substring(0, this.host.indexOf('/'));
+		}
+
+		var atIndex = this.host.indexOf('@');
+		if (atIndex >= 0) {
+			var credentials = this.host.substring(0, atIndex);
+			var colonIndex = credentials.indexOf(':');
+
+			if (colonIndex >= 0) {
+				this.username = credentials.substring(0, colonIndex);
+				this.password = credentials.substring(colonIndex);
+			}
+			else {
+				this.username = credentials;
+			}
+			this.host = this.host.substring(atIndex + 1);
+		}
+
+		var host_ipv6 = this.host.indexOf(']');
+		if (host_ipv6 >= 0) {
+			if (host_ipv6 < (this.host.length - 1)) {
+				host_ipv6++;
+				var host_less = this.host.substring(host_ipv6);
+
+				var portColonIndex = host_less.indexOf(':');
+				if (portColonIndex >= 0) {
+					this.port = host_less.substring(portColonIndex + 1);
+					this.host = this.host.substring(0, host_ipv6);
+				}
+			}
+		}
+		else {
+			var portColonIndex = this.host.indexOf(':');
+			if (portColonIndex >= 0) {
+				this.port = this.host.substring(portColonIndex + 1);
+				this.host = this.host.substring(0, portColonIndex);
+			}
+		}
+		this.file = this.url.substring(protocolSepIndex + 3);
+		this.file = this.file.substring(this.file.indexOf('/'));
+
+		if (this.file == this.host) {
+			this.file = '';
+		}
+	}
+	else {
+		this.file = this.url;
+	}
+
+	if (this.file.indexOf('?') >= 0) {
+		this.file = this.file.substring(0, this.file.indexOf('?'));
+	}
+
+	var refSepIndex = this.file.indexOf('#');
+	if (refSepIndex >= 0) {
+		this.reference = this.file.substring(refSepIndex + 1);
+		this.file = this.file.substring(0, refSepIndex);
+	}
+
+	this.path = this.file;
+	if (this.query.length > 0) {
+		this.file += '?' + this.query;
+	}
+	if (this.query.length > 0) {
+		this.formatArguments();
+	}
+};
 
-var Curl = Class.create();
 Curl.prototype = {
 
 	url:		'', // actually, it's deprecated/private variable
@@ -33,120 +121,6 @@ Curl.prototype = {
 	query:		'',
 	args:		null,
 
-	/**
-	 * WARNING: the class doesn't support parsing query strings with multi-dimentional arrays.
-	 *
-	 * @param url
-	 * @param add_sid	boolean	Add SID to given URL. Default: true.
-	 */
-	initialize: function(url, add_sid) {
-		url = url || location.href;
-		if (typeof add_sid === 'undefined') {
-			add_sid = true;
-		}
-
-		this.url = url;
-		this.args = {};
-
-		this.query = (this.url.indexOf('?') >= 0) ? this.url.substring(this.url.indexOf('?') + 1) : '';
-		if (this.query.indexOf('#') >= 0) {
-			this.query = this.query.substring(0, this.query.indexOf('#'));
-		}
-
-		var protocolSepIndex = this.url.indexOf('://');
-		if (protocolSepIndex >= 0) {
-			this.protocol = this.url.substring(0, protocolSepIndex).toLowerCase();
-			this.host = this.url.substring(protocolSepIndex + 3);
-
-			if (this.host.indexOf('/') >= 0) {
-				this.host = this.host.substring(0, this.host.indexOf('/'));
-			}
-
-			var atIndex = this.host.indexOf('@');
-			if (atIndex >= 0) {
-				var credentials = this.host.substring(0, atIndex);
-				var colonIndex = credentials.indexOf(':');
-
-				if (colonIndex >= 0) {
-					this.username = credentials.substring(0, colonIndex);
-					this.password = credentials.substring(colonIndex);
-				}
-				else {
-					this.username = credentials;
-				}
-				this.host = this.host.substring(atIndex + 1);
-			}
-
-			var host_ipv6 = this.host.indexOf(']');
-			if (host_ipv6 >= 0) {
-				if (host_ipv6 < (this.host.length - 1)) {
-					host_ipv6++;
-					var host_less = this.host.substring(host_ipv6);
-
-					var portColonIndex = host_less.indexOf(':');
-					if (portColonIndex >= 0) {
-						this.port = host_less.substring(portColonIndex + 1);
-						this.host = this.host.substring(0, host_ipv6);
-					}
-				}
-			}
-			else {
-				var portColonIndex = this.host.indexOf(':');
-				if (portColonIndex >= 0) {
-					this.port = this.host.substring(portColonIndex + 1);
-					this.host = this.host.substring(0, portColonIndex);
-				}
-			}
-			this.file = this.url.substring(protocolSepIndex + 3);
-			this.file = this.file.substring(this.file.indexOf('/'));
-
-			if (this.file == this.host) {
-				this.file = '';
-			}
-		}
-		else {
-			this.file = this.url;
-		}
-
-		if (this.file.indexOf('?') >= 0) {
-			this.file = this.file.substring(0, this.file.indexOf('?'));
-		}
-
-		var refSepIndex = this.file.indexOf('#');
-		if (refSepIndex >= 0) {
-			this.reference = this.file.substring(refSepIndex + 1);
-			this.file = this.file.substring(0, refSepIndex);
-		}
-
-		this.path = this.file;
-		if (this.query.length > 0) {
-			this.file += '?' + this.query;
-		}
-		if (this.query.length > 0) {
-			this.formatArguments();
-		}
-
-		if (add_sid) {
-			this.addSID();
-		}
-	},
-
-	addSID: function() {
-		var sid = '';
-		var possition = parseInt(location.href.indexOf('sid='));
-
-		if (possition > -1) {
-			sid = location.href.substr(possition + 4, 16);
-		}
-		else {
-			sid = jQuery('meta[name="csrf-token"]').attr('content');
-		}
-
-		if ((/[\da-z]{16}/i).test(sid)) {
-			this.setArgument('sid', sid);
-		}
-	},
-
 	formatQuery: function() {
 		this.query = jQuery.param(this.args);
 	},
@@ -221,6 +195,48 @@ Curl.prototype = {
 		return this.args;
 	},
 
+	/**
+	 * Get query parameters as javascript object, support indexed names up to 3 levels only.
+	 */
+	getArgumentsObject: function() {
+		let result = {},
+			parts;
+
+		for (const name in this.args) {
+			parts = name.replace(/\]/g, '').split('[');
+
+			switch (parts.length) {
+				case 1:
+					result[name] = this.args[name];
+					break;
+
+				case 2:
+					// name[0]
+					if (!(parts[0] in result)) {
+						result[parts[0]] = [];
+					}
+
+					result[parts[0]][parts[1]] = this.args[name];
+					break;
+
+				case 3:
+					// name[0][property]
+					if (!(parts[0] in result)) {
+						result[parts[0]] = [];
+					}
+
+					if (!(parts[1] in result[parts[0]])) {
+						result[parts[0]][parts[1]] = {}
+					}
+
+					result[parts[0]][parts[1]][parts[2]] = this.args[name];
+					break;
+			}
+		}
+
+		return result;
+	},
+
 	getUrl: function() {
 		this.formatQuery();
 
