@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@ class CTabFilter extends CBaseComponent {
 		this._fetch = {};
 		this._idx_namespace = options.idx;
 		this._timeselector = null;
+		this._csrf_token = this._options.csrf_token;
 
 		this.init(options);
 		this.registerEvents();
@@ -56,13 +57,17 @@ class CTabFilter extends CBaseComponent {
 			options.data[options.selected].expanded = true;
 		}
 
-		for (const template of this._target.querySelectorAll('[type="text/x-jquery-tmpl"][data-template]')) {
+		for (const template of this._target.querySelectorAll('[data-template]')) {
 			this._templates[template.getAttribute('data-template')] = template;
-		};
+		}
 
 		for (const title of this._target.querySelectorAll('nav [data-target]')) {
 			item = this.create(title, options.data[index] || {});
 
+			if (index > 0) {
+				item.initUnsavedIndicator();
+			}
+
 			if (options.selected == index) {
 				item.renderContentTemplate();
 				this.setSelectedItem(item);
@@ -249,6 +254,7 @@ class CTabFilter extends CBaseComponent {
 		}
 
 		body.idx = this._idx_namespace + '.' + property;
+		body._csrf_token = this._csrf_token;
 
 		if (property !== 'properties') {
 			this._fetch[property] = new AbortController();
@@ -279,7 +285,6 @@ class CTabFilter extends CBaseComponent {
 
 			if (item) {
 				item.updateCounter(value);
-				return;
 			}
 		});
 
@@ -295,6 +300,7 @@ class CTabFilter extends CBaseComponent {
 	 */
 	setSelectedItem(item) {
 		this._active_item = item;
+		this._active_item.unsetExpandedSubfilters();
 		item.setSelected();
 
 		if (item !== this._timeselector) {
@@ -349,6 +355,22 @@ class CTabFilter extends CBaseComponent {
 		this._active_item.setBrowserLocationToApplyUrl();
 	}
 
+	/**
+	 * Set expanded subfilter name.
+	 */
+	setExpandedSubfilters(name) {
+		return this._active_item.setExpandedSubfilters(name);
+	}
+
+	/**
+	 * Retrieve expanded subfilter names.
+	 *
+	 * @returns {array}
+	 */
+	getExpandedSubfilters() {
+		return this._active_item.getExpandedSubfilters();
+	}
+
 	/**
 	 * Register tab filter events, called once during initialization.
 	 */
@@ -600,14 +622,14 @@ class CTabFilter extends CBaseComponent {
 			 * Action on 'Update' button press.
 			 */
 			buttonUpdateAction: () => {
-				var params = this._active_item.getFilterParams();
+				var params = this._active_item.getFilterParams(false);
 
 				this.profileUpdate('properties', {
 					idx2: this._active_item._index,
 					value_str: params.toString()
 				})
 				.then(() => {
-					this._active_item.updateApplyUrl();
+					this._active_item.updateApplyUrl(false);
 					this._active_item.setBrowserLocation(params);
 					this._active_item.resetUnsavedState();
 				});
@@ -628,9 +650,10 @@ class CTabFilter extends CBaseComponent {
 			 * Action on 'Apply' button press.
 			 */
 			buttonApplyAction: () => {
+				this._active_item.unsetExpandedSubfilters();
 				this._active_item.emptySubfilter();
-				this._active_item.updateUnsavedState();
-				this._active_item.updateApplyUrl();
+				this._active_item.updateUnsavedState(false);
+				this._active_item.updateApplyUrl(false);
 				this._active_item.setBrowserLocationToApplyUrl();
 			},
 
@@ -639,7 +662,7 @@ class CTabFilter extends CBaseComponent {
 			 */
 			buttonResetAction: () => {
 				let current_url = new Curl(),
-					url = new Curl('zabbix.php', false);
+					url = new Curl('zabbix.php');
 
 				url.setArgument('action', current_url.getArgument('action'));
 				url.setArgument('filter_reset', 1);
@@ -710,6 +733,8 @@ class CTabFilter extends CBaseComponent {
 			stop: (_, ui) => {
 				const $item = ui.item;
 
+				ui.item[0].classList.remove(TABFILTERITEM_STYLE_FOCUSED);
+
 				/**
 				 * Remove inline style position, left and top that stay after sortable.
 				 * This styles broken tabs layout.
@@ -723,7 +748,8 @@ class CTabFilter extends CBaseComponent {
 				}
 			},
 			axis: 'x',
-			containment: 'parent'
+			containment: 'parent',
+			helper : 'clone'
 		});
 
 		const container = this._target.querySelector('.ui-sortable-container').parentNode;
@@ -745,7 +771,14 @@ class CTabFilter extends CBaseComponent {
 		this._filters_footer.querySelector('[name="filter_new"]')
 			.addEventListener('click', this._events.buttonSaveAsAction);
 		this._filters_footer.querySelector('[name="filter_apply"]')
-			.addEventListener('click', this._events.buttonApplyAction);
+			.addEventListener('click', () => {
+				if (this._active_item._index == 0) {
+					this._events.buttonUpdateAction();
+				}
+				else {
+					this._events.buttonApplyAction();
+				}
+			});
 		this._filters_footer.querySelector('[name="filter_reset"]')
 			.addEventListener('click', this._events.buttonResetAction);
 		this._filters_footer.addEventListener('click', this._events.buttonActionNotify);
