@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 jQuery(function($) {
 	var $container = $('.filter-space').first(),
 		xhr = null,
-		endpoint = new Curl('zabbix.php', false),
+		endpoint = new Curl('zabbix.php'),
 		element = {
 			from: $container.find('[name=from]'),
 			to: $container.find('[name=to]'),
@@ -45,7 +45,6 @@ jQuery(function($) {
 		ui_disabled = false;
 
 	endpoint.setArgument('action', 'timeselector.update');
-	endpoint.addSID();
 	endpoint.setArgument('type', 11); // PAGE_TYPE_TEXT_RETURN_JSON
 
 	$.subscribe('timeselector.rangechange timeselector.decrement timeselector.increment timeselector.zoomout' +
@@ -298,8 +297,6 @@ jQuery(function($) {
 				url.unsetArgument('page');
 			}
 
-			url.unsetArgument('sid');
-
 			history.replaceState(history.state, '', url.getUrl());
 		}
 	}
@@ -325,7 +322,7 @@ jQuery(function($) {
 		.on('click', 'a', function(event) {
 			// Prevent click on graph image parent <a/> element when clicked inside graph selectable area.
 			if ($(event.target).is('img') && typeof $(event.target).data('zbx_sbox') !== 'undefined' && prevent_click
-					&& $(this).hasClass('dashbrd-widget-graph-link')) {
+					&& $(this).hasClass('dashboard-widget-graph-link')) {
 				return cancelEvent(event);
 			}
 		});
@@ -376,7 +373,7 @@ jQuery(function($) {
 		}
 
 		prevent_click = true;
-		noclick_area = $('<div/>')
+		noclick_area = $('<div>')
 			.css({
 				position: 'absolute',
 				top: 0,
@@ -389,7 +386,7 @@ jQuery(function($) {
 			.insertAfter(parent);
 
 		selection = {
-			dom: $('<div class="graph-selection"/>')
+			dom: $('<div>', {class: 'graph-selection'})
 				.css({
 					position: 'absolute',
 					top: data.top,
@@ -491,7 +488,9 @@ jQuery(function($) {
 		});
 	}
 
-	checkDisableTimeSelectorUI();
+	if (!$container.data('disable-initial-check')) {
+		checkDisableTimeSelectorUI();
+	}
 });
 
 /**
@@ -530,9 +529,7 @@ var timeControl = {
 			src: location.href,
 			dynamic: 1,
 			loadSBox: 0,
-			loadImage: 0,
-			mainObject: 0, // object on changing will reflect on all others
-			onDashboard: 0 // object is on dashboard
+			loadImage: 0
 		}, objData);
 
 		var _this = this;
@@ -576,7 +573,7 @@ var timeControl = {
 				if (isset('graphtype', obj.objDims)) {
 					// graph size might have changed regardless of graph's type
 
-					var graphUrl = new Curl(obj.src, false);
+					var graphUrl = new Curl(obj.src);
 					graphUrl.setArgument('width', Math.floor(obj.objDims.width));
 					graphUrl.setArgument('height', Math.floor(obj.objDims.graphHeight));
 
@@ -592,7 +589,6 @@ var timeControl = {
 						this.refreshImage(id);
 					}
 				}
-
 			}
 		}
 	},
@@ -610,15 +606,15 @@ var timeControl = {
 				from_ts: obj.timeline.from_ts,
 				to_ts: obj.timeline.to_ts
 			},
-			url = new Curl(obj.src, false);
+			url = new Curl(obj.src);
 
 		url.setArgument('_', (new Date()).getTime().toString(34));
 
 		if (img.length == 0) {
 			window.flickerfreeScreen.setElementProgressState(obj.id, true);
-			img = jQuery('<img/>', {id: id}).appendTo(('#'+obj.containerid)).on('load', function() {
+			img = jQuery('<img>', {id: id}).appendTo(('#'+obj.containerid)).on('load', function() {
 				window.flickerfreeScreen.setElementProgressState(obj.id, false);
-				img.closest('.dashbrd-grid-widget').trigger('load.image', {imageid: id});
+				img.closest('.dashboard-grid-widget').trigger('load.image', {imageid: id});
 			});
 
 			var xhr = (obj.loadSBox == 0)
@@ -640,7 +636,7 @@ var timeControl = {
 
 	refreshImage: function(id) {
 		var obj = this.objectList[id],
-			url = new Curl(obj.src, false),
+			url = new Curl(obj.src),
 			img = jQuery('#' + id),
 			zbx_sbox = img.data('zbx_sbox');
 
@@ -654,12 +650,12 @@ var timeControl = {
 		url.setArgument('to', obj.timeline.to);
 
 		var container = jQuery('#' + obj.containerid),
-			clone = jQuery('<img/>', {
+			clone = jQuery('<img>', {
 				id: img.attr('id'),
-				'class': img.attr('class')
+				class: img.attr('class')
 			})
 			.one('load', function() {
-				img.closest('.dashbrd-grid-widget').trigger('load.image', {imageid: img.attr('id')});
+				img.closest('.dashboard-grid-widget').trigger('load.image', {imageid: img.attr('id')});
 				img.replaceWith(clone);
 				window.flickerfreeScreen.setElementProgressState(obj.id, false);
 			});
@@ -688,7 +684,7 @@ var timeControl = {
 		}
 
 		// link
-		var graphUrl = new Curl(container.attr('href'), false);
+		var graphUrl = new Curl(container.attr('href'));
 		graphUrl.setArgument('width', obj.objDims.width);
 		graphUrl.setArgument('from', obj.timeline.from);
 		graphUrl.setArgument('to', obj.timeline.to);
@@ -705,7 +701,7 @@ var timeControl = {
 	disableAllSBox: function() {
 		jQuery.each(this.objectList, function(i, obj) {
 			if (obj.loadSBox == 1) {
-				jQuery('#'+obj.containerid).removeClass('dashbrd-widget-graph-link');
+				jQuery('#'+obj.containerid).removeClass('dashboard-widget-graph-link');
 			}
 		});
 		jQuery(document).off('dblclick mousedown', 'img');
@@ -718,7 +714,7 @@ var timeControl = {
 	 */
 	objectUpdate: function(data) {
 		if (timeControl.refreshPage) {
-			var url = new Curl(location.href, false);
+			var url = new Curl(location.href);
 			url.unsetArgument('output');
 
 			// Always reset "page" when reloading with updated time range.
