@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@ require_once dirname(__FILE__).'/include/config.inc.php';
 
 $page['title'] = _('Notification report');
 $page['file'] = 'report4.php';
+$page['scripts'] = ['report4.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -34,68 +35,57 @@ $fields = [
 ];
 check_fields($fields);
 
-if (getRequest('media_type')) {
-	$mediaTypeData = API::MediaType()->get([
-		'mediatypeids' => [$_REQUEST['media_type']],
+$media_type = getRequest('media_type', 0);
+
+if ($media_type != 0) {
+	$db_media_type = API::MediaType()->get([
+		'mediatypeids' => $media_type,
 		'countOutput' => true
 	]);
-	if (!$mediaTypeData) {
+
+	if (!$db_media_type) {
 		access_deny ();
 	}
 }
 
 $year = getRequest('year', intval(date('Y')));
 $period = getRequest('period', 'weekly');
-$media_type = getRequest('media_type', 0);
-
-$_REQUEST['year'] = $year;
-$_REQUEST['period'] = $period;
-$_REQUEST['media_type'] = $media_type;
-
-$currentYear = date('Y');
-
-// fetch media types
+$current_year = date('Y');
 $media_types = [];
+
 $db_media_types = API::MediaType()->get([
 	'output' => ['name'],
 	'preservekeys' => true
 ]);
-
 CArrayHelper::sort($db_media_types, ['name']);
-foreach ($db_media_types as $mediatypeid => $db_media_type) {
-	$media_types[$mediatypeid] = $db_media_type['name'];
-}
 
-$widget = (new CWidget())->setTitle(_('Notifications'));
+$media_types = array_column($db_media_types, 'name', 'mediatypeid');
 
-// if no media types were defined, we have nothing to show
-if (zbx_empty($media_types)) {
-	$table = new CTableInfo();
-	$widget->addItem($table)->show();
-}
-else {
+$html_page = (new CHtmlPage())
+	->setTitle(_('Notifications'))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::REPORT4));
+
+if ($media_types) {
 	$table = (new CTableInfo())->makeVerticalRotation();
 
-	// fetch the year of the first alert
-	if (($firstAlert = DBfetch(DBselect('SELECT MIN(a.clock) AS clock FROM alerts a'))) && $firstAlert['clock']) {
-		$minYear = date('Y', $firstAlert['clock']);
+	// Fetch the year of the first alert.
+	if (($first_alert = DBfetch(DBselect('SELECT MIN(a.clock) AS clock FROM alerts a'))) && $first_alert['clock']) {
+		$min_year = date('Y', $first_alert['clock']);
 	}
-	// if no alerts exist, use the current year
+	// If no alerts exist, use the current year.
 	else {
-		$minYear = date('Y');
+		$min_year = date('Y');
 	}
 
 	$select_media_type = (new CSelect('media_type'))
 		->setValue($media_type)
 		->setFocusableElementId('media-type')
-		->onChange('$(this).closest("form").submit()')
 		->addOption(new CSelectOption(0, _('all')))
 		->addOptions(CSelect::createOptionsFromArray($media_types));
 
 	$select_period = (new CSelect('period'))
 		->setValue($period)
 		->setFocusableElementId('period')
-		->onChange('$(this).closest("form").submit()')
 		->addOptions(CSelect::createOptionsFromArray([
 			'daily' => _('Daily'),
 			'weekly' => _('Weekly'),
@@ -115,35 +105,34 @@ else {
 			$select_period
 		]);
 
-	if ($period != 'yearly') {
-		$cmb_year = (new CSelect('year'))
+	if ($period !== 'yearly') {
+		$year_select = (new CSelect('year'))
 			->setValue($year)
-			->setFocusableElementId('year')
-			->onChange('$(this).closest("form").submit()');
+			->setFocusableElementId('year');
 
-		for ($y = $minYear; $y <= date('Y'); $y++) {
-			$cmb_year->addOption(new CSelectOption($y, $y));
+		for ($y = $min_year; $y <= date('Y'); $y++) {
+			$year_select->addOption(new CSelectOption($y, $y));
 		}
 
 		$controls->addItem([
-			new CLabel(_('Year'), $cmb_year->getFocusableElementId()),
+			new CLabel(_('Year'), $year_select->getFocusableElementId()),
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-			$cmb_year
+			$year_select
 		]);
 	}
 
-	$widget->setControls((new CForm('get'))
-		->cleanItems()
+	$html_page->setControls((new CForm('get'))
 		->setAttribute('aria-label', _('Main filter'))
 		->addItem($controls)
+		->setName('report4')
 	);
 
 	$header = [];
 	$users = [];
 
 	$db_users = API::User()->get([
-		'output' => ['userid', 'alias', 'name', 'surname'],
-		'sortfield' => 'alias'
+		'output' => ['userid', 'username', 'name', 'surname'],
+		'sortfield' => 'username'
 	]);
 
 	foreach ($db_users as $user_data) {
@@ -158,37 +147,35 @@ else {
 
 	switch ($period) {
 		case 'yearly':
-			$minTime = mktime(0, 0, 0, 1, 1, $minYear);
+			$min_time = mktime(0, 0, 0, 1, 1, $min_year);
 
-			$dateFormat = _x('Y', DATE_FORMAT_CONTEXT);
+			$date_format = _x('Y', DATE_FORMAT_CONTEXT);
 			array_unshift($header, _('Year'));
 
-			for ($i = $minYear; $i <= date('Y'); $i++) {
+			for ($i = $min_year; $i <= date('Y'); $i++) {
 				$intervals[mktime(0, 0, 0, 1, 1, $i)] = mktime(0, 0, 0, 1, 1, $i + 1);
 			}
-
 			break;
 
 		case 'monthly':
-			$minTime = mktime(0, 0, 0, 1, 1, $year);
+			$min_time = mktime(0, 0, 0, 1, 1, $year);
 
-			$dateFormat = _x('F', DATE_FORMAT_CONTEXT);
+			$date_format = _x('F', DATE_FORMAT_CONTEXT);
 			array_unshift($header, _('Month'));
 
-			$max = ($year == $currentYear) ? date('n') : 12;
+			$max = ($year == $current_year) ? date('n') : 12;
 			for ($i = 1; $i <= $max; $i++) {
 				$intervals[mktime(0, 0, 0, $i, 1, $year)] = mktime(0, 0, 0, $i + 1, 1, $year);
 			}
-
 			break;
 
 		case 'daily':
-			$minTime = mktime(0, 0, 0, 1, 1, $year);
+			$min_time = mktime(0, 0, 0, 1, 1, $year);
 
-			$dateFormat = DATE_FORMAT;
+			$date_format = DATE_FORMAT;
 			array_unshift($header, _('Day'));
 
-			$max = ($year == $currentYear) ? date('z') + 1 : date('z', mktime(0, 0, 0, 12, 31, $year)) + 1;
+			$max = ($year == $current_year) ? date('z') + 1 : date('z', mktime(0, 0, 0, 12, 31, $year)) + 1;
 			for ($i = 1; $i <= $max; $i++) {
 				$intervals[mktime(0, 0, 0, 1, $i, $year)] = mktime(0, 0, 0, 1, $i + 1, $year);
 			}
@@ -199,114 +186,79 @@ else {
 			$time = mktime(0, 0, 0, 1, 1, $year);
 			$wd = date('w', $time);
 			$wd = ($wd == 0) ? 6 : $wd - 1;
-			$minTime = $time - $wd * SEC_PER_DAY;
+			$min_time = $time - $wd * SEC_PER_DAY;
 
-			$dateFormat = DATE_TIME_FORMAT;
+			$date_format = DATE_TIME_FORMAT;
 			array_unshift($header, _('From'), _('Till'));
 
-			$max = ($year == $currentYear) ? date('W') - 1 : 52;
+			$max = ($year == $current_year) ? date('W') - 1 : 52;
 			for ($i = 0; $i <= $max; $i++) {
-				$intervals[strtotime('+'.$i.' week', $minTime)] = strtotime('+'.($i + 1).' week', $minTime);
+				$intervals[strtotime('+'.$i.' week', $min_time)] = strtotime('+'.($i + 1).' week', $min_time);
 			}
-
 			break;
 	}
 
-	// time till
-	$maxTime = ($year == $currentYear || $period === 'yearly') ? time() : mktime(0, 0, 0, 1, 1, $year + 1);
+	// Time till.
+	$max_time = ($year == $current_year || $period === 'yearly') ? time() : mktime(0, 0, 0, 1, 1, $year + 1);
 
-	// fetch alerts
+	// Fetch alerts.
 	$alerts = [];
-	foreach (eventSourceObjects() as $sourceObject) {
+	foreach (eventSourceObjects() as $source_object) {
 		$alerts = array_merge($alerts, API::Alert()->get([
 			'output' => ['mediatypeid', 'userid', 'clock'],
-			'eventsource' => $sourceObject['source'],
-			'eventobject' => $sourceObject['object'],
-			'mediatypeids' => (getRequest('media_type')) ? getRequest('media_type') : null,
-			'time_from' => $minTime,
-			'time_till' => $maxTime
+			'eventsource' => $source_object['source'],
+			'eventobject' => $source_object['object'],
+			'mediatypeids' => ($media_type != 0) ? $media_type : null,
+			'time_from' => $min_time,
+			'time_till' => $max_time
 		]));
 	}
-	// sort alerts in chronological order so we could easily iterate through them later
+	// Sort alerts in chronological order so we could easily iterate through them later.
 	CArrayHelper::sort($alerts, ['clock']);
 
 	$table->setHeader($header);
 
-	if ($media_type > 0) {
-		$media_types = [$media_type => $media_types[$media_type]];
-	}
-
 	foreach ($intervals as $from => $till) {
-		// interval start
-		$row = [zbx_date2str($dateFormat, $from)];
+		// Interval start.
+		$row = [zbx_date2str($date_format, $from)];
 
-		// interval end, displayed only for week intervals
-		if ($period == 'weekly') {
-			$row[] = zbx_date2str($dateFormat, min($till, time()));
+		// Interval end, displayed only for week intervals.
+		if ($period === 'weekly') {
+			$row[] = zbx_date2str($date_format, min($till, time()));
 		}
 
-		// counting alert count for each user and media type
-		$summary = [];
+		$notifications = [];
 		foreach ($users as $userid) {
-			$summary[$userid] = [];
-			$summary[$userid]['total'] = 0;
-			$summary[$userid]['medias'] = [];
-			foreach ($media_types as $media_type_nr => $mt) {
-				$summary[$userid]['medias'][$media_type_nr] = 0;
-			}
+			$notifications[$userid] = 0;
 		}
 
-		// loop through alerts until we reach an alert from the next interval
+		// Loop through alerts until we reach an alert from the next interval.
 		while ($alert = current($alerts)) {
 			if ($alert['clock'] >= $till) {
 				break;
 			}
 
-			if (isset($summary[$alert['userid']])) {
-				$summary[$alert['userid']]['total']++;
-				if (isset($summary[$alert['userid']]['medias'][$alert['mediatypeid']])) {
-					$summary[$alert['userid']]['medias'][$alert['mediatypeid']]++;
-				}
-				else {
-					$summary[$alert['userid']]['medias'][$alert['mediatypeid']] = 1;
-				}
+			if (array_key_exists($alert['userid'], $notifications)) {
+				$notifications[$alert['userid']]++;
 			}
 
 			next($alerts);
 		}
 
-		foreach ($summary as $s) {
-			if ($s['total'] == 0) {
-				array_push($row, '');
-			}
-			else {
-				array_push($row, [$s['total'], ($media_type == 0) ? SPACE.'('.implode('/', $s['medias']).')' : '']);
-			}
+		foreach ($notifications as $notification_count) {
+			$row[] = ($notification_count == 0) ? '' : [$notification_count];
 		}
 
 		$table->addRow($row);
 	}
-
-	$widget->addItem($table);
-
-	if ($media_type == 0) {
-		$links = [];
-		foreach ($media_types as $id => $name) {
-			$links[] = (CWebUser::getType() < USER_TYPE_SUPER_ADMIN)
-				? $name
-				: new CLink($name, 'zabbix.php?action=mediatype.edit&mediatypeid='.$id);
-			$links[] = SPACE.'/'.SPACE;
-		}
-		array_pop($links);
-
-		$linksDiv = new CDiv([SPACE._('all').SPACE.'('.SPACE, $links, SPACE.')']);
-
-		$widget
-			->addItem(new CTag('br'))
-			->addItem($linksDiv);
-	}
-
-	$widget->show();
+}
+else {
+	// If no media types were defined, there is nothing to show.
+	$table = new CTableInfo();
 }
 
+$html_page
+	->addItem($table)
+	->show();
+
 require_once dirname(__FILE__).'/include/page_footer.php';
