@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -69,8 +69,8 @@ $fields = [
 	'retry' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,				null],
 	'cancel' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,				null],
 	'finish' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,				null],
-	'next' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,				null],
-	'back' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,				null]
+	'next' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ONLY_ARRAY,	null,	null],
+	'back' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ONLY_ARRAY,	null,	null]
 ];
 
 $check_fields_result = check_fields($fields, false);
@@ -160,8 +160,11 @@ DBclose();
 $setup_wizard = new CSetupWizard();
 
 // page title
-(new CPageHeader(_('Installation'), substr($default_lang, 0, strpos($default_lang, '_'))))
-	->addCssFile('assets/styles/'.CHtml::encode($default_theme).'.css')
+$page_header = (new CHtmlPageHeader(_('Installation'), substr($default_lang, 0, strpos($default_lang, '_'))));
+
+$page_header
+	->setTheme($default_theme)
+	->addCssFile('assets/styles/'.$page_header->getTheme().'.css')
 	->addJsFile((new CUrl('js/browsers.js'))->getUrl())
 	->addJsFile((new CUrl('jsLoader.php'))
 		->setArgument('ver', ZABBIX_VERSION)
@@ -174,7 +177,7 @@ $setup_wizard = new CSetupWizard();
 		->setArgument('files', ['setup.js'])
 		->getUrl()
 	)
-	->display();
+	->show();
 
 /*
  * Displaying
